"""
***************************************************************************
    g_extension_manage.py
    ------------------
    Date                 : May 2023
    Copyright            : (C) 2023 by Alister Hood
    Email                : alister.hood at gmail dot com
***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************
"""

__author__ = "Alister Hood"
__date__ = "May 2023"
__copyright__ = "(C) 2023, Alister Hood"


def processInputs(alg, parameters, context, feedback):
    pass
