/********************************************************************************
** Form generated from reading UI file 'qgsprocessingreliefcolorswidgetbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QGSPROCESSINGRELIEFCOLORSWIDGETBASE_H
#define UI_QGSPROCESSINGRELIEFCOLORSWIDGETBASE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include "qgspanelwidget.h"

QT_BEGIN_NAMESPACE

class Ui_QgsProcessingReliefColorsWidgetBase
{
public:
    QGridLayout *gridLayout;
    QTreeWidget *mReliefColorsWidget;
    QVBoxLayout *verticalLayout;
    QToolButton *mButtonAdd;
    QToolButton *mButtonRemove;
    QToolButton *mButtonUp;
    QToolButton *mButtonDown;
    QSpacerItem *verticalSpacer;
    QToolButton *mButtonLoad;
    QToolButton *mButtonSave;
    QToolButton *mButtonAuto;

    void setupUi(QgsPanelWidget *QgsProcessingReliefColorsWidgetBase)
    {
        if (QgsProcessingReliefColorsWidgetBase->objectName().isEmpty())
            QgsProcessingReliefColorsWidgetBase->setObjectName("QgsProcessingReliefColorsWidgetBase");
        QgsProcessingReliefColorsWidgetBase->resize(505, 237);
        QgsProcessingReliefColorsWidgetBase->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(QgsProcessingReliefColorsWidgetBase);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        mReliefColorsWidget = new QTreeWidget(QgsProcessingReliefColorsWidgetBase);
        mReliefColorsWidget->setObjectName("mReliefColorsWidget");

        gridLayout->addWidget(mReliefColorsWidget, 0, 0, 1, 1);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        mButtonAdd = new QToolButton(QgsProcessingReliefColorsWidgetBase);
        mButtonAdd->setObjectName("mButtonAdd");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/images/themes/default/symbologyAdd.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mButtonAdd->setIcon(icon);

        verticalLayout->addWidget(mButtonAdd);

        mButtonRemove = new QToolButton(QgsProcessingReliefColorsWidgetBase);
        mButtonRemove->setObjectName("mButtonRemove");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/images/themes/default/symbologyRemove.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mButtonRemove->setIcon(icon1);

        verticalLayout->addWidget(mButtonRemove);

        mButtonUp = new QToolButton(QgsProcessingReliefColorsWidgetBase);
        mButtonUp->setObjectName("mButtonUp");
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/images/themes/default/mActionArrowUp.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mButtonUp->setIcon(icon2);

        verticalLayout->addWidget(mButtonUp);

        mButtonDown = new QToolButton(QgsProcessingReliefColorsWidgetBase);
        mButtonDown->setObjectName("mButtonDown");
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/images/themes/default/mActionArrowDown.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mButtonDown->setIcon(icon3);

        verticalLayout->addWidget(mButtonDown);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        mButtonLoad = new QToolButton(QgsProcessingReliefColorsWidgetBase);
        mButtonLoad->setObjectName("mButtonLoad");
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/images/themes/default/mActionFileOpen.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mButtonLoad->setIcon(icon4);

        verticalLayout->addWidget(mButtonLoad);

        mButtonSave = new QToolButton(QgsProcessingReliefColorsWidgetBase);
        mButtonSave->setObjectName("mButtonSave");
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/images/themes/default/mActionFileSave.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mButtonSave->setIcon(icon5);

        verticalLayout->addWidget(mButtonSave);

        mButtonAuto = new QToolButton(QgsProcessingReliefColorsWidgetBase);
        mButtonAuto->setObjectName("mButtonAuto");
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/images/themes/default/mActionReload.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mButtonAuto->setIcon(icon6);

        verticalLayout->addWidget(mButtonAuto);


        gridLayout->addLayout(verticalLayout, 0, 1, 1, 1);


        retranslateUi(QgsProcessingReliefColorsWidgetBase);

        QMetaObject::connectSlotsByName(QgsProcessingReliefColorsWidgetBase);
    } // setupUi

    void retranslateUi(QgsPanelWidget *QgsProcessingReliefColorsWidgetBase)
    {
        QTreeWidgetItem *___qtreewidgetitem = mReliefColorsWidget->headerItem();
        ___qtreewidgetitem->setText(2, QCoreApplication::translate("QgsProcessingReliefColorsWidgetBase", "Color", nullptr));
        ___qtreewidgetitem->setText(1, QCoreApplication::translate("QgsProcessingReliefColorsWidgetBase", "Upper bound", nullptr));
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("QgsProcessingReliefColorsWidgetBase", "Lower bound", nullptr));
#if QT_CONFIG(tooltip)
        mButtonAdd->setToolTip(QCoreApplication::translate("QgsProcessingReliefColorsWidgetBase", "Add row", nullptr));
#endif // QT_CONFIG(tooltip)
        mButtonAdd->setText(QCoreApplication::translate("QgsProcessingReliefColorsWidgetBase", "\342\200\246", nullptr));
#if QT_CONFIG(tooltip)
        mButtonRemove->setToolTip(QCoreApplication::translate("QgsProcessingReliefColorsWidgetBase", "Remove row", nullptr));
#endif // QT_CONFIG(tooltip)
        mButtonRemove->setText(QCoreApplication::translate("QgsProcessingReliefColorsWidgetBase", "\342\200\246", nullptr));
#if QT_CONFIG(tooltip)
        mButtonUp->setToolTip(QCoreApplication::translate("QgsProcessingReliefColorsWidgetBase", "Move up", nullptr));
#endif // QT_CONFIG(tooltip)
        mButtonUp->setText(QCoreApplication::translate("QgsProcessingReliefColorsWidgetBase", "\342\200\246", nullptr));
#if QT_CONFIG(tooltip)
        mButtonDown->setToolTip(QCoreApplication::translate("QgsProcessingReliefColorsWidgetBase", "Move down", nullptr));
#endif // QT_CONFIG(tooltip)
        mButtonDown->setText(QCoreApplication::translate("QgsProcessingReliefColorsWidgetBase", "\342\200\246", nullptr));
#if QT_CONFIG(tooltip)
        mButtonLoad->setToolTip(QCoreApplication::translate("QgsProcessingReliefColorsWidgetBase", "Load colors from file", nullptr));
#endif // QT_CONFIG(tooltip)
        mButtonLoad->setText(QCoreApplication::translate("QgsProcessingReliefColorsWidgetBase", "\342\200\246", nullptr));
#if QT_CONFIG(tooltip)
        mButtonSave->setToolTip(QCoreApplication::translate("QgsProcessingReliefColorsWidgetBase", "Save colors to file", nullptr));
#endif // QT_CONFIG(tooltip)
        mButtonSave->setText(QCoreApplication::translate("QgsProcessingReliefColorsWidgetBase", "\342\200\246", nullptr));
#if QT_CONFIG(tooltip)
        mButtonAuto->setToolTip(QCoreApplication::translate("QgsProcessingReliefColorsWidgetBase", "Generate color table automatically", nullptr));
#endif // QT_CONFIG(tooltip)
        mButtonAuto->setText(QCoreApplication::translate("QgsProcessingReliefColorsWidgetBase", "\342\200\246", nullptr));
        (void)QgsProcessingReliefColorsWidgetBase;
    } // retranslateUi

};

namespace Ui {
    class QgsProcessingReliefColorsWidgetBase: public Ui_QgsProcessingReliefColorsWidgetBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QGSPROCESSINGRELIEFCOLORSWIDGETBASE_H
