/********************************************************************************
** Form generated from reading UI file 'qgsformannotationdialogbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QGSFORMANNOTATIONDIALOGBASE_H
#define UI_QGSFORMANNOTATIONDIALOGBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QWidget>
#include "qgscodeeditorhtml.h"

QT_BEGIN_NAMESPACE

class Ui_QgsFormAnnotationDialogBase
{
public:
    QGridLayout *gridLayout;
    QStackedWidget *mStackedWidget;
    QWidget *page;
    QWidget *page_2;
    QRadioButton *mSourceRadioButton;
    QHBoxLayout *horizontalLayout;
    QRadioButton *mFileRadioButton;
    QLineEdit *mFileLineEdit;
    QToolButton *mBrowseToolButton;
    QDialogButtonBox *mButtonBox;
    QgsCodeEditorHTML *mHtmlSourceTextEdit;
    QCheckBox *mLiveCheckBox;

    void setupUi(QDialog *QgsFormAnnotationDialogBase)
    {
        if (QgsFormAnnotationDialogBase->objectName().isEmpty())
            QgsFormAnnotationDialogBase->setObjectName("QgsFormAnnotationDialogBase");
        QgsFormAnnotationDialogBase->resize(433, 246);
        gridLayout = new QGridLayout(QgsFormAnnotationDialogBase);
        gridLayout->setObjectName("gridLayout");
        mStackedWidget = new QStackedWidget(QgsFormAnnotationDialogBase);
        mStackedWidget->setObjectName("mStackedWidget");
        page = new QWidget();
        page->setObjectName("page");
        mStackedWidget->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        mStackedWidget->addWidget(page_2);

        gridLayout->addWidget(mStackedWidget, 3, 0, 1, 1);

        mSourceRadioButton = new QRadioButton(QgsFormAnnotationDialogBase);
        mSourceRadioButton->setObjectName("mSourceRadioButton");

        gridLayout->addWidget(mSourceRadioButton, 1, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        mFileRadioButton = new QRadioButton(QgsFormAnnotationDialogBase);
        mFileRadioButton->setObjectName("mFileRadioButton");
        mFileRadioButton->setChecked(true);

        horizontalLayout->addWidget(mFileRadioButton);

        mFileLineEdit = new QLineEdit(QgsFormAnnotationDialogBase);
        mFileLineEdit->setObjectName("mFileLineEdit");

        horizontalLayout->addWidget(mFileLineEdit);

        mBrowseToolButton = new QToolButton(QgsFormAnnotationDialogBase);
        mBrowseToolButton->setObjectName("mBrowseToolButton");

        horizontalLayout->addWidget(mBrowseToolButton);


        gridLayout->addLayout(horizontalLayout, 0, 0, 1, 1);

        mButtonBox = new QDialogButtonBox(QgsFormAnnotationDialogBase);
        mButtonBox->setObjectName("mButtonBox");
        mButtonBox->setOrientation(Qt::Horizontal);
        mButtonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Cancel|QDialogButtonBox::Help|QDialogButtonBox::Ok);

        gridLayout->addWidget(mButtonBox, 5, 0, 1, 1);

        mHtmlSourceTextEdit = new QgsCodeEditorHTML(QgsFormAnnotationDialogBase);
        mHtmlSourceTextEdit->setObjectName("mHtmlSourceTextEdit");

        gridLayout->addWidget(mHtmlSourceTextEdit, 2, 0, 1, 1);

        mLiveCheckBox = new QCheckBox(QgsFormAnnotationDialogBase);
        mLiveCheckBox->setObjectName("mLiveCheckBox");

        gridLayout->addWidget(mLiveCheckBox, 4, 0, 1, 1);


        retranslateUi(QgsFormAnnotationDialogBase);
        QObject::connect(mButtonBox, &QDialogButtonBox::accepted, QgsFormAnnotationDialogBase, qOverload<>(&QDialog::accept));
        QObject::connect(mButtonBox, &QDialogButtonBox::rejected, QgsFormAnnotationDialogBase, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(QgsFormAnnotationDialogBase);
    } // setupUi

    void retranslateUi(QDialog *QgsFormAnnotationDialogBase)
    {
        QgsFormAnnotationDialogBase->setWindowTitle(QCoreApplication::translate("QgsFormAnnotationDialogBase", "Form Annotation", nullptr));
        mSourceRadioButton->setText(QCoreApplication::translate("QgsFormAnnotationDialogBase", "Source", nullptr));
        mFileRadioButton->setText(QCoreApplication::translate("QgsFormAnnotationDialogBase", "File", nullptr));
        mBrowseToolButton->setText(QCoreApplication::translate("QgsFormAnnotationDialogBase", "\342\200\246", nullptr));
        mLiveCheckBox->setText(QCoreApplication::translate("QgsFormAnnotationDialogBase", "Live update", nullptr));
    } // retranslateUi

};

namespace Ui {
    class QgsFormAnnotationDialogBase: public Ui_QgsFormAnnotationDialogBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QGSFORMANNOTATIONDIALOGBASE_H
