/********************************************************************************
** Form generated from reading UI file 'qgsarcgisimageserversourcewidgetbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QGSARCGISIMAGESERVERSOURCEWIDGETBASE_H
#define UI_QGSARCGISIMAGESERVERSOURCEWIDGETBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include "qgsauthsettingswidget.h"
#include "qgsprovidersourcewidget.h"

QT_BEGIN_NAMESPACE

class Ui_QgsArcGisImageServerSourceWidgetBase
{
public:
    QGridLayout *gridLayout;
    QComboBox *mImageFormatCombo;
    QLabel *lblReferer;
    QGroupBox *mAuthGroupBox;
    QVBoxLayout *verticalLayout_3;
    QgsAuthSettingsWidget *mAuthSettings;
    QLineEdit *mEditReferer;
    QLabel *label;
    QCheckBox *mUseTilesCheckBox;

    void setupUi(QgsProviderSourceWidget *QgsArcGisImageServerSourceWidgetBase)
    {
        if (QgsArcGisImageServerSourceWidgetBase->objectName().isEmpty())
            QgsArcGisImageServerSourceWidgetBase->setObjectName("QgsArcGisImageServerSourceWidgetBase");
        QgsArcGisImageServerSourceWidgetBase->resize(382, 132);
        gridLayout = new QGridLayout(QgsArcGisImageServerSourceWidgetBase);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        mImageFormatCombo = new QComboBox(QgsArcGisImageServerSourceWidgetBase);
        mImageFormatCombo->setObjectName("mImageFormatCombo");

        gridLayout->addWidget(mImageFormatCombo, 0, 1, 1, 1);

        lblReferer = new QLabel(QgsArcGisImageServerSourceWidgetBase);
        lblReferer->setObjectName("lblReferer");

        gridLayout->addWidget(lblReferer, 3, 0, 1, 1);

        mAuthGroupBox = new QGroupBox(QgsArcGisImageServerSourceWidgetBase);
        mAuthGroupBox->setObjectName("mAuthGroupBox");
        verticalLayout_3 = new QVBoxLayout(mAuthGroupBox);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(6, 6, 6, 6);
        mAuthSettings = new QgsAuthSettingsWidget(mAuthGroupBox);
        mAuthSettings->setObjectName("mAuthSettings");

        verticalLayout_3->addWidget(mAuthSettings);


        gridLayout->addWidget(mAuthGroupBox, 2, 0, 1, 2);

        mEditReferer = new QLineEdit(QgsArcGisImageServerSourceWidgetBase);
        mEditReferer->setObjectName("mEditReferer");

        gridLayout->addWidget(mEditReferer, 3, 1, 1, 1);

        label = new QLabel(QgsArcGisImageServerSourceWidgetBase);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        mUseTilesCheckBox = new QCheckBox(QgsArcGisImageServerSourceWidgetBase);
        mUseTilesCheckBox->setObjectName("mUseTilesCheckBox");

        gridLayout->addWidget(mUseTilesCheckBox, 1, 0, 1, 2);

#if QT_CONFIG(shortcut)
        lblReferer->setBuddy(mEditReferer);
#endif // QT_CONFIG(shortcut)

        retranslateUi(QgsArcGisImageServerSourceWidgetBase);

        QMetaObject::connectSlotsByName(QgsArcGisImageServerSourceWidgetBase);
    } // setupUi

    void retranslateUi(QgsProviderSourceWidget *QgsArcGisImageServerSourceWidgetBase)
    {
        QgsArcGisImageServerSourceWidgetBase->setWindowTitle(QCoreApplication::translate("QgsArcGisImageServerSourceWidgetBase", "ArcGIS REST Connection", nullptr));
        lblReferer->setText(QCoreApplication::translate("QgsArcGisImageServerSourceWidgetBase", "Referer", nullptr));
        mAuthGroupBox->setTitle(QCoreApplication::translate("QgsArcGisImageServerSourceWidgetBase", "Authentication", nullptr));
#if QT_CONFIG(tooltip)
        mEditReferer->setToolTip(QCoreApplication::translate("QgsArcGisImageServerSourceWidgetBase", "Optional custom referer", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(QCoreApplication::translate("QgsArcGisImageServerSourceWidgetBase", "Image format", nullptr));
        mUseTilesCheckBox->setText(QCoreApplication::translate("QgsArcGisImageServerSourceWidgetBase", "Retrieve data via tiles", nullptr));
    } // retranslateUi

};

namespace Ui {
    class QgsArcGisImageServerSourceWidgetBase: public Ui_QgsArcGisImageServerSourceWidgetBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QGSARCGISIMAGESERVERSOURCEWIDGETBASE_H
