/********************************************************************************
** Form generated from reading UI file 'qgsheatmappixelsizewidgetbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QGSHEATMAPPIXELSIZEWIDGETBASE_H
#define UI_QGSHEATMAPPIXELSIZEWIDGETBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include "qgsdoublespinbox.h"
#include "qgspanelwidget.h"
#include "qgsspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_QgsHeatmapPixelSizeWidgetBase
{
public:
    QGridLayout *gridLayout;
    QgsSpinBox *mRowsSpinBox;
    QLabel *columnLabel;
    QgsDoubleSpinBox *mCellXSpinBox;
    QLabel *cellsizeXLabel;
    QgsSpinBox *mColumnsSpinBox;
    QLabel *rowLabel;
    QLabel *cellsizeYLabel;
    QgsDoubleSpinBox *mCellYSpinBox;
    QSpacerItem *horizontalSpacer_2;

    void setupUi(QgsPanelWidget *QgsHeatmapPixelSizeWidgetBase)
    {
        if (QgsHeatmapPixelSizeWidgetBase->objectName().isEmpty())
            QgsHeatmapPixelSizeWidgetBase->setObjectName("QgsHeatmapPixelSizeWidgetBase");
        QgsHeatmapPixelSizeWidgetBase->resize(475, 60);
        QgsHeatmapPixelSizeWidgetBase->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(QgsHeatmapPixelSizeWidgetBase);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        mRowsSpinBox = new QgsSpinBox(QgsHeatmapPixelSizeWidgetBase);
        mRowsSpinBox->setObjectName("mRowsSpinBox");
        mRowsSpinBox->setKeyboardTracking(false);
        mRowsSpinBox->setMinimum(1);
        mRowsSpinBox->setMaximum(999999);

        gridLayout->addWidget(mRowsSpinBox, 0, 1, 1, 1);

        columnLabel = new QLabel(QgsHeatmapPixelSizeWidgetBase);
        columnLabel->setObjectName("columnLabel");

        gridLayout->addWidget(columnLabel, 0, 2, 1, 1);

        mCellXSpinBox = new QgsDoubleSpinBox(QgsHeatmapPixelSizeWidgetBase);
        mCellXSpinBox->setObjectName("mCellXSpinBox");
        mCellXSpinBox->setDecimals(6);
        mCellXSpinBox->setMaximum(999999999.000000000000000);

        gridLayout->addWidget(mCellXSpinBox, 1, 1, 1, 1);

        cellsizeXLabel = new QLabel(QgsHeatmapPixelSizeWidgetBase);
        cellsizeXLabel->setObjectName("cellsizeXLabel");

        gridLayout->addWidget(cellsizeXLabel, 1, 0, 1, 1);

        mColumnsSpinBox = new QgsSpinBox(QgsHeatmapPixelSizeWidgetBase);
        mColumnsSpinBox->setObjectName("mColumnsSpinBox");
        mColumnsSpinBox->setKeyboardTracking(false);
        mColumnsSpinBox->setMinimum(1);
        mColumnsSpinBox->setMaximum(999999);

        gridLayout->addWidget(mColumnsSpinBox, 0, 3, 1, 1);

        rowLabel = new QLabel(QgsHeatmapPixelSizeWidgetBase);
        rowLabel->setObjectName("rowLabel");

        gridLayout->addWidget(rowLabel, 0, 0, 1, 1);

        cellsizeYLabel = new QLabel(QgsHeatmapPixelSizeWidgetBase);
        cellsizeYLabel->setObjectName("cellsizeYLabel");

        gridLayout->addWidget(cellsizeYLabel, 1, 2, 1, 1);

        mCellYSpinBox = new QgsDoubleSpinBox(QgsHeatmapPixelSizeWidgetBase);
        mCellYSpinBox->setObjectName("mCellYSpinBox");
        mCellYSpinBox->setDecimals(6);
        mCellYSpinBox->setMaximum(999999999.000000000000000);

        gridLayout->addWidget(mCellYSpinBox, 1, 3, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_2, 0, 4, 1, 1);

        gridLayout->setColumnStretch(4, 1);
#if QT_CONFIG(shortcut)
        columnLabel->setBuddy(mColumnsSpinBox);
        rowLabel->setBuddy(mRowsSpinBox);
#endif // QT_CONFIG(shortcut)

        retranslateUi(QgsHeatmapPixelSizeWidgetBase);

        QMetaObject::connectSlotsByName(QgsHeatmapPixelSizeWidgetBase);
    } // setupUi

    void retranslateUi(QgsPanelWidget *QgsHeatmapPixelSizeWidgetBase)
    {
#if QT_CONFIG(tooltip)
        mRowsSpinBox->setToolTip(QCoreApplication::translate("QgsHeatmapPixelSizeWidgetBase", "Number of rows (pixels) in output raster", nullptr));
#endif // QT_CONFIG(tooltip)
        columnLabel->setText(QCoreApplication::translate("QgsHeatmapPixelSizeWidgetBase", "Columns", nullptr));
#if QT_CONFIG(tooltip)
        mCellXSpinBox->setToolTip(QCoreApplication::translate("QgsHeatmapPixelSizeWidgetBase", "Resolution of each pixel in output raster, in layer units", nullptr));
#endif // QT_CONFIG(tooltip)
        cellsizeXLabel->setText(QCoreApplication::translate("QgsHeatmapPixelSizeWidgetBase", "Pixel size X", nullptr));
#if QT_CONFIG(tooltip)
        mColumnsSpinBox->setToolTip(QCoreApplication::translate("QgsHeatmapPixelSizeWidgetBase", "Number of columns (pixels) in output raster", nullptr));
#endif // QT_CONFIG(tooltip)
        rowLabel->setText(QCoreApplication::translate("QgsHeatmapPixelSizeWidgetBase", "Rows", nullptr));
        cellsizeYLabel->setText(QCoreApplication::translate("QgsHeatmapPixelSizeWidgetBase", "Pixel size Y", nullptr));
#if QT_CONFIG(tooltip)
        mCellYSpinBox->setToolTip(QCoreApplication::translate("QgsHeatmapPixelSizeWidgetBase", "Resolution of each pixel in output raster, in layer units", nullptr));
#endif // QT_CONFIG(tooltip)
        (void)QgsHeatmapPixelSizeWidgetBase;
    } // retranslateUi

};

namespace Ui {
    class QgsHeatmapPixelSizeWidgetBase: public Ui_QgsHeatmapPixelSizeWidgetBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QGSHEATMAPPIXELSIZEWIDGETBASE_H
