/********************************************************************************
** Form generated from reading UI file 'qgsprocessingrastersourceoptionsbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QGSPROCESSINGRASTERSOURCEOPTIONSBASE_H
#define UI_QGSPROCESSINGRASTERSOURCEOPTIONSBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "qgspanelwidget.h"
#include "qgsscalewidget.h"
#include "qgsscrollarea.h"
#include "qgsspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_QgsProcessingRasterSourceOptionsBase
{
public:
    QVBoxLayout *verticalLayout;
    QgsScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QGridLayout *gridLayout;
    QLabel *label_2;
    QgsScaleWidget *mReferenceScale;
    QSpacerItem *verticalSpacer;
    QLabel *label;
    QgsSpinBox *mDpiSpinBox;

    void setupUi(QgsPanelWidget *QgsProcessingRasterSourceOptionsBase)
    {
        if (QgsProcessingRasterSourceOptionsBase->objectName().isEmpty())
            QgsProcessingRasterSourceOptionsBase->setObjectName("QgsProcessingRasterSourceOptionsBase");
        QgsProcessingRasterSourceOptionsBase->resize(448, 197);
        QgsProcessingRasterSourceOptionsBase->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(QgsProcessingRasterSourceOptionsBase);
        verticalLayout->setSpacing(2);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        scrollArea = new QgsScrollArea(QgsProcessingRasterSourceOptionsBase);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 448, 197));
        gridLayout = new QGridLayout(scrollAreaWidgetContents);
        gridLayout->setObjectName("gridLayout");
        label_2 = new QLabel(scrollAreaWidgetContents);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 4, 0, 1, 1);

        mReferenceScale = new QgsScaleWidget(scrollAreaWidgetContents);
        mReferenceScale->setObjectName("mReferenceScale");
        mReferenceScale->setShowCurrentScaleButton(true);
        mReferenceScale->setScale(true);
        mReferenceScale->setMinScale(false);

        gridLayout->addWidget(mReferenceScale, 1, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 6, 0, 1, 1);

        label = new QLabel(scrollAreaWidgetContents);
        label->setObjectName("label");

        gridLayout->addWidget(label, 1, 0, 1, 1);

        mDpiSpinBox = new QgsSpinBox(scrollAreaWidgetContents);
        mDpiSpinBox->setObjectName("mDpiSpinBox");
        mDpiSpinBox->setMaximum(3000);
        mDpiSpinBox->setValue(96);
        mDpiSpinBox->setExpressionsEnabled(false);

        gridLayout->addWidget(mDpiSpinBox, 4, 1, 1, 1);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout->addWidget(scrollArea);


        retranslateUi(QgsProcessingRasterSourceOptionsBase);

        QMetaObject::connectSlotsByName(QgsProcessingRasterSourceOptionsBase);
    } // setupUi

    void retranslateUi(QgsPanelWidget *QgsProcessingRasterSourceOptionsBase)
    {
        label_2->setText(QCoreApplication::translate("QgsProcessingRasterSourceOptionsBase", "Service resolution", nullptr));
#if QT_CONFIG(tooltip)
        mReferenceScale->setToolTip(QCoreApplication::translate("QgsProcessingRasterSourceOptionsBase", "If set, gives the algorithm a hint on the reference scale that should be used to retrieve an image from the WMS server", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(QCoreApplication::translate("QgsProcessingRasterSourceOptionsBase", "Reference scale", nullptr));
#if QT_CONFIG(tooltip)
        mDpiSpinBox->setToolTip(QCoreApplication::translate("QgsProcessingRasterSourceOptionsBase", "If set, gives the algorithm a hint on the DPI used by the WMS server", nullptr));
#endif // QT_CONFIG(tooltip)
        mDpiSpinBox->setSuffix(QCoreApplication::translate("QgsProcessingRasterSourceOptionsBase", " dpi", nullptr));
        (void)QgsProcessingRasterSourceOptionsBase;
    } // retranslateUi

};

namespace Ui {
    class QgsProcessingRasterSourceOptionsBase: public Ui_QgsProcessingRasterSourceOptionsBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QGSPROCESSINGRASTERSOURCEOPTIONSBASE_H
