/********************************************************************************
** Form generated from reading UI file 'map3dconfigwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAP3DCONFIGWIDGET_H
#define UI_MAP3DCONFIGWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "qgsambientocclusionsettingswidget.h"
#include "qgscolorbutton.h"
#include "qgsdoublespinbox.h"
#include "qgsextentgroupbox.h"
#include "qgslightswidget.h"
#include "qgsmaplayercombobox.h"
#include "qgsmessagebar.h"
#include "qgsphongmaterialwidget.h"
#include "qgsscrollarea.h"
#include "qgsspinbox.h"
#include "qgsstackedwidget.h"

QT_BEGIN_NAMESPACE

class Ui_Map3DConfigWidget
{
public:
    QVBoxLayout *verticalLayout_4;
    QgsMessageBar *mMessageBar;
    QSplitter *m3DOptionsSplitter;
    QFrame *m3DOptionsListFrame;
    QVBoxLayout *verticalLayout_23;
    QListWidget *m3DOptionsListWidget;
    QFrame *m3DOptionGroupsFrame;
    QVBoxLayout *verticalLayout_3;
    QStackedWidget *m3DOptionsStackedWidget;
    QWidget *mPageGeneral;
    QVBoxLayout *verticalLayout_5;
    QgsScrollArea *scrollAreaGeneral;
    QWidget *scrollAreaGeneralWidgetContents;
    QVBoxLayout *verticalLayout_7;
    QgsExtentGroupBox *groupExtent;
    QGroupBox *groupBoxBackground;
    QGridLayout *gridLayout_7;
    QComboBox *comboBox;
    QgsStackedWidget *stackedWidget;
    QWidget *pageGradient;
    QGridLayout *gridLayout_9;
    QLabel *labelBottomColor;
    QgsColorButton *mBtnGradientBottomColor;
    QgsColorButton *mBtnGradientTopColor;
    QLabel *labelTopColor;
    QSpacerItem *verticalSpacer_2;
    QWidget *pageSkybox;
    QGridLayout *gridLayout_10;
    QSpacerItem *verticalSpacer;
    QWidget *mPageTerrain;
    QVBoxLayout *verticalLayout_61;
    QgsScrollArea *scrollAreaTerrain;
    QWidget *scrollAreaTerrainWidgetContents;
    QVBoxLayout *verticalLayoutTerrain;
    QGroupBox *groupTerrain;
    QGridLayout *gridLayout1;
    QLabel *labelTerrainResolution;
    QgsDoubleSpinBox *spinTerrainSkirtHeight;
    QComboBox *cboTerrainType;
    QLabel *labelTerrainSkirtHeight;
    QgsMapLayerComboBox *cboTerrainLayer;
    QgsSpinBox *spinTerrainResolution;
    QLabel *labelTerrainLayer;
    QgsDoubleSpinBox *spinTerrainScale;
    QLabel *labelTerrainType;
    QLabel *labelTerrainScale;
    QgsDoubleSpinBox *terrainElevationOffsetSpinBox;
    QLabel *labelterrainElevationOffset;
    QFrame *groupMeshTerrainShading;
    QVBoxLayout *verticalLayout;
    QGroupBox *groupTerrainShading;
    QVBoxLayout *verticalLayout_2;
    QgsPhongMaterialWidget *widgetTerrainMaterial;
    QSpacerItem *verticalSpacerTerrain;
    QWidget *mPageLight;
    QVBoxLayout *verticalLayout_64;
    QgsScrollArea *scrollAreaLight;
    QWidget *scrollAreaLightWidgetContents;
    QVBoxLayout *verticalLayoutLight;
    QGroupBox *groupLights;
    QVBoxLayout *verticalLayout_31;
    QgsLightsWidget *widgetLights;
    QSpacerItem *verticalSpacerLights;
    QWidget *mPageEffects;
    QVBoxLayout *verticalLayout_6;
    QgsScrollArea *scrollAreaShadow;
    QWidget *scrollAreaShadowWidgetContents;
    QVBoxLayout *verticalLayoutShadow;
    QGroupBox *groupShadowRendering;
    QGridLayout *gridLayout_6;
    QGridLayout *gridLayout_5;
    QGroupBox *edlGroupBox;
    QGridLayout *gridLayout_2;
    QgsDoubleSpinBox *edlStrengthSpinBox;
    QLabel *label;
    QLabel *label_2;
    QgsSpinBox *edlDistanceSpinBox;
    QgsAmbientOcclusionSettingsWidget *mAmbientOcclusionSettingsWidget;
    QGroupBox *mBloomGroupBox;
    QGridLayout *gridLayout_4;
    QLabel *label_9;
    QgsDoubleSpinBox *mSpinBloomIntensity;
    QLabel *label_10;
    QgsDoubleSpinBox *mSpinBloomRadius;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout_3;
    QgsDoubleSpinBox *mSpinExposure;
    QLabel *label_8;
    QLabel *label_3;
    QComboBox *mComboToneMapping;
    QSpacerItem *verticalSpacerShadow;
    QWidget *mPageCameraSkybox;
    QVBoxLayout *verticalLayout_62;
    QgsScrollArea *scrollAreaCameraSkybox;
    QWidget *scrollAreaCameraSkyboxWidgetContents;
    QVBoxLayout *verticalLayoutCameraSkybox;
    QGroupBox *cameraTerrain;
    QGridLayout *gridLayout;
    QLabel *labelCameraProjectionType;
    QLabel *labelFieldofView;
    QComboBox *cboCameraProjectionType;
    QLabel *labelCameraNavigationMode;
    QComboBox *mCameraNavigationModeCombo;
    QgsDoubleSpinBox *mCameraMovementSpeed;
    QLabel *labelMovementSpeed;
    QgsDoubleSpinBox *spinCameraFieldOfView;
    QGroupBox *mGroupBox3dAxis;
    QFormLayout *formLayout_2;
    QLabel *mLabel3dAxisType_2;
    QComboBox *mCbo3dAxisType;
    QLabel *mLabel3dAxisHorizPos_2;
    QComboBox *mCbo3dAxisHorizPos;
    QLabel *mLabel3dAxisVertPos_2;
    QComboBox *mCbo3dAxisVertPos;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_8;
    QCheckBox *mSync2DTo3DCheckbox;
    QCheckBox *mSync3DTo2DCheckbox;
    QCheckBox *mVisualizeExtentCheckBox;
    QCheckBox *mShowMapOverlayCheckBox;
    QSpacerItem *verticalSpacerCameraSkybox;
    QWidget *mPageAdvanced;
    QVBoxLayout *verticalLayout_63;
    QgsScrollArea *scrollAreaAdvanced;
    QWidget *scrollAreaAdvancedWidgetContents;
    QVBoxLayout *verticalLayoutAdvanced;
    QGroupBox *groupAdvanced;
    QGridLayout *gridLayout_11;
    QGridLayout *gridLayoutAdvanced;
    QCheckBox *chkShowLabels;
    QgsSpinBox *spinMapResolution;
    QLabel *labelZoomLevels;
    QLabel *label_5;
    QgsDoubleSpinBox *spinGroundError;
    QLabel *label_7;
    QCheckBox *mFpsCounterCheckBox;
    QLabel *label_6;
    QgsDoubleSpinBox *spinScreenError;
    QLabel *label_4;
    QCheckBox *chkShowDebugPanel;
    QSpacerItem *verticalSpacerAdvanced;

    void setupUi(QWidget *Map3DConfigWidget)
    {
        if (Map3DConfigWidget->objectName().isEmpty())
            Map3DConfigWidget->setObjectName("Map3DConfigWidget");
        Map3DConfigWidget->resize(881, 665);
        verticalLayout_4 = new QVBoxLayout(Map3DConfigWidget);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        mMessageBar = new QgsMessageBar(Map3DConfigWidget);
        mMessageBar->setObjectName("mMessageBar");

        verticalLayout_4->addWidget(mMessageBar);

        m3DOptionsSplitter = new QSplitter(Map3DConfigWidget);
        m3DOptionsSplitter->setObjectName("m3DOptionsSplitter");
        m3DOptionsSplitter->setOrientation(Qt::Orientation::Horizontal);
        m3DOptionsSplitter->setChildrenCollapsible(false);
        m3DOptionsListFrame = new QFrame(m3DOptionsSplitter);
        m3DOptionsListFrame->setObjectName("m3DOptionsListFrame");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m3DOptionsListFrame->sizePolicy().hasHeightForWidth());
        m3DOptionsListFrame->setSizePolicy(sizePolicy);
        m3DOptionsListFrame->setMinimumSize(QSize(32, 0));
        m3DOptionsListFrame->setFrameShape(QFrame::Shape::NoFrame);
        m3DOptionsListFrame->setFrameShadow(QFrame::Shadow::Raised);
        verticalLayout_23 = new QVBoxLayout(m3DOptionsListFrame);
        verticalLayout_23->setObjectName("verticalLayout_23");
        verticalLayout_23->setContentsMargins(3, 0, 0, 0);
        m3DOptionsListWidget = new QListWidget(m3DOptionsListFrame);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/images/themes/default/propertyicons/general.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        QListWidgetItem *__qlistwidgetitem = new QListWidgetItem(m3DOptionsListWidget);
        __qlistwidgetitem->setIcon(icon);
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/images/themes/default/mLayoutItem3DMap.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        QListWidgetItem *__qlistwidgetitem1 = new QListWidgetItem(m3DOptionsListWidget);
        __qlistwidgetitem1->setIcon(icon1);
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/images/themes/default/mActionHighlightFeature.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        QListWidgetItem *__qlistwidgetitem2 = new QListWidgetItem(m3DOptionsListWidget);
        __qlistwidgetitem2->setIcon(icon2);
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/images/themes/default/mIconShadow.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        QListWidgetItem *__qlistwidgetitem3 = new QListWidgetItem(m3DOptionsListWidget);
        __qlistwidgetitem3->setIcon(icon3);
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/images/themes/default/mIconCamera.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        QListWidgetItem *__qlistwidgetitem4 = new QListWidgetItem(m3DOptionsListWidget);
        __qlistwidgetitem4->setIcon(icon4);
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/images/themes/default/propertyicons/system.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        QListWidgetItem *__qlistwidgetitem5 = new QListWidgetItem(m3DOptionsListWidget);
        __qlistwidgetitem5->setIcon(icon5);
        m3DOptionsListWidget->setObjectName("m3DOptionsListWidget");
        m3DOptionsListWidget->setMinimumSize(QSize(32, 0));
        m3DOptionsListWidget->setProperty("showDropIndicator", QVariant(false));
        m3DOptionsListWidget->setIconSize(QSize(20, 20));

        verticalLayout_23->addWidget(m3DOptionsListWidget);

        m3DOptionsSplitter->addWidget(m3DOptionsListFrame);
        m3DOptionGroupsFrame = new QFrame(m3DOptionsSplitter);
        m3DOptionGroupsFrame->setObjectName("m3DOptionGroupsFrame");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(10);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(m3DOptionGroupsFrame->sizePolicy().hasHeightForWidth());
        m3DOptionGroupsFrame->setSizePolicy(sizePolicy1);
        m3DOptionGroupsFrame->setFrameShape(QFrame::Shape::NoFrame);
        m3DOptionGroupsFrame->setFrameShadow(QFrame::Shadow::Raised);
        verticalLayout_3 = new QVBoxLayout(m3DOptionGroupsFrame);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        m3DOptionsStackedWidget = new QStackedWidget(m3DOptionGroupsFrame);
        m3DOptionsStackedWidget->setObjectName("m3DOptionsStackedWidget");
        mPageGeneral = new QWidget();
        mPageGeneral->setObjectName("mPageGeneral");
        verticalLayout_5 = new QVBoxLayout(mPageGeneral);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(-1, 0, 0, 0);
        scrollAreaGeneral = new QgsScrollArea(mPageGeneral);
        scrollAreaGeneral->setObjectName("scrollAreaGeneral");
        scrollAreaGeneral->setFrameShape(QFrame::Shape::NoFrame);
        scrollAreaGeneral->setWidgetResizable(true);
        scrollAreaGeneralWidgetContents = new QWidget();
        scrollAreaGeneralWidgetContents->setObjectName("scrollAreaGeneralWidgetContents");
        scrollAreaGeneralWidgetContents->setGeometry(QRect(0, 0, 811, 648));
        verticalLayout_7 = new QVBoxLayout(scrollAreaGeneralWidgetContents);
        verticalLayout_7->setObjectName("verticalLayout_7");
        verticalLayout_7->setContentsMargins(0, 0, 0, 0);
        groupExtent = new QgsExtentGroupBox(scrollAreaGeneralWidgetContents);
        groupExtent->setObjectName("groupExtent");

        verticalLayout_7->addWidget(groupExtent);

        groupBoxBackground = new QGroupBox(scrollAreaGeneralWidgetContents);
        groupBoxBackground->setObjectName("groupBoxBackground");
        groupBoxBackground->setCheckable(true);
        groupBoxBackground->setChecked(false);
        gridLayout_7 = new QGridLayout(groupBoxBackground);
        gridLayout_7->setObjectName("gridLayout_7");
        comboBox = new QComboBox(groupBoxBackground);
        comboBox->setObjectName("comboBox");

        gridLayout_7->addWidget(comboBox, 0, 0, 1, 1);

        stackedWidget = new QgsStackedWidget(groupBoxBackground);
        stackedWidget->setObjectName("stackedWidget");
        pageGradient = new QWidget();
        pageGradient->setObjectName("pageGradient");
        gridLayout_9 = new QGridLayout(pageGradient);
        gridLayout_9->setObjectName("gridLayout_9");
        gridLayout_9->setContentsMargins(0, 0, 0, 0);
        labelBottomColor = new QLabel(pageGradient);
        labelBottomColor->setObjectName("labelBottomColor");

        gridLayout_9->addWidget(labelBottomColor, 1, 0, 1, 1);

        mBtnGradientBottomColor = new QgsColorButton(pageGradient);
        mBtnGradientBottomColor->setObjectName("mBtnGradientBottomColor");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(mBtnGradientBottomColor->sizePolicy().hasHeightForWidth());
        mBtnGradientBottomColor->setSizePolicy(sizePolicy2);

        gridLayout_9->addWidget(mBtnGradientBottomColor, 1, 1, 1, 1);

        mBtnGradientTopColor = new QgsColorButton(pageGradient);
        mBtnGradientTopColor->setObjectName("mBtnGradientTopColor");
        sizePolicy2.setHeightForWidth(mBtnGradientTopColor->sizePolicy().hasHeightForWidth());
        mBtnGradientTopColor->setSizePolicy(sizePolicy2);

        gridLayout_9->addWidget(mBtnGradientTopColor, 0, 1, 1, 1);

        labelTopColor = new QLabel(pageGradient);
        labelTopColor->setObjectName("labelTopColor");

        gridLayout_9->addWidget(labelTopColor, 0, 0, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_9->addItem(verticalSpacer_2, 2, 1, 1, 1);

        gridLayout_9->setColumnStretch(1, 1);
        stackedWidget->addWidget(pageGradient);
        pageSkybox = new QWidget();
        pageSkybox->setObjectName("pageSkybox");
        gridLayout_10 = new QGridLayout(pageSkybox);
        gridLayout_10->setObjectName("gridLayout_10");
        gridLayout_10->setContentsMargins(0, 0, 0, 0);
        stackedWidget->addWidget(pageSkybox);

        gridLayout_7->addWidget(stackedWidget, 1, 0, 1, 1);


        verticalLayout_7->addWidget(groupBoxBackground);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_7->addItem(verticalSpacer);

        scrollAreaGeneral->setWidget(scrollAreaGeneralWidgetContents);

        verticalLayout_5->addWidget(scrollAreaGeneral);

        m3DOptionsStackedWidget->addWidget(mPageGeneral);
        mPageTerrain = new QWidget();
        mPageTerrain->setObjectName("mPageTerrain");
        verticalLayout_61 = new QVBoxLayout(mPageTerrain);
        verticalLayout_61->setObjectName("verticalLayout_61");
        verticalLayout_61->setContentsMargins(-1, 0, 0, 0);
        scrollAreaTerrain = new QgsScrollArea(mPageTerrain);
        scrollAreaTerrain->setObjectName("scrollAreaTerrain");
        scrollAreaTerrain->setFrameShape(QFrame::Shape::NoFrame);
        scrollAreaTerrain->setWidgetResizable(true);
        scrollAreaTerrainWidgetContents = new QWidget();
        scrollAreaTerrainWidgetContents->setObjectName("scrollAreaTerrainWidgetContents");
        scrollAreaTerrainWidgetContents->setGeometry(QRect(0, 0, 811, 648));
        verticalLayoutTerrain = new QVBoxLayout(scrollAreaTerrainWidgetContents);
        verticalLayoutTerrain->setObjectName("verticalLayoutTerrain");
        verticalLayoutTerrain->setContentsMargins(0, 0, 0, 0);
        groupTerrain = new QGroupBox(scrollAreaTerrainWidgetContents);
        groupTerrain->setObjectName("groupTerrain");
        groupTerrain->setCheckable(true);
        groupTerrain->setChecked(false);
        gridLayout1 = new QGridLayout(groupTerrain);
        gridLayout1->setObjectName("gridLayout1");
        labelTerrainResolution = new QLabel(groupTerrain);
        labelTerrainResolution->setObjectName("labelTerrainResolution");

        gridLayout1->addWidget(labelTerrainResolution, 3, 0, 1, 1);

        spinTerrainSkirtHeight = new QgsDoubleSpinBox(groupTerrain);
        spinTerrainSkirtHeight->setObjectName("spinTerrainSkirtHeight");
        spinTerrainSkirtHeight->setDecimals(1);
        spinTerrainSkirtHeight->setMaximum(10000.000000000000000);
        spinTerrainSkirtHeight->setSingleStep(10.000000000000000);

        gridLayout1->addWidget(spinTerrainSkirtHeight, 4, 1, 1, 2);

        cboTerrainType = new QComboBox(groupTerrain);
        cboTerrainType->setObjectName("cboTerrainType");

        gridLayout1->addWidget(cboTerrainType, 0, 1, 1, 2);

        labelTerrainSkirtHeight = new QLabel(groupTerrain);
        labelTerrainSkirtHeight->setObjectName("labelTerrainSkirtHeight");

        gridLayout1->addWidget(labelTerrainSkirtHeight, 4, 0, 1, 1);

        cboTerrainLayer = new QgsMapLayerComboBox(groupTerrain);
        cboTerrainLayer->setObjectName("cboTerrainLayer");

        gridLayout1->addWidget(cboTerrainLayer, 1, 1, 1, 2);

        spinTerrainResolution = new QgsSpinBox(groupTerrain);
        spinTerrainResolution->setObjectName("spinTerrainResolution");
        spinTerrainResolution->setMaximum(4096);

        gridLayout1->addWidget(spinTerrainResolution, 3, 1, 1, 2);

        labelTerrainLayer = new QLabel(groupTerrain);
        labelTerrainLayer->setObjectName("labelTerrainLayer");

        gridLayout1->addWidget(labelTerrainLayer, 1, 0, 1, 1);

        spinTerrainScale = new QgsDoubleSpinBox(groupTerrain);
        spinTerrainScale->setObjectName("spinTerrainScale");
        spinTerrainScale->setValue(1.000000000000000);

        gridLayout1->addWidget(spinTerrainScale, 2, 1, 1, 2);

        labelTerrainType = new QLabel(groupTerrain);
        labelTerrainType->setObjectName("labelTerrainType");

        gridLayout1->addWidget(labelTerrainType, 0, 0, 1, 1);

        labelTerrainScale = new QLabel(groupTerrain);
        labelTerrainScale->setObjectName("labelTerrainScale");

        gridLayout1->addWidget(labelTerrainScale, 2, 0, 1, 1);

        terrainElevationOffsetSpinBox = new QgsDoubleSpinBox(groupTerrain);
        terrainElevationOffsetSpinBox->setObjectName("terrainElevationOffsetSpinBox");
        terrainElevationOffsetSpinBox->setMinimum(-1000000.000000000000000);
        terrainElevationOffsetSpinBox->setMaximum(1000000.000000000000000);

        gridLayout1->addWidget(terrainElevationOffsetSpinBox, 5, 1, 1, 2);

        labelterrainElevationOffset = new QLabel(groupTerrain);
        labelterrainElevationOffset->setObjectName("labelterrainElevationOffset");

        gridLayout1->addWidget(labelterrainElevationOffset, 5, 0, 1, 1);


        verticalLayoutTerrain->addWidget(groupTerrain);

        groupMeshTerrainShading = new QFrame(scrollAreaTerrainWidgetContents);
        groupMeshTerrainShading->setObjectName("groupMeshTerrainShading");
        verticalLayout = new QVBoxLayout(groupMeshTerrainShading);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);

        verticalLayoutTerrain->addWidget(groupMeshTerrainShading);

        groupTerrainShading = new QGroupBox(scrollAreaTerrainWidgetContents);
        groupTerrainShading->setObjectName("groupTerrainShading");
        groupTerrainShading->setCheckable(true);
        verticalLayout_2 = new QVBoxLayout(groupTerrainShading);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        widgetTerrainMaterial = new QgsPhongMaterialWidget(groupTerrainShading);
        widgetTerrainMaterial->setObjectName("widgetTerrainMaterial");
        widgetTerrainMaterial->setProperty("hasOpacity", QVariant(false));

        verticalLayout_2->addWidget(widgetTerrainMaterial);


        verticalLayoutTerrain->addWidget(groupTerrainShading);

        verticalSpacerTerrain = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayoutTerrain->addItem(verticalSpacerTerrain);

        scrollAreaTerrain->setWidget(scrollAreaTerrainWidgetContents);

        verticalLayout_61->addWidget(scrollAreaTerrain);

        m3DOptionsStackedWidget->addWidget(mPageTerrain);
        mPageLight = new QWidget();
        mPageLight->setObjectName("mPageLight");
        verticalLayout_64 = new QVBoxLayout(mPageLight);
        verticalLayout_64->setObjectName("verticalLayout_64");
        verticalLayout_64->setContentsMargins(-1, 0, 0, 0);
        scrollAreaLight = new QgsScrollArea(mPageLight);
        scrollAreaLight->setObjectName("scrollAreaLight");
        scrollAreaLight->setFrameShape(QFrame::Shape::NoFrame);
        scrollAreaLight->setWidgetResizable(true);
        scrollAreaLightWidgetContents = new QWidget();
        scrollAreaLightWidgetContents->setObjectName("scrollAreaLightWidgetContents");
        scrollAreaLightWidgetContents->setGeometry(QRect(0, 0, 811, 648));
        verticalLayoutLight = new QVBoxLayout(scrollAreaLightWidgetContents);
        verticalLayoutLight->setObjectName("verticalLayoutLight");
        verticalLayoutLight->setContentsMargins(0, 0, 0, 0);
        groupLights = new QGroupBox(scrollAreaLightWidgetContents);
        groupLights->setObjectName("groupLights");
        verticalLayout_31 = new QVBoxLayout(groupLights);
        verticalLayout_31->setObjectName("verticalLayout_31");
        verticalLayout_31->setContentsMargins(0, 0, 0, 0);
        widgetLights = new QgsLightsWidget(groupLights);
        widgetLights->setObjectName("widgetLights");

        verticalLayout_31->addWidget(widgetLights);


        verticalLayoutLight->addWidget(groupLights);

        verticalSpacerLights = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayoutLight->addItem(verticalSpacerLights);

        scrollAreaLight->setWidget(scrollAreaLightWidgetContents);

        verticalLayout_64->addWidget(scrollAreaLight);

        m3DOptionsStackedWidget->addWidget(mPageLight);
        mPageEffects = new QWidget();
        mPageEffects->setObjectName("mPageEffects");
        verticalLayout_6 = new QVBoxLayout(mPageEffects);
        verticalLayout_6->setObjectName("verticalLayout_6");
        verticalLayout_6->setContentsMargins(-1, 0, 0, 0);
        scrollAreaShadow = new QgsScrollArea(mPageEffects);
        scrollAreaShadow->setObjectName("scrollAreaShadow");
        scrollAreaShadow->setFrameShape(QFrame::Shape::NoFrame);
        scrollAreaShadow->setWidgetResizable(true);
        scrollAreaShadowWidgetContents = new QWidget();
        scrollAreaShadowWidgetContents->setObjectName("scrollAreaShadowWidgetContents");
        scrollAreaShadowWidgetContents->setGeometry(QRect(0, 0, 811, 648));
        verticalLayoutShadow = new QVBoxLayout(scrollAreaShadowWidgetContents);
        verticalLayoutShadow->setObjectName("verticalLayoutShadow");
        verticalLayoutShadow->setContentsMargins(0, 0, 0, 0);
        groupShadowRendering = new QGroupBox(scrollAreaShadowWidgetContents);
        groupShadowRendering->setObjectName("groupShadowRendering");
        groupShadowRendering->setCheckable(true);
        groupShadowRendering->setChecked(false);
        gridLayout_6 = new QGridLayout(groupShadowRendering);
        gridLayout_6->setObjectName("gridLayout_6");
        gridLayout_6->setContentsMargins(0, 0, 0, 0);
        gridLayout_5 = new QGridLayout();
        gridLayout_5->setObjectName("gridLayout_5");

        gridLayout_6->addLayout(gridLayout_5, 0, 0, 1, 1);


        verticalLayoutShadow->addWidget(groupShadowRendering);

        edlGroupBox = new QGroupBox(scrollAreaShadowWidgetContents);
        edlGroupBox->setObjectName("edlGroupBox");
        edlGroupBox->setCheckable(true);
        edlGroupBox->setChecked(false);
        gridLayout_2 = new QGridLayout(edlGroupBox);
        gridLayout_2->setObjectName("gridLayout_2");
        edlStrengthSpinBox = new QgsDoubleSpinBox(edlGroupBox);
        edlStrengthSpinBox->setObjectName("edlStrengthSpinBox");
        edlStrengthSpinBox->setMaximum(10000.000000000000000);
        edlStrengthSpinBox->setSingleStep(100.000000000000000);
        edlStrengthSpinBox->setValue(1000.000000000000000);

        gridLayout_2->addWidget(edlStrengthSpinBox, 0, 1, 1, 1);

        label = new QLabel(edlGroupBox);
        label->setObjectName("label");

        gridLayout_2->addWidget(label, 0, 0, 1, 1);

        label_2 = new QLabel(edlGroupBox);
        label_2->setObjectName("label_2");

        gridLayout_2->addWidget(label_2, 1, 0, 1, 1);

        edlDistanceSpinBox = new QgsSpinBox(edlGroupBox);
        edlDistanceSpinBox->setObjectName("edlDistanceSpinBox");
        edlDistanceSpinBox->setMinimum(1);
        edlDistanceSpinBox->setMaximum(20);

        gridLayout_2->addWidget(edlDistanceSpinBox, 1, 1, 1, 1);


        verticalLayoutShadow->addWidget(edlGroupBox);

        mAmbientOcclusionSettingsWidget = new QgsAmbientOcclusionSettingsWidget(scrollAreaShadowWidgetContents);
        mAmbientOcclusionSettingsWidget->setObjectName("mAmbientOcclusionSettingsWidget");

        verticalLayoutShadow->addWidget(mAmbientOcclusionSettingsWidget);

        mBloomGroupBox = new QGroupBox(scrollAreaShadowWidgetContents);
        mBloomGroupBox->setObjectName("mBloomGroupBox");
        mBloomGroupBox->setCheckable(true);
        mBloomGroupBox->setChecked(false);
        gridLayout_4 = new QGridLayout(mBloomGroupBox);
        gridLayout_4->setObjectName("gridLayout_4");
        label_9 = new QLabel(mBloomGroupBox);
        label_9->setObjectName("label_9");

        gridLayout_4->addWidget(label_9, 0, 0, 1, 1);

        mSpinBloomIntensity = new QgsDoubleSpinBox(mBloomGroupBox);
        mSpinBloomIntensity->setObjectName("mSpinBloomIntensity");
        mSpinBloomIntensity->setMaximum(10000.000000000000000);
        mSpinBloomIntensity->setSingleStep(0.010000000000000);
        mSpinBloomIntensity->setValue(0.050000000000000);

        gridLayout_4->addWidget(mSpinBloomIntensity, 0, 1, 1, 1);

        label_10 = new QLabel(mBloomGroupBox);
        label_10->setObjectName("label_10");

        gridLayout_4->addWidget(label_10, 1, 0, 1, 1);

        mSpinBloomRadius = new QgsDoubleSpinBox(mBloomGroupBox);
        mSpinBloomRadius->setObjectName("mSpinBloomRadius");
        mSpinBloomRadius->setDecimals(1);
        mSpinBloomRadius->setMaximum(10.000000000000000);
        mSpinBloomRadius->setSingleStep(0.100000000000000);
        mSpinBloomRadius->setValue(5.000000000000000);

        gridLayout_4->addWidget(mSpinBloomRadius, 1, 1, 1, 1);


        verticalLayoutShadow->addWidget(mBloomGroupBox);

        groupBox_2 = new QGroupBox(scrollAreaShadowWidgetContents);
        groupBox_2->setObjectName("groupBox_2");
        gridLayout_3 = new QGridLayout(groupBox_2);
        gridLayout_3->setObjectName("gridLayout_3");
        mSpinExposure = new QgsDoubleSpinBox(groupBox_2);
        mSpinExposure->setObjectName("mSpinExposure");
        mSpinExposure->setMinimum(-5.000000000000000);
        mSpinExposure->setMaximum(5.000000000000000);
        mSpinExposure->setSingleStep(0.050000000000000);
        mSpinExposure->setValue(0.000000000000000);

        gridLayout_3->addWidget(mSpinExposure, 1, 1, 1, 1);

        label_8 = new QLabel(groupBox_2);
        label_8->setObjectName("label_8");

        gridLayout_3->addWidget(label_8, 1, 0, 1, 1);

        label_3 = new QLabel(groupBox_2);
        label_3->setObjectName("label_3");

        gridLayout_3->addWidget(label_3, 0, 0, 1, 1);

        mComboToneMapping = new QComboBox(groupBox_2);
        mComboToneMapping->setObjectName("mComboToneMapping");

        gridLayout_3->addWidget(mComboToneMapping, 0, 1, 1, 1);


        verticalLayoutShadow->addWidget(groupBox_2);

        verticalSpacerShadow = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayoutShadow->addItem(verticalSpacerShadow);

        scrollAreaShadow->setWidget(scrollAreaShadowWidgetContents);

        verticalLayout_6->addWidget(scrollAreaShadow);

        m3DOptionsStackedWidget->addWidget(mPageEffects);
        mPageCameraSkybox = new QWidget();
        mPageCameraSkybox->setObjectName("mPageCameraSkybox");
        verticalLayout_62 = new QVBoxLayout(mPageCameraSkybox);
        verticalLayout_62->setObjectName("verticalLayout_62");
        verticalLayout_62->setContentsMargins(-1, 0, 0, 0);
        scrollAreaCameraSkybox = new QgsScrollArea(mPageCameraSkybox);
        scrollAreaCameraSkybox->setObjectName("scrollAreaCameraSkybox");
        scrollAreaCameraSkybox->setFrameShape(QFrame::Shape::NoFrame);
        scrollAreaCameraSkybox->setWidgetResizable(true);
        scrollAreaCameraSkyboxWidgetContents = new QWidget();
        scrollAreaCameraSkyboxWidgetContents->setObjectName("scrollAreaCameraSkyboxWidgetContents");
        scrollAreaCameraSkyboxWidgetContents->setGeometry(QRect(0, 0, 385, 579));
        verticalLayoutCameraSkybox = new QVBoxLayout(scrollAreaCameraSkyboxWidgetContents);
        verticalLayoutCameraSkybox->setObjectName("verticalLayoutCameraSkybox");
        verticalLayoutCameraSkybox->setContentsMargins(0, 0, 0, 0);
        cameraTerrain = new QGroupBox(scrollAreaCameraSkyboxWidgetContents);
        cameraTerrain->setObjectName("cameraTerrain");
        gridLayout = new QGridLayout(cameraTerrain);
        gridLayout->setObjectName("gridLayout");
        labelCameraProjectionType = new QLabel(cameraTerrain);
        labelCameraProjectionType->setObjectName("labelCameraProjectionType");

        gridLayout->addWidget(labelCameraProjectionType, 1, 0, 1, 1);

        labelFieldofView = new QLabel(cameraTerrain);
        labelFieldofView->setObjectName("labelFieldofView");

        gridLayout->addWidget(labelFieldofView, 2, 0, 1, 1);

        cboCameraProjectionType = new QComboBox(cameraTerrain);
        cboCameraProjectionType->setObjectName("cboCameraProjectionType");

        gridLayout->addWidget(cboCameraProjectionType, 1, 1, 1, 2);

        labelCameraNavigationMode = new QLabel(cameraTerrain);
        labelCameraNavigationMode->setObjectName("labelCameraNavigationMode");

        gridLayout->addWidget(labelCameraNavigationMode, 3, 0, 1, 1);

        mCameraNavigationModeCombo = new QComboBox(cameraTerrain);
        mCameraNavigationModeCombo->setObjectName("mCameraNavigationModeCombo");

        gridLayout->addWidget(mCameraNavigationModeCombo, 3, 1, 1, 2);

        mCameraMovementSpeed = new QgsDoubleSpinBox(cameraTerrain);
        mCameraMovementSpeed->setObjectName("mCameraMovementSpeed");
        mCameraMovementSpeed->setMinimum(0.010000000000000);
        mCameraMovementSpeed->setSingleStep(1.000000000000000);
        mCameraMovementSpeed->setValue(5.000000000000000);

        gridLayout->addWidget(mCameraMovementSpeed, 4, 1, 1, 2);

        labelMovementSpeed = new QLabel(cameraTerrain);
        labelMovementSpeed->setObjectName("labelMovementSpeed");

        gridLayout->addWidget(labelMovementSpeed, 4, 0, 1, 1);

        spinCameraFieldOfView = new QgsDoubleSpinBox(cameraTerrain);
        spinCameraFieldOfView->setObjectName("spinCameraFieldOfView");
        spinCameraFieldOfView->setMaximum(180.000000000000000);

        gridLayout->addWidget(spinCameraFieldOfView, 2, 1, 1, 2);


        verticalLayoutCameraSkybox->addWidget(cameraTerrain);

        mGroupBox3dAxis = new QGroupBox(scrollAreaCameraSkyboxWidgetContents);
        mGroupBox3dAxis->setObjectName("mGroupBox3dAxis");
        mGroupBox3dAxis->setCheckable(true);
        formLayout_2 = new QFormLayout(mGroupBox3dAxis);
        formLayout_2->setObjectName("formLayout_2");
        mLabel3dAxisType_2 = new QLabel(mGroupBox3dAxis);
        mLabel3dAxisType_2->setObjectName("mLabel3dAxisType_2");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, mLabel3dAxisType_2);

        mCbo3dAxisType = new QComboBox(mGroupBox3dAxis);
        mCbo3dAxisType->setObjectName("mCbo3dAxisType");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::FieldRole, mCbo3dAxisType);

        mLabel3dAxisHorizPos_2 = new QLabel(mGroupBox3dAxis);
        mLabel3dAxisHorizPos_2->setObjectName("mLabel3dAxisHorizPos_2");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::LabelRole, mLabel3dAxisHorizPos_2);

        mCbo3dAxisHorizPos = new QComboBox(mGroupBox3dAxis);
        mCbo3dAxisHorizPos->setObjectName("mCbo3dAxisHorizPos");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::FieldRole, mCbo3dAxisHorizPos);

        mLabel3dAxisVertPos_2 = new QLabel(mGroupBox3dAxis);
        mLabel3dAxisVertPos_2->setObjectName("mLabel3dAxisVertPos_2");

        formLayout_2->setWidget(2, QFormLayout::ItemRole::LabelRole, mLabel3dAxisVertPos_2);

        mCbo3dAxisVertPos = new QComboBox(mGroupBox3dAxis);
        mCbo3dAxisVertPos->setObjectName("mCbo3dAxisVertPos");

        formLayout_2->setWidget(2, QFormLayout::ItemRole::FieldRole, mCbo3dAxisVertPos);


        verticalLayoutCameraSkybox->addWidget(mGroupBox3dAxis);

        groupBox = new QGroupBox(scrollAreaCameraSkyboxWidgetContents);
        groupBox->setObjectName("groupBox");
        gridLayout_8 = new QGridLayout(groupBox);
        gridLayout_8->setObjectName("gridLayout_8");
        mSync2DTo3DCheckbox = new QCheckBox(groupBox);
        mSync2DTo3DCheckbox->setObjectName("mSync2DTo3DCheckbox");

        gridLayout_8->addWidget(mSync2DTo3DCheckbox, 0, 0, 1, 1);

        mSync3DTo2DCheckbox = new QCheckBox(groupBox);
        mSync3DTo2DCheckbox->setObjectName("mSync3DTo2DCheckbox");

        gridLayout_8->addWidget(mSync3DTo2DCheckbox, 1, 0, 1, 1);

        mVisualizeExtentCheckBox = new QCheckBox(groupBox);
        mVisualizeExtentCheckBox->setObjectName("mVisualizeExtentCheckBox");

        gridLayout_8->addWidget(mVisualizeExtentCheckBox, 2, 0, 1, 1);


        verticalLayoutCameraSkybox->addWidget(groupBox);

        mShowMapOverlayCheckBox = new QCheckBox(scrollAreaCameraSkyboxWidgetContents);
        mShowMapOverlayCheckBox->setObjectName("mShowMapOverlayCheckBox");
        mShowMapOverlayCheckBox->setStyleSheet(QString::fromUtf8("font-weight: bold"));

        verticalLayoutCameraSkybox->addWidget(mShowMapOverlayCheckBox);

        verticalSpacerCameraSkybox = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayoutCameraSkybox->addItem(verticalSpacerCameraSkybox);

        scrollAreaCameraSkybox->setWidget(scrollAreaCameraSkyboxWidgetContents);

        verticalLayout_62->addWidget(scrollAreaCameraSkybox);

        m3DOptionsStackedWidget->addWidget(mPageCameraSkybox);
        mPageAdvanced = new QWidget();
        mPageAdvanced->setObjectName("mPageAdvanced");
        verticalLayout_63 = new QVBoxLayout(mPageAdvanced);
        verticalLayout_63->setObjectName("verticalLayout_63");
        verticalLayout_63->setContentsMargins(-1, 0, 0, 0);
        scrollAreaAdvanced = new QgsScrollArea(mPageAdvanced);
        scrollAreaAdvanced->setObjectName("scrollAreaAdvanced");
        scrollAreaAdvanced->setFrameShape(QFrame::Shape::NoFrame);
        scrollAreaAdvanced->setWidgetResizable(true);
        scrollAreaAdvancedWidgetContents = new QWidget();
        scrollAreaAdvancedWidgetContents->setObjectName("scrollAreaAdvancedWidgetContents");
        scrollAreaAdvancedWidgetContents->setGeometry(QRect(0, 0, 359, 312));
        verticalLayoutAdvanced = new QVBoxLayout(scrollAreaAdvancedWidgetContents);
        verticalLayoutAdvanced->setObjectName("verticalLayoutAdvanced");
        verticalLayoutAdvanced->setContentsMargins(0, 0, 0, 0);
        groupAdvanced = new QGroupBox(scrollAreaAdvancedWidgetContents);
        groupAdvanced->setObjectName("groupAdvanced");
        gridLayout_11 = new QGridLayout(groupAdvanced);
        gridLayout_11->setObjectName("gridLayout_11");
        gridLayoutAdvanced = new QGridLayout();
        gridLayoutAdvanced->setObjectName("gridLayoutAdvanced");
        chkShowLabels = new QCheckBox(groupAdvanced);
        chkShowLabels->setObjectName("chkShowLabels");

        gridLayoutAdvanced->addWidget(chkShowLabels, 6, 0, 1, 2);

        spinMapResolution = new QgsSpinBox(groupAdvanced);
        spinMapResolution->setObjectName("spinMapResolution");
        spinMapResolution->setMaximum(4096);

        gridLayoutAdvanced->addWidget(spinMapResolution, 0, 1, 1, 1);

        labelZoomLevels = new QLabel(groupAdvanced);
        labelZoomLevels->setObjectName("labelZoomLevels");

        gridLayoutAdvanced->addWidget(labelZoomLevels, 3, 1, 1, 1);

        label_5 = new QLabel(groupAdvanced);
        label_5->setObjectName("label_5");

        gridLayoutAdvanced->addWidget(label_5, 1, 0, 1, 1);

        spinGroundError = new QgsDoubleSpinBox(groupAdvanced);
        spinGroundError->setObjectName("spinGroundError");
        spinGroundError->setDecimals(1);
        spinGroundError->setMinimum(0.100000000000000);
        spinGroundError->setMaximum(1000.000000000000000);
        spinGroundError->setValue(1.000000000000000);

        gridLayoutAdvanced->addWidget(spinGroundError, 2, 1, 1, 1);

        label_7 = new QLabel(groupAdvanced);
        label_7->setObjectName("label_7");

        gridLayoutAdvanced->addWidget(label_7, 3, 0, 1, 1);

        mFpsCounterCheckBox = new QCheckBox(groupAdvanced);
        mFpsCounterCheckBox->setObjectName("mFpsCounterCheckBox");

        gridLayoutAdvanced->addWidget(mFpsCounterCheckBox, 7, 0, 1, 2);

        label_6 = new QLabel(groupAdvanced);
        label_6->setObjectName("label_6");

        gridLayoutAdvanced->addWidget(label_6, 2, 0, 1, 1);

        spinScreenError = new QgsDoubleSpinBox(groupAdvanced);
        spinScreenError->setObjectName("spinScreenError");
        spinScreenError->setDecimals(1);

        gridLayoutAdvanced->addWidget(spinScreenError, 1, 1, 1, 1);

        label_4 = new QLabel(groupAdvanced);
        label_4->setObjectName("label_4");

        gridLayoutAdvanced->addWidget(label_4, 0, 0, 1, 1);

        chkShowDebugPanel = new QCheckBox(groupAdvanced);
        chkShowDebugPanel->setObjectName("chkShowDebugPanel");

        gridLayoutAdvanced->addWidget(chkShowDebugPanel, 8, 0, 1, 2);


        gridLayout_11->addLayout(gridLayoutAdvanced, 0, 0, 1, 1);

        verticalSpacerAdvanced = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_11->addItem(verticalSpacerAdvanced, 2, 0, 1, 1);


        verticalLayoutAdvanced->addWidget(groupAdvanced);

        scrollAreaAdvanced->setWidget(scrollAreaAdvancedWidgetContents);

        verticalLayout_63->addWidget(scrollAreaAdvanced);

        m3DOptionsStackedWidget->addWidget(mPageAdvanced);

        verticalLayout_3->addWidget(m3DOptionsStackedWidget);

        m3DOptionsSplitter->addWidget(m3DOptionGroupsFrame);

        verticalLayout_4->addWidget(m3DOptionsSplitter);

        verticalLayout_4->setStretch(1, 1);
        QWidget::setTabOrder(m3DOptionsListWidget, scrollAreaTerrain);
        QWidget::setTabOrder(scrollAreaTerrain, cboTerrainType);
        QWidget::setTabOrder(cboTerrainType, cboTerrainLayer);
        QWidget::setTabOrder(cboTerrainLayer, spinTerrainScale);
        QWidget::setTabOrder(spinTerrainScale, spinTerrainResolution);
        QWidget::setTabOrder(spinTerrainResolution, spinTerrainSkirtHeight);
        QWidget::setTabOrder(spinTerrainSkirtHeight, terrainElevationOffsetSpinBox);
        QWidget::setTabOrder(terrainElevationOffsetSpinBox, groupTerrainShading);
        QWidget::setTabOrder(groupTerrainShading, scrollAreaLight);
        QWidget::setTabOrder(scrollAreaLight, scrollAreaShadow);
        QWidget::setTabOrder(scrollAreaShadow, groupShadowRendering);
        QWidget::setTabOrder(groupShadowRendering, scrollAreaCameraSkybox);
        QWidget::setTabOrder(scrollAreaCameraSkybox, cboCameraProjectionType);
        QWidget::setTabOrder(cboCameraProjectionType, spinCameraFieldOfView);
        QWidget::setTabOrder(spinCameraFieldOfView, mCameraNavigationModeCombo);
        QWidget::setTabOrder(mCameraNavigationModeCombo, mCameraMovementSpeed);
        QWidget::setTabOrder(mCameraMovementSpeed, scrollAreaAdvanced);
        QWidget::setTabOrder(scrollAreaAdvanced, spinMapResolution);
        QWidget::setTabOrder(spinMapResolution, spinScreenError);
        QWidget::setTabOrder(spinScreenError, spinGroundError);
        QWidget::setTabOrder(spinGroundError, chkShowLabels);
        QWidget::setTabOrder(chkShowLabels, mFpsCounterCheckBox);
        QWidget::setTabOrder(mFpsCounterCheckBox, edlGroupBox);
        QWidget::setTabOrder(edlGroupBox, edlStrengthSpinBox);
        QWidget::setTabOrder(edlStrengthSpinBox, edlDistanceSpinBox);

        retranslateUi(Map3DConfigWidget);

        m3DOptionsStackedWidget->setCurrentIndex(0);
        stackedWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(Map3DConfigWidget);
    } // setupUi

    void retranslateUi(QWidget *Map3DConfigWidget)
    {
        Map3DConfigWidget->setWindowTitle(QCoreApplication::translate("Map3DConfigWidget", "Configure 3D Map Rendering", nullptr));

        const bool __sortingEnabled = m3DOptionsListWidget->isSortingEnabled();
        m3DOptionsListWidget->setSortingEnabled(false);
        QListWidgetItem *___qlistwidgetitem = m3DOptionsListWidget->item(0);
        ___qlistwidgetitem->setText(QCoreApplication::translate("Map3DConfigWidget", "General", nullptr));
#if QT_CONFIG(tooltip)
        ___qlistwidgetitem->setToolTip(QCoreApplication::translate("Map3DConfigWidget", "General settings", nullptr));
#endif // QT_CONFIG(tooltip)
        QListWidgetItem *___qlistwidgetitem1 = m3DOptionsListWidget->item(1);
        ___qlistwidgetitem1->setText(QCoreApplication::translate("Map3DConfigWidget", "Terrain", nullptr));
#if QT_CONFIG(tooltip)
        ___qlistwidgetitem1->setToolTip(QCoreApplication::translate("Map3DConfigWidget", "Terrain settings", nullptr));
#endif // QT_CONFIG(tooltip)
        QListWidgetItem *___qlistwidgetitem2 = m3DOptionsListWidget->item(2);
        ___qlistwidgetitem2->setText(QCoreApplication::translate("Map3DConfigWidget", "Lights", nullptr));
#if QT_CONFIG(tooltip)
        ___qlistwidgetitem2->setToolTip(QCoreApplication::translate("Map3DConfigWidget", "Lights settings", nullptr));
#endif // QT_CONFIG(tooltip)
        QListWidgetItem *___qlistwidgetitem3 = m3DOptionsListWidget->item(3);
        ___qlistwidgetitem3->setText(QCoreApplication::translate("Map3DConfigWidget", "Effects", nullptr));
#if QT_CONFIG(tooltip)
        ___qlistwidgetitem3->setToolTip(QCoreApplication::translate("Map3DConfigWidget", "Effects settings", nullptr));
#endif // QT_CONFIG(tooltip)
        QListWidgetItem *___qlistwidgetitem4 = m3DOptionsListWidget->item(4);
        ___qlistwidgetitem4->setText(QCoreApplication::translate("Map3DConfigWidget", "Camera", nullptr));
#if QT_CONFIG(tooltip)
        ___qlistwidgetitem4->setToolTip(QCoreApplication::translate("Map3DConfigWidget", "Camera and skybox settings", nullptr));
#endif // QT_CONFIG(tooltip)
        QListWidgetItem *___qlistwidgetitem5 = m3DOptionsListWidget->item(5);
        ___qlistwidgetitem5->setText(QCoreApplication::translate("Map3DConfigWidget", "Advanced", nullptr));
#if QT_CONFIG(tooltip)
        ___qlistwidgetitem5->setToolTip(QCoreApplication::translate("Map3DConfigWidget", "Advanced settings", nullptr));
#endif // QT_CONFIG(tooltip)
        m3DOptionsListWidget->setSortingEnabled(__sortingEnabled);

        groupExtent->setTitle(QCoreApplication::translate("Map3DConfigWidget", "Extent", nullptr));
        groupBoxBackground->setTitle(QCoreApplication::translate("Map3DConfigWidget", "Background", nullptr));
        labelBottomColor->setText(QCoreApplication::translate("Map3DConfigWidget", "Bottom color", nullptr));
        labelTopColor->setText(QCoreApplication::translate("Map3DConfigWidget", "Top color", nullptr));
        groupTerrain->setTitle(QCoreApplication::translate("Map3DConfigWidget", "Terrain", nullptr));
        labelTerrainResolution->setText(QCoreApplication::translate("Map3DConfigWidget", "Tile resolution", nullptr));
        spinTerrainSkirtHeight->setSuffix(QCoreApplication::translate("Map3DConfigWidget", " map units", nullptr));
        labelTerrainSkirtHeight->setText(QCoreApplication::translate("Map3DConfigWidget", "Skirt height", nullptr));
        spinTerrainResolution->setSuffix(QCoreApplication::translate("Map3DConfigWidget", " px", nullptr));
        labelTerrainLayer->setText(QCoreApplication::translate("Map3DConfigWidget", "Elevation", nullptr));
        labelTerrainType->setText(QCoreApplication::translate("Map3DConfigWidget", "Type", nullptr));
        labelTerrainScale->setText(QCoreApplication::translate("Map3DConfigWidget", "Vertical scale", nullptr));
        labelterrainElevationOffset->setText(QCoreApplication::translate("Map3DConfigWidget", "Offset", nullptr));
        groupTerrainShading->setTitle(QCoreApplication::translate("Map3DConfigWidget", "Terrain Shading", nullptr));
        groupLights->setTitle(QCoreApplication::translate("Map3DConfigWidget", "Lights", nullptr));
        groupShadowRendering->setTitle(QCoreApplication::translate("Map3DConfigWidget", "Show Shadows", nullptr));
        edlGroupBox->setTitle(QCoreApplication::translate("Map3DConfigWidget", "Show Eye Dome Lighting", nullptr));
        label->setText(QCoreApplication::translate("Map3DConfigWidget", "Lighting strength", nullptr));
        label_2->setText(QCoreApplication::translate("Map3DConfigWidget", "Lighting distance", nullptr));
        mBloomGroupBox->setTitle(QCoreApplication::translate("Map3DConfigWidget", "Show Bloom Lighting Effect", nullptr));
        label_9->setText(QCoreApplication::translate("Map3DConfigWidget", "Strength", nullptr));
#if QT_CONFIG(tooltip)
        mSpinBloomIntensity->setToolTip(QCoreApplication::translate("Map3DConfigWidget", "Controls the overall intensity of the bloom lighting effect", nullptr));
#endif // QT_CONFIG(tooltip)
        label_10->setText(QCoreApplication::translate("Map3DConfigWidget", "Radius", nullptr));
#if QT_CONFIG(tooltip)
        mSpinBloomRadius->setToolTip(QCoreApplication::translate("Map3DConfigWidget", "Controls the overall size of the bloom lighting effect", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBox_2->setTitle(QCoreApplication::translate("Map3DConfigWidget", "Color Grading", nullptr));
#if QT_CONFIG(tooltip)
        label_8->setToolTip(QCoreApplication::translate("Map3DConfigWidget", "Adjusts the overall brightness of the 3D scene. A value of 0.0 leaves the original brightness unchanged. Positive values will brighten the scene, while negative values will darken it.", nullptr));
#endif // QT_CONFIG(tooltip)
        label_8->setText(QCoreApplication::translate("Map3DConfigWidget", "Exposure adjustment", nullptr));
        label_3->setText(QCoreApplication::translate("Map3DConfigWidget", "Tone mapping", nullptr));
#if QT_CONFIG(tooltip)
        mComboToneMapping->setToolTip(QCoreApplication::translate("Map3DConfigWidget", "Determines how extremely bright, High Dynamic Range (HDR) colors are displayed on a standard monitor.", nullptr));
#endif // QT_CONFIG(tooltip)
        cameraTerrain->setTitle(QCoreApplication::translate("Map3DConfigWidget", "Camera", nullptr));
        labelCameraProjectionType->setText(QCoreApplication::translate("Map3DConfigWidget", "Projection type", nullptr));
        labelFieldofView->setText(QCoreApplication::translate("Map3DConfigWidget", "Field of View", nullptr));
        labelCameraNavigationMode->setText(QCoreApplication::translate("Map3DConfigWidget", "Navigation mode", nullptr));
        labelMovementSpeed->setText(QCoreApplication::translate("Map3DConfigWidget", "Movement speed", nullptr));
        spinCameraFieldOfView->setSuffix(QCoreApplication::translate("Map3DConfigWidget", "\302\260", nullptr));
        mGroupBox3dAxis->setTitle(QCoreApplication::translate("Map3DConfigWidget", "Show 3D Axis", nullptr));
        mLabel3dAxisType_2->setText(QCoreApplication::translate("Map3DConfigWidget", "Axis type", nullptr));
        mLabel3dAxisHorizPos_2->setText(QCoreApplication::translate("Map3DConfigWidget", "Horizontal position", nullptr));
        mLabel3dAxisVertPos_2->setText(QCoreApplication::translate("Map3DConfigWidget", "Vertical position", nullptr));
        groupBox->setTitle(QCoreApplication::translate("Map3DConfigWidget", "Navigation Synchronization", nullptr));
        mSync2DTo3DCheckbox->setText(QCoreApplication::translate("Map3DConfigWidget", "2D map view follows 3D camera", nullptr));
        mSync3DTo2DCheckbox->setText(QCoreApplication::translate("Map3DConfigWidget", "3D camera follows 2D map view", nullptr));
        mVisualizeExtentCheckBox->setText(QCoreApplication::translate("Map3DConfigWidget", "Show visible camera area in 2D map view", nullptr));
        mShowMapOverlayCheckBox->setText(QCoreApplication::translate("Map3DConfigWidget", "Show 2D map overlay", nullptr));
        groupAdvanced->setTitle(QCoreApplication::translate("Map3DConfigWidget", "Advanced Settings", nullptr));
        chkShowLabels->setText(QCoreApplication::translate("Map3DConfigWidget", "Show labels", nullptr));
        spinMapResolution->setSuffix(QCoreApplication::translate("Map3DConfigWidget", " px", nullptr));
        labelZoomLevels->setText(QCoreApplication::translate("Map3DConfigWidget", "0", nullptr));
        label_5->setText(QCoreApplication::translate("Map3DConfigWidget", "Max. screen error", nullptr));
        spinGroundError->setSuffix(QCoreApplication::translate("Map3DConfigWidget", " map units", nullptr));
        label_7->setText(QCoreApplication::translate("Map3DConfigWidget", "Zoom levels", nullptr));
        mFpsCounterCheckBox->setText(QCoreApplication::translate("Map3DConfigWidget", "Show frames per second (FPS)", nullptr));
        label_6->setText(QCoreApplication::translate("Map3DConfigWidget", "Max. ground error", nullptr));
        spinScreenError->setSuffix(QCoreApplication::translate("Map3DConfigWidget", " px", nullptr));
        label_4->setText(QCoreApplication::translate("Map3DConfigWidget", "Map tile resolution", nullptr));
        chkShowDebugPanel->setText(QCoreApplication::translate("Map3DConfigWidget", "Show debug panel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Map3DConfigWidget: public Ui_Map3DConfigWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MAP3DCONFIGWIDGET_H
