/********************************************************************************
** Form generated from reading UI file 'qgsnewauxiliaryfielddialogbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QGSNEWAUXILIARYFIELDDIALOGBASE_H
#define UI_QGSNEWAUXILIARYFIELDDIALOGBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_QgsNewAuxiliaryFieldDialogBase
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label_3;
    QFormLayout *formLayout;
    QLabel *label;
    QComboBox *mType;
    QLabel *label_2;
    QLineEdit *mName;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *QgsNewAuxiliaryFieldDialogBase)
    {
        if (QgsNewAuxiliaryFieldDialogBase->objectName().isEmpty())
            QgsNewAuxiliaryFieldDialogBase->setObjectName("QgsNewAuxiliaryFieldDialogBase");
        QgsNewAuxiliaryFieldDialogBase->resize(440, 190);
        verticalLayout = new QVBoxLayout(QgsNewAuxiliaryFieldDialogBase);
        verticalLayout->setObjectName("verticalLayout");
        label_3 = new QLabel(QgsNewAuxiliaryFieldDialogBase);
        label_3->setObjectName("label_3");

        verticalLayout->addWidget(label_3);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        label = new QLabel(QgsNewAuxiliaryFieldDialogBase);
        label->setObjectName("label");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, label);

        mType = new QComboBox(QgsNewAuxiliaryFieldDialogBase);
        mType->setObjectName("mType");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, mType);

        label_2 = new QLabel(QgsNewAuxiliaryFieldDialogBase);
        label_2->setObjectName("label_2");

        formLayout->setWidget(4, QFormLayout::ItemRole::LabelRole, label_2);

        mName = new QLineEdit(QgsNewAuxiliaryFieldDialogBase);
        mName->setObjectName("mName");

        formLayout->setWidget(4, QFormLayout::ItemRole::FieldRole, mName);


        verticalLayout->addLayout(formLayout);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(QgsNewAuxiliaryFieldDialogBase);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(QgsNewAuxiliaryFieldDialogBase);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, QgsNewAuxiliaryFieldDialogBase, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, QgsNewAuxiliaryFieldDialogBase, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(QgsNewAuxiliaryFieldDialogBase);
    } // setupUi

    void retranslateUi(QDialog *QgsNewAuxiliaryFieldDialogBase)
    {
        QgsNewAuxiliaryFieldDialogBase->setWindowTitle(QCoreApplication::translate("QgsNewAuxiliaryFieldDialogBase", "Auxiliary Storage : New Auxiliary Field", nullptr));
        label_3->setText(QCoreApplication::translate("QgsNewAuxiliaryFieldDialogBase", "New auxiliary field parameters", nullptr));
        label->setText(QCoreApplication::translate("QgsNewAuxiliaryFieldDialogBase", "Type", nullptr));
        label_2->setText(QCoreApplication::translate("QgsNewAuxiliaryFieldDialogBase", "Name", nullptr));
    } // retranslateUi

};

namespace Ui {
    class QgsNewAuxiliaryFieldDialogBase: public Ui_QgsNewAuxiliaryFieldDialogBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QGSNEWAUXILIARYFIELDDIALOGBASE_H
