/********************************************************************************
** Form generated from reading UI file 'qgsfeaturearrayuserinputwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QGSFEATUREARRAYUSERINPUTWIDGET_H
#define UI_QGSFEATUREARRAYUSERINPUTWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>
#include "qgsdoublespinbox.h"

QT_BEGIN_NAMESPACE

class Ui_QgsFeatureArrayUserInputBase
{
public:
    QGridLayout *gridLayout;
    QgsDoubleSpinBox *mFeatureSpacingSpinBox;
    QLabel *mFeatureCountLabel;
    QSpinBox *mFeatureCountSpinBox;
    QLabel *mFeatureSpacingLabel;
    QComboBox *mModeComboBox;
    QLabel *mModeLabel;

    void setupUi(QWidget *QgsFeatureArrayUserInputBase)
    {
        if (QgsFeatureArrayUserInputBase->objectName().isEmpty())
            QgsFeatureArrayUserInputBase->setObjectName("QgsFeatureArrayUserInputBase");
        QgsFeatureArrayUserInputBase->resize(312, 120);
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(QgsFeatureArrayUserInputBase->sizePolicy().hasHeightForWidth());
        QgsFeatureArrayUserInputBase->setSizePolicy(sizePolicy);
        QgsFeatureArrayUserInputBase->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(QgsFeatureArrayUserInputBase);
        gridLayout->setObjectName("gridLayout");
        mFeatureSpacingSpinBox = new QgsDoubleSpinBox(QgsFeatureArrayUserInputBase);
        mFeatureSpacingSpinBox->setObjectName("mFeatureSpacingSpinBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(mFeatureSpacingSpinBox->sizePolicy().hasHeightForWidth());
        mFeatureSpacingSpinBox->setSizePolicy(sizePolicy1);
        mFeatureSpacingSpinBox->setMinimum(0.000000000000000);
        mFeatureSpacingSpinBox->setMaximum(999999.989999999990687);

        gridLayout->addWidget(mFeatureSpacingSpinBox, 1, 2, 1, 1);

        mFeatureCountLabel = new QLabel(QgsFeatureArrayUserInputBase);
        mFeatureCountLabel->setObjectName("mFeatureCountLabel");
        mFeatureCountLabel->setEnabled(true);
        mFeatureCountLabel->setWordWrap(true);

        gridLayout->addWidget(mFeatureCountLabel, 3, 0, 1, 1);

        mFeatureCountSpinBox = new QSpinBox(QgsFeatureArrayUserInputBase);
        mFeatureCountSpinBox->setObjectName("mFeatureCountSpinBox");
        mFeatureCountSpinBox->setMaximum(200);
        mFeatureCountSpinBox->setMinimum(1);

        gridLayout->addWidget(mFeatureCountSpinBox, 3, 2, 1, 1);

        mFeatureSpacingLabel = new QLabel(QgsFeatureArrayUserInputBase);
        mFeatureSpacingLabel->setObjectName("mFeatureSpacingLabel");

        gridLayout->addWidget(mFeatureSpacingLabel, 1, 0, 1, 1);

        mModeComboBox = new QComboBox(QgsFeatureArrayUserInputBase);
        mModeComboBox->setObjectName("mModeComboBox");

        gridLayout->addWidget(mModeComboBox, 0, 2, 1, 1);

        mModeLabel = new QLabel(QgsFeatureArrayUserInputBase);
        mModeLabel->setObjectName("mModeLabel");

        gridLayout->addWidget(mModeLabel, 0, 0, 1, 1);

#if QT_CONFIG(shortcut)
        mFeatureSpacingLabel->setBuddy(mFeatureSpacingSpinBox);
#endif // QT_CONFIG(shortcut)

        retranslateUi(QgsFeatureArrayUserInputBase);

        QMetaObject::connectSlotsByName(QgsFeatureArrayUserInputBase);
    } // setupUi

    void retranslateUi(QWidget *QgsFeatureArrayUserInputBase)
    {
        mFeatureCountLabel->setText(QCoreApplication::translate("QgsFeatureArrayUserInputBase", "Number of new features", nullptr));
        mFeatureSpacingLabel->setText(QCoreApplication::translate("QgsFeatureArrayUserInputBase", "Spacing between features (map units)", nullptr));
        mModeLabel->setText(QCoreApplication::translate("QgsFeatureArrayUserInputBase", "Mode", nullptr));
        (void)QgsFeatureArrayUserInputBase;
    } // retranslateUi

};

namespace Ui {
    class QgsFeatureArrayUserInputBase: public Ui_QgsFeatureArrayUserInputBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QGSFEATUREARRAYUSERINPUTWIDGET_H
