/********************************************************************************
** Form generated from reading UI file 'qgsadvanceddigitizingfloaterbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QGSADVANCEDDIGITIZINGFLOATERBASE_H
#define UI_QGSADVANCEDDIGITIZINGFLOATERBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_QgsAdvancedDigitizingFloaterBase
{
public:
    QGridLayout *gridLayout;
    QLineEdit *mXLineEdit;
    QLabel *mBearingLabel;
    QLineEdit *mTotalAreaLineEdit;
    QLabel *mLengthMeasurementTypeLabel;
    QLabel *mDistanceLabel;
    QLineEdit *mYLineEdit;
    QLabel *mXLabel;
    QLabel *mAreaMeasurementTypeLabel;
    QLabel *mCommonAngleSnappingLabel;
    QLineEdit *mMLineEdit;
    QLabel *mAreaLabel;
    QLineEdit *mCommonAngleSnappingLineEdit;
    QLineEdit *mDistanceLineEdit;
    QLabel *mAngleLabel;
    QLabel *mMLabel;
    QLabel *mTotalLengthLabel;
    QLineEdit *mBearingLineEdit;
    QLabel *mZLabel;
    QLabel *mYLabel;
    QLineEdit *mZLineEdit;
    QLineEdit *mAngleLineEdit;
    QLineEdit *mTotalLengthLineEdit;
    QLabel *mWeightLabel;
    QLineEdit *mWeightLineEdit;

    void setupUi(QWidget *QgsAdvancedDigitizingFloaterBase)
    {
        if (QgsAdvancedDigitizingFloaterBase->objectName().isEmpty())
            QgsAdvancedDigitizingFloaterBase->setObjectName("QgsAdvancedDigitizingFloaterBase");
        QgsAdvancedDigitizingFloaterBase->resize(310, 233);
        QPalette palette;
        QBrush brush(QColor(255, 255, 255, 127));
        brush.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::Base, brush);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::Base, brush);
        QBrush brush1(QColor(240, 240, 240, 255));
        brush1.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::Base, brush1);
        QgsAdvancedDigitizingFloaterBase->setPalette(palette);
        QgsAdvancedDigitizingFloaterBase->setAutoFillBackground(true);
        QgsAdvancedDigitizingFloaterBase->setStyleSheet(QString::fromUtf8("QLineEdit{background-color: rgba(255,255,255,127)}"));
        gridLayout = new QGridLayout(QgsAdvancedDigitizingFloaterBase);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setHorizontalSpacing(2);
        gridLayout->setVerticalSpacing(0);
        gridLayout->setContentsMargins(2, 0, 0, 0);
        mXLineEdit = new QLineEdit(QgsAdvancedDigitizingFloaterBase);
        mXLineEdit->setObjectName("mXLineEdit");
        mXLineEdit->setMinimumSize(QSize(40, 0));
        mXLineEdit->setFrame(false);

        gridLayout->addWidget(mXLineEdit, 4, 3, 1, 1);

        mBearingLabel = new QLabel(QgsAdvancedDigitizingFloaterBase);
        mBearingLabel->setObjectName("mBearingLabel");

        gridLayout->addWidget(mBearingLabel, 9, 0, 1, 2);

        mTotalAreaLineEdit = new QLineEdit(QgsAdvancedDigitizingFloaterBase);
        mTotalAreaLineEdit->setObjectName("mTotalAreaLineEdit");
        mTotalAreaLineEdit->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        mTotalAreaLineEdit->setFrame(false);
        mTotalAreaLineEdit->setReadOnly(true);

        gridLayout->addWidget(mTotalAreaLineEdit, 11, 3, 1, 1);

        mLengthMeasurementTypeLabel = new QLabel(QgsAdvancedDigitizingFloaterBase);
        mLengthMeasurementTypeLabel->setObjectName("mLengthMeasurementTypeLabel");
        mLengthMeasurementTypeLabel->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        gridLayout->addWidget(mLengthMeasurementTypeLabel, 12, 0, 1, 1);

        mDistanceLabel = new QLabel(QgsAdvancedDigitizingFloaterBase);
        mDistanceLabel->setObjectName("mDistanceLabel");
        mDistanceLabel->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        gridLayout->addWidget(mDistanceLabel, 2, 0, 1, 2);

        mYLineEdit = new QLineEdit(QgsAdvancedDigitizingFloaterBase);
        mYLineEdit->setObjectName("mYLineEdit");
        mYLineEdit->setMinimumSize(QSize(40, 0));
        mYLineEdit->setFrame(false);

        gridLayout->addWidget(mYLineEdit, 5, 3, 1, 1);

        mXLabel = new QLabel(QgsAdvancedDigitizingFloaterBase);
        mXLabel->setObjectName("mXLabel");
        mXLabel->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        gridLayout->addWidget(mXLabel, 4, 0, 1, 2);

        mAreaMeasurementTypeLabel = new QLabel(QgsAdvancedDigitizingFloaterBase);
        mAreaMeasurementTypeLabel->setObjectName("mAreaMeasurementTypeLabel");
        mAreaMeasurementTypeLabel->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        gridLayout->addWidget(mAreaMeasurementTypeLabel, 11, 0, 1, 1);

        mCommonAngleSnappingLabel = new QLabel(QgsAdvancedDigitizingFloaterBase);
        mCommonAngleSnappingLabel->setObjectName("mCommonAngleSnappingLabel");

        gridLayout->addWidget(mCommonAngleSnappingLabel, 10, 0, 1, 2);

        mMLineEdit = new QLineEdit(QgsAdvancedDigitizingFloaterBase);
        mMLineEdit->setObjectName("mMLineEdit");
        mMLineEdit->setSizeIncrement(QSize(40, 0));
        mMLineEdit->setFrame(false);

        gridLayout->addWidget(mMLineEdit, 7, 3, 1, 1);

        mAreaLabel = new QLabel(QgsAdvancedDigitizingFloaterBase);
        mAreaLabel->setObjectName("mAreaLabel");
        mAreaLabel->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        gridLayout->addWidget(mAreaLabel, 11, 1, 1, 1);

        mCommonAngleSnappingLineEdit = new QLineEdit(QgsAdvancedDigitizingFloaterBase);
        mCommonAngleSnappingLineEdit->setObjectName("mCommonAngleSnappingLineEdit");
        mCommonAngleSnappingLineEdit->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        mCommonAngleSnappingLineEdit->setFrame(false);
        mCommonAngleSnappingLineEdit->setReadOnly(true);

        gridLayout->addWidget(mCommonAngleSnappingLineEdit, 10, 3, 1, 1);

        mDistanceLineEdit = new QLineEdit(QgsAdvancedDigitizingFloaterBase);
        mDistanceLineEdit->setObjectName("mDistanceLineEdit");
        mDistanceLineEdit->setFrame(false);
        mDistanceLineEdit->setAlignment(Qt::AlignmentFlag::AlignLeading|Qt::AlignmentFlag::AlignLeft|Qt::AlignmentFlag::AlignVCenter);

        gridLayout->addWidget(mDistanceLineEdit, 2, 3, 1, 1);

        mAngleLabel = new QLabel(QgsAdvancedDigitizingFloaterBase);
        mAngleLabel->setObjectName("mAngleLabel");
        mAngleLabel->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        gridLayout->addWidget(mAngleLabel, 3, 0, 1, 2);

        mMLabel = new QLabel(QgsAdvancedDigitizingFloaterBase);
        mMLabel->setObjectName("mMLabel");
        mMLabel->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        gridLayout->addWidget(mMLabel, 7, 0, 1, 2);

        mTotalLengthLabel = new QLabel(QgsAdvancedDigitizingFloaterBase);
        mTotalLengthLabel->setObjectName("mTotalLengthLabel");
        mTotalLengthLabel->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        gridLayout->addWidget(mTotalLengthLabel, 12, 1, 1, 1);

        mBearingLineEdit = new QLineEdit(QgsAdvancedDigitizingFloaterBase);
        mBearingLineEdit->setObjectName("mBearingLineEdit");
        mBearingLineEdit->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        mBearingLineEdit->setFrame(false);
        mBearingLineEdit->setReadOnly(true);

        gridLayout->addWidget(mBearingLineEdit, 9, 3, 1, 1);

        mZLabel = new QLabel(QgsAdvancedDigitizingFloaterBase);
        mZLabel->setObjectName("mZLabel");
        mZLabel->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        gridLayout->addWidget(mZLabel, 6, 0, 1, 2);

        mYLabel = new QLabel(QgsAdvancedDigitizingFloaterBase);
        mYLabel->setObjectName("mYLabel");
        mYLabel->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        gridLayout->addWidget(mYLabel, 5, 0, 1, 2);

        mZLineEdit = new QLineEdit(QgsAdvancedDigitizingFloaterBase);
        mZLineEdit->setObjectName("mZLineEdit");
        mZLineEdit->setSizeIncrement(QSize(40, 0));
        mZLineEdit->setFrame(false);

        gridLayout->addWidget(mZLineEdit, 6, 3, 1, 1);

        mAngleLineEdit = new QLineEdit(QgsAdvancedDigitizingFloaterBase);
        mAngleLineEdit->setObjectName("mAngleLineEdit");
        mAngleLineEdit->setMinimumSize(QSize(40, 0));
        mAngleLineEdit->setFrame(false);

        gridLayout->addWidget(mAngleLineEdit, 3, 3, 1, 1);

        mTotalLengthLineEdit = new QLineEdit(QgsAdvancedDigitizingFloaterBase);
        mTotalLengthLineEdit->setObjectName("mTotalLengthLineEdit");
        mTotalLengthLineEdit->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        mTotalLengthLineEdit->setFrame(false);
        mTotalLengthLineEdit->setReadOnly(true);

        gridLayout->addWidget(mTotalLengthLineEdit, 12, 3, 1, 1);

        mWeightLabel = new QLabel(QgsAdvancedDigitizingFloaterBase);
        mWeightLabel->setObjectName("mWeightLabel");
        mWeightLabel->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        gridLayout->addWidget(mWeightLabel, 8, 0, 1, 1);

        mWeightLineEdit = new QLineEdit(QgsAdvancedDigitizingFloaterBase);
        mWeightLineEdit->setObjectName("mWeightLineEdit");
        mWeightLineEdit->setFrame(false);

        gridLayout->addWidget(mWeightLineEdit, 8, 3, 1, 1);

        QWidget::setTabOrder(mDistanceLineEdit, mAngleLineEdit);
        QWidget::setTabOrder(mAngleLineEdit, mXLineEdit);
        QWidget::setTabOrder(mXLineEdit, mYLineEdit);
        QWidget::setTabOrder(mYLineEdit, mZLineEdit);
        QWidget::setTabOrder(mZLineEdit, mMLineEdit);
        QWidget::setTabOrder(mMLineEdit, mBearingLineEdit);
        QWidget::setTabOrder(mBearingLineEdit, mCommonAngleSnappingLineEdit);
        QWidget::setTabOrder(mCommonAngleSnappingLineEdit, mTotalAreaLineEdit);
        QWidget::setTabOrder(mTotalAreaLineEdit, mTotalLengthLineEdit);

        retranslateUi(QgsAdvancedDigitizingFloaterBase);

        QMetaObject::connectSlotsByName(QgsAdvancedDigitizingFloaterBase);
    } // setupUi

    void retranslateUi(QWidget *QgsAdvancedDigitizingFloaterBase)
    {
#if QT_CONFIG(tooltip)
        mXLineEdit->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        mXLineEdit->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "-", nullptr));
        mBearingLabel->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "\342\206\273", nullptr));
        mTotalAreaLineEdit->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "-", nullptr));
        mLengthMeasurementTypeLabel->setText(QString());
        mDistanceLabel->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "d", nullptr));
#if QT_CONFIG(tooltip)
        mYLineEdit->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        mYLineEdit->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "-", nullptr));
        mXLabel->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "x", nullptr));
        mAreaMeasurementTypeLabel->setText(QString());
        mCommonAngleSnappingLabel->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "\342\210\240", nullptr));
        mMLineEdit->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "-", nullptr));
        mAreaLabel->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "k", nullptr));
        mCommonAngleSnappingLineEdit->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "-", nullptr));
#if QT_CONFIG(tooltip)
        mDistanceLineEdit->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        mDistanceLineEdit->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "-", nullptr));
        mAngleLabel->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "a", nullptr));
        mMLabel->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "m", nullptr));
        mTotalLengthLabel->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "p", nullptr));
        mZLabel->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "z", nullptr));
        mYLabel->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "y", nullptr));
        mZLineEdit->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "-", nullptr));
#if QT_CONFIG(tooltip)
        mAngleLineEdit->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        mAngleLineEdit->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "-", nullptr));
        mTotalLengthLineEdit->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "-", nullptr));
        mWeightLabel->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "w", nullptr));
        mWeightLineEdit->setText(QCoreApplication::translate("QgsAdvancedDigitizingFloaterBase", "-", nullptr));
        (void)QgsAdvancedDigitizingFloaterBase;
    } // retranslateUi

};

namespace Ui {
    class QgsAdvancedDigitizingFloaterBase: public Ui_QgsAdvancedDigitizingFloaterBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QGSADVANCEDDIGITIZINGFLOATERBASE_H
