/********************************************************************************
** Form generated from reading UI file 'qgscategorized3drendererwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QGSCATEGORIZED3DRENDERERWIDGET_H
#define UI_QGSCATEGORIZED3DRENDERERWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QWidget>
#include <qgs3dsymbolbutton.h>
#include <qgsfieldexpressionwidget.h>
#include "qgscolorrampbutton.h"
#include "qgssymbolbutton.h"

QT_BEGIN_NAMESPACE

class Ui_QgsCategorized3DRendererWidget
{
public:
    QGridLayout *gridLayout;
    QLabel *label_3;
    QgsFieldExpressionWidget *mExpressionWidget;
    QLabel *symbolLabel;
    Qgs3DSymbolButton *mBtnChangeCategorizedSymbol;
    QLabel *label_10;
    QHBoxLayout *horizontalLayout;
    QgsColorRampButton *mBtnColorRamp;
    QTreeView *mViewCategories;
    QHBoxLayout *horizontalLayout_3;
    QPushButton *mBtnAddCategories;
    QPushButton *mBtnAddCategory;
    QPushButton *mBtnDeleteCategories;
    QPushButton *mBtnDeleteAllCategories;
    QPushButton *mBtnDeleteUnusedCategories;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *QgsCategorized3DRendererWidget)
    {
        if (QgsCategorized3DRendererWidget->objectName().isEmpty())
            QgsCategorized3DRendererWidget->setObjectName("QgsCategorized3DRendererWidget");
        QgsCategorized3DRendererWidget->resize(424, 368);
        gridLayout = new QGridLayout(QgsCategorized3DRendererWidget);
        gridLayout->setObjectName("gridLayout");
        label_3 = new QLabel(QgsCategorized3DRendererWidget);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        mExpressionWidget = new QgsFieldExpressionWidget(QgsCategorized3DRendererWidget);
        mExpressionWidget->setObjectName("mExpressionWidget");
        mExpressionWidget->setFocusPolicy(Qt::StrongFocus);

        gridLayout->addWidget(mExpressionWidget, 0, 1, 1, 1);

        symbolLabel = new QLabel(QgsCategorized3DRendererWidget);
        symbolLabel->setObjectName("symbolLabel");
        sizePolicy.setHeightForWidth(symbolLabel->sizePolicy().hasHeightForWidth());
        symbolLabel->setSizePolicy(sizePolicy);

        gridLayout->addWidget(symbolLabel, 1, 0, 1, 1);

        mBtnChangeCategorizedSymbol = new Qgs3DSymbolButton(QgsCategorized3DRendererWidget);
        mBtnChangeCategorizedSymbol->setObjectName("mBtnChangeCategorizedSymbol");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(mBtnChangeCategorizedSymbol->sizePolicy().hasHeightForWidth());
        mBtnChangeCategorizedSymbol->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(mBtnChangeCategorizedSymbol, 1, 1, 1, 1);

        label_10 = new QLabel(QgsCategorized3DRendererWidget);
        label_10->setObjectName("label_10");
        sizePolicy.setHeightForWidth(label_10->sizePolicy().hasHeightForWidth());
        label_10->setSizePolicy(sizePolicy);

        gridLayout->addWidget(label_10, 2, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        mBtnColorRamp = new QgsColorRampButton(QgsCategorized3DRendererWidget);
        mBtnColorRamp->setObjectName("mBtnColorRamp");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(mBtnColorRamp->sizePolicy().hasHeightForWidth());
        mBtnColorRamp->setSizePolicy(sizePolicy2);
        mBtnColorRamp->setMinimumSize(QSize(120, 0));
        mBtnColorRamp->setMaximumSize(QSize(16777215, 16777215));

        horizontalLayout->addWidget(mBtnColorRamp);


        gridLayout->addLayout(horizontalLayout, 2, 1, 1, 1);

        mViewCategories = new QTreeView(QgsCategorized3DRendererWidget);
        mViewCategories->setObjectName("mViewCategories");
        mViewCategories->setContextMenuPolicy(Qt::CustomContextMenu);
        mViewCategories->setDragDropMode(QAbstractItemView::InternalMove);
        mViewCategories->setDefaultDropAction(Qt::IgnoreAction);
        mViewCategories->setSelectionMode(QAbstractItemView::ExtendedSelection);
        mViewCategories->setIconSize(QSize(16, 16));
        mViewCategories->setRootIsDecorated(false);
        mViewCategories->setItemsExpandable(false);
        mViewCategories->setSortingEnabled(true);
        mViewCategories->setAllColumnsShowFocus(true);

        gridLayout->addWidget(mViewCategories, 3, 0, 1, 2);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        mBtnAddCategories = new QPushButton(QgsCategorized3DRendererWidget);
        mBtnAddCategories->setObjectName("mBtnAddCategories");

        horizontalLayout_3->addWidget(mBtnAddCategories);

        mBtnAddCategory = new QPushButton(QgsCategorized3DRendererWidget);
        mBtnAddCategory->setObjectName("mBtnAddCategory");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/images/themes/default/symbologyAdd.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mBtnAddCategory->setIcon(icon);

        horizontalLayout_3->addWidget(mBtnAddCategory);

        mBtnDeleteCategories = new QPushButton(QgsCategorized3DRendererWidget);
        mBtnDeleteCategories->setObjectName("mBtnDeleteCategories");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/images/themes/default/symbologyRemove.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mBtnDeleteCategories->setIcon(icon1);

        horizontalLayout_3->addWidget(mBtnDeleteCategories);

        mBtnDeleteAllCategories = new QPushButton(QgsCategorized3DRendererWidget);
        mBtnDeleteAllCategories->setObjectName("mBtnDeleteAllCategories");

        horizontalLayout_3->addWidget(mBtnDeleteAllCategories);

        mBtnDeleteUnusedCategories = new QPushButton(QgsCategorized3DRendererWidget);
        mBtnDeleteUnusedCategories->setObjectName("mBtnDeleteUnusedCategories");

        horizontalLayout_3->addWidget(mBtnDeleteUnusedCategories);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(spacerItem);


        gridLayout->addLayout(horizontalLayout_3, 4, 0, 1, 2);

        mViewCategories->raise();
        mBtnChangeCategorizedSymbol->raise();
        symbolLabel->raise();
        label_10->raise();
        mExpressionWidget->raise();
        label_3->raise();
        QWidget::setTabOrder(mExpressionWidget, mBtnChangeCategorizedSymbol);
        QWidget::setTabOrder(mBtnChangeCategorizedSymbol, mBtnColorRamp);
        QWidget::setTabOrder(mBtnColorRamp, mViewCategories);
        QWidget::setTabOrder(mViewCategories, mBtnAddCategories);
        QWidget::setTabOrder(mBtnAddCategories, mBtnAddCategory);
        QWidget::setTabOrder(mBtnAddCategory, mBtnDeleteCategories);
        QWidget::setTabOrder(mBtnDeleteCategories, mBtnDeleteAllCategories);

        retranslateUi(QgsCategorized3DRendererWidget);

        QMetaObject::connectSlotsByName(QgsCategorized3DRendererWidget);
    } // setupUi

    void retranslateUi(QWidget *QgsCategorized3DRendererWidget)
    {
        label_3->setText(QCoreApplication::translate("QgsCategorized3DRendererWidget", "Value", nullptr));
        symbolLabel->setText(QCoreApplication::translate("QgsCategorized3DRendererWidget", "Symbol", nullptr));
        mBtnChangeCategorizedSymbol->setText(QString());
        label_10->setText(QCoreApplication::translate("QgsCategorized3DRendererWidget", "Color ramp", nullptr));
        mBtnAddCategories->setText(QCoreApplication::translate("QgsCategorized3DRendererWidget", "Classify", nullptr));
#if QT_CONFIG(tooltip)
        mBtnAddCategory->setToolTip(QCoreApplication::translate("QgsCategorized3DRendererWidget", "Add", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        mBtnDeleteCategories->setToolTip(QCoreApplication::translate("QgsCategorized3DRendererWidget", "Delete", nullptr));
#endif // QT_CONFIG(tooltip)
        mBtnDeleteAllCategories->setText(QCoreApplication::translate("QgsCategorized3DRendererWidget", "Delete All", nullptr));
        mBtnDeleteUnusedCategories->setText(QCoreApplication::translate("QgsCategorized3DRendererWidget", "Delete Unused", nullptr));
        (void)QgsCategorized3DRendererWidget;
    } // retranslateUi

};

namespace Ui {
    class QgsCategorized3DRendererWidget: public Ui_QgsCategorized3DRendererWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QGSCATEGORIZED3DRENDERERWIDGET_H
