/********************************************************************************
** Form generated from reading UI file 'qgshistorywidgetbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QGSHISTORYWIDGETBASE_H
#define UI_QGSHISTORYWIDGETBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "qgsfilterlineedit.h"
#include "qgspanelwidget.h"

QT_BEGIN_NAMESPACE

class Ui_QgsHistoryWidgetBase
{
public:
    QVBoxLayout *verticalLayout_2;
    QSplitter *mSplitter;
    QWidget *layoutWidget;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QgsFilterLineEdit *mFilterEdit;
    QTreeView *mTreeView;
    QStackedWidget *mContainerStackedWidget;
    QWidget *mEntryContainerStackedWidgetPage1;

    void setupUi(QgsPanelWidget *QgsHistoryWidgetBase)
    {
        if (QgsHistoryWidgetBase->objectName().isEmpty())
            QgsHistoryWidgetBase->setObjectName("QgsHistoryWidgetBase");
        QgsHistoryWidgetBase->resize(387, 220);
        verticalLayout_2 = new QVBoxLayout(QgsHistoryWidgetBase);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        mSplitter = new QSplitter(QgsHistoryWidgetBase);
        mSplitter->setObjectName("mSplitter");
        mSplitter->setOrientation(Qt::Vertical);
        layoutWidget = new QWidget(mSplitter);
        layoutWidget->setObjectName("layoutWidget");
        verticalLayout = new QVBoxLayout(layoutWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        mFilterEdit = new QgsFilterLineEdit(layoutWidget);
        mFilterEdit->setObjectName("mFilterEdit");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(mFilterEdit->sizePolicy().hasHeightForWidth());
        mFilterEdit->setSizePolicy(sizePolicy);
        mFilterEdit->setMinimumSize(QSize(0, 0));
        mFilterEdit->setBaseSize(QSize(0, 0));

        horizontalLayout->addWidget(mFilterEdit);


        verticalLayout->addLayout(horizontalLayout);

        mTreeView = new QTreeView(layoutWidget);
        mTreeView->setObjectName("mTreeView");
        mTreeView->header()->setVisible(false);

        verticalLayout->addWidget(mTreeView);

        mSplitter->addWidget(layoutWidget);
        mContainerStackedWidget = new QStackedWidget(mSplitter);
        mContainerStackedWidget->setObjectName("mContainerStackedWidget");
        mEntryContainerStackedWidgetPage1 = new QWidget();
        mEntryContainerStackedWidgetPage1->setObjectName("mEntryContainerStackedWidgetPage1");
        mContainerStackedWidget->addWidget(mEntryContainerStackedWidgetPage1);
        mSplitter->addWidget(mContainerStackedWidget);

        verticalLayout_2->addWidget(mSplitter);


        retranslateUi(QgsHistoryWidgetBase);

        QMetaObject::connectSlotsByName(QgsHistoryWidgetBase);
    } // setupUi

    void retranslateUi(QgsPanelWidget *QgsHistoryWidgetBase)
    {
        QgsHistoryWidgetBase->setWindowTitle(QCoreApplication::translate("QgsHistoryWidgetBase", "History", nullptr));
        mFilterEdit->setPlaceholderText(QCoreApplication::translate("QgsHistoryWidgetBase", "Search", nullptr));
    } // retranslateUi

};

namespace Ui {
    class QgsHistoryWidgetBase: public Ui_QgsHistoryWidgetBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QGSHISTORYWIDGETBASE_H
