/********************************************************************************
** Form generated from reading UI file 'qgslightswidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QGSLIGHTSWIDGET_H
#define UI_QGSLIGHTSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDial>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListView>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "qgscolorbutton.h"
#include "qgsdatetimeedit.h"
#include "qgsdoublespinbox.h"

QT_BEGIN_NAMESPACE

class Ui_QgsLightsWidget
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QListView *mLightsListView;
    QVBoxLayout *verticalLayout_2;
    QToolButton *btnAddLight;
    QToolButton *btnRemoveLight;
    QSpacerItem *verticalSpacer;
    QStackedWidget *mStackedWidget;
    QWidget *page_2;
    QWidget *page;
    QGridLayout *gridLayout;
    QLabel *labelPointLightCrs;
    QLabel *label_3;
    QgsDoubleSpinBox *spinPositionX;
    QLabel *label_9;
    QgsDoubleSpinBox *spinIntensity;
    QLabel *label_7;
    QgsDoubleSpinBox *spinPositionY;
    QLabel *label_2;
    QgsDoubleSpinBox *spinPositionZ;
    QgsColorButton *btnColor;
    QgsDoubleSpinBox *spinA0;
    QgsDoubleSpinBox *spinA1;
    QgsDoubleSpinBox *spinA2;
    QLabel *label_4;
    QLabel *label_8;
    QLabel *label_5;
    QLabel *label;
    QLabel *label_6;
    QWidget *page_3;
    QGridLayout *gridLayout_3;
    QLabel *label_14;
    QHBoxLayout *directionLightLayout;
    QSpacerItem *horizontalSpacer_2;
    QLabel *label_12;
    QDoubleSpinBox *spinBoxAzimuth;
    QWidget *directionCmpassWidget;
    QDial *dialAzimuth;
    QLabel *altitudeSpinBox;
    QDoubleSpinBox *spinBoxAltitude;
    QSlider *sliderAltitude;
    QSpacerItem *horizontalSpacer;
    QgsDoubleSpinBox *spinDirectionalIntensity;
    QLabel *label_16;
    QgsColorButton *btnDirectionalColor;
    QLabel *label_11;
    QGridLayout *gridLayout_5;
    QLabel *label_13;
    QLabel *labelY;
    QLabel *label_10;
    QLabel *label_17;
    QLabel *labelX;
    QLabel *labelZ;
    QWidget *mPageSunLight;
    QGridLayout *gridLayout_2;
    QgsColorButton *mSunColorButton;
    QLabel *label_18;
    QLabel *label_21;
    QLabel *label_19;
    QgsDateTimeEdit *mSunTimeWidget;
    QLabel *label_22;
    QgsDoubleSpinBox *mSunTemperatureSpin;
    QLabel *mLabelSunTime;
    QgsDoubleSpinBox *mSunIntensitySpin;
    QLabel *label_20;
    QgsDoubleSpinBox *mSunPressureSpin;
    QgsDoubleSpinBox *mSunElevationSpin;
    QSpacerItem *verticalSpacer_2;

    void setupUi(QWidget *QgsLightsWidget)
    {
        if (QgsLightsWidget->objectName().isEmpty())
            QgsLightsWidget->setObjectName("QgsLightsWidget");
        QgsLightsWidget->resize(465, 633);
        QgsLightsWidget->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout = new QVBoxLayout(QgsLightsWidget);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(-1, 0, -1, -1);
        mLightsListView = new QListView(QgsLightsWidget);
        mLightsListView->setObjectName("mLightsListView");

        horizontalLayout_2->addWidget(mLightsListView);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, -1, -1, -1);
        btnAddLight = new QToolButton(QgsLightsWidget);
        btnAddLight->setObjectName("btnAddLight");
        btnAddLight->setPopupMode(QToolButton::ToolButtonPopupMode::InstantPopup);

        verticalLayout_2->addWidget(btnAddLight);

        btnRemoveLight = new QToolButton(QgsLightsWidget);
        btnRemoveLight->setObjectName("btnRemoveLight");

        verticalLayout_2->addWidget(btnRemoveLight);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        horizontalLayout_2->addLayout(verticalLayout_2);


        verticalLayout->addLayout(horizontalLayout_2);

        mStackedWidget = new QStackedWidget(QgsLightsWidget);
        mStackedWidget->setObjectName("mStackedWidget");
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        mStackedWidget->addWidget(page_2);
        page = new QWidget();
        page->setObjectName("page");
        gridLayout = new QGridLayout(page);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        labelPointLightCrs = new QLabel(page);
        labelPointLightCrs->setObjectName("labelPointLightCrs");

        gridLayout->addWidget(labelPointLightCrs, 0, 0, 1, 2);

        label_3 = new QLabel(page);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 4, 0, 1, 1);

        spinPositionX = new QgsDoubleSpinBox(page);
        spinPositionX->setObjectName("spinPositionX");
        spinPositionX->setDecimals(1);
        spinPositionX->setMinimum(-99999999.000000000000000);
        spinPositionX->setMaximum(99999999.000000000000000);

        gridLayout->addWidget(spinPositionX, 1, 1, 1, 1);

        label_9 = new QLabel(page);
        label_9->setObjectName("label_9");

        gridLayout->addWidget(label_9, 10, 0, 1, 1);

        spinIntensity = new QgsDoubleSpinBox(page);
        spinIntensity->setObjectName("spinIntensity");
        spinIntensity->setDecimals(2);
        spinIntensity->setMaximum(999999.000000000000000);
        spinIntensity->setSingleStep(0.100000000000000);

        gridLayout->addWidget(spinIntensity, 6, 1, 1, 1);

        label_7 = new QLabel(page);
        label_7->setObjectName("label_7");

        gridLayout->addWidget(label_7, 8, 0, 1, 1);

        spinPositionY = new QgsDoubleSpinBox(page);
        spinPositionY->setObjectName("spinPositionY");
        spinPositionY->setDecimals(1);
        spinPositionY->setMinimum(-99999999.000000000000000);
        spinPositionY->setMaximum(99999999.000000000000000);

        gridLayout->addWidget(spinPositionY, 2, 1, 1, 1);

        label_2 = new QLabel(page);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 2, 0, 1, 1);

        spinPositionZ = new QgsDoubleSpinBox(page);
        spinPositionZ->setObjectName("spinPositionZ");
        spinPositionZ->setDecimals(1);
        spinPositionZ->setMinimum(-99999999.000000000000000);
        spinPositionZ->setMaximum(99999999.000000000000000);

        gridLayout->addWidget(spinPositionZ, 4, 1, 1, 1);

        btnColor = new QgsColorButton(page);
        btnColor->setObjectName("btnColor");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(btnColor->sizePolicy().hasHeightForWidth());
        btnColor->setSizePolicy(sizePolicy);
        btnColor->setMinimumSize(QSize(120, 0));

        gridLayout->addWidget(btnColor, 5, 1, 1, 1);

        spinA0 = new QgsDoubleSpinBox(page);
        spinA0->setObjectName("spinA0");
        spinA0->setMaximum(9.000000000000000);
        spinA0->setSingleStep(0.100000000000000);

        gridLayout->addWidget(spinA0, 8, 1, 1, 1);

        spinA1 = new QgsDoubleSpinBox(page);
        spinA1->setObjectName("spinA1");
        spinA1->setMaximum(9.000000000000000);
        spinA1->setSingleStep(0.100000000000000);

        gridLayout->addWidget(spinA1, 9, 1, 1, 1);

        spinA2 = new QgsDoubleSpinBox(page);
        spinA2->setObjectName("spinA2");
        spinA2->setMaximum(9.000000000000000);
        spinA2->setSingleStep(0.100000000000000);

        gridLayout->addWidget(spinA2, 10, 1, 1, 1);

        label_4 = new QLabel(page);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 6, 0, 1, 1);

        label_8 = new QLabel(page);
        label_8->setObjectName("label_8");

        gridLayout->addWidget(label_8, 9, 0, 1, 1);

        label_5 = new QLabel(page);
        label_5->setObjectName("label_5");

        gridLayout->addWidget(label_5, 5, 0, 1, 1);

        label = new QLabel(page);
        label->setObjectName("label");

        gridLayout->addWidget(label, 1, 0, 1, 1);

        label_6 = new QLabel(page);
        label_6->setObjectName("label_6");

        gridLayout->addWidget(label_6, 7, 0, 1, 2);

        mStackedWidget->addWidget(page);
        page_3 = new QWidget();
        page_3->setObjectName("page_3");
        gridLayout_3 = new QGridLayout(page_3);
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        label_14 = new QLabel(page_3);
        label_14->setObjectName("label_14");

        gridLayout_3->addWidget(label_14, 7, 0, 1, 1);

        directionLightLayout = new QHBoxLayout();
        directionLightLayout->setObjectName("directionLightLayout");
        directionLightLayout->setContentsMargins(-1, 11, -1, -1);
        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        directionLightLayout->addItem(horizontalSpacer_2);

        label_12 = new QLabel(page_3);
        label_12->setObjectName("label_12");

        directionLightLayout->addWidget(label_12);

        spinBoxAzimuth = new QDoubleSpinBox(page_3);
        spinBoxAzimuth->setObjectName("spinBoxAzimuth");
        spinBoxAzimuth->setMaximum(360.000000000000000);

        directionLightLayout->addWidget(spinBoxAzimuth);

        directionCmpassWidget = new QWidget(page_3);
        directionCmpassWidget->setObjectName("directionCmpassWidget");

        directionLightLayout->addWidget(directionCmpassWidget);

        dialAzimuth = new QDial(page_3);
        dialAzimuth->setObjectName("dialAzimuth");
        dialAzimuth->setMinimum(0);
        dialAzimuth->setValue(0);
        dialAzimuth->setSliderPosition(0);
        dialAzimuth->setWrapping(true);
        dialAzimuth->setNotchTarget(10.000000000000000);
        dialAzimuth->setNotchesVisible(true);

        directionLightLayout->addWidget(dialAzimuth);

        altitudeSpinBox = new QLabel(page_3);
        altitudeSpinBox->setObjectName("altitudeSpinBox");

        directionLightLayout->addWidget(altitudeSpinBox);

        spinBoxAltitude = new QDoubleSpinBox(page_3);
        spinBoxAltitude->setObjectName("spinBoxAltitude");
        spinBoxAltitude->setMinimum(-90.000000000000000);
        spinBoxAltitude->setMaximum(90.000000000000000);

        directionLightLayout->addWidget(spinBoxAltitude);

        sliderAltitude = new QSlider(page_3);
        sliderAltitude->setObjectName("sliderAltitude");
        sliderAltitude->setMinimum(0);
        sliderAltitude->setMaximum(90);
        sliderAltitude->setTracking(true);
        sliderAltitude->setOrientation(Qt::Orientation::Vertical);
        sliderAltitude->setTickPosition(QSlider::TickPosition::NoTicks);
        sliderAltitude->setTickInterval(2);

        directionLightLayout->addWidget(sliderAltitude);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        directionLightLayout->addItem(horizontalSpacer);


        gridLayout_3->addLayout(directionLightLayout, 1, 0, 1, 2);

        spinDirectionalIntensity = new QgsDoubleSpinBox(page_3);
        spinDirectionalIntensity->setObjectName("spinDirectionalIntensity");
        spinDirectionalIntensity->setDecimals(2);
        spinDirectionalIntensity->setMaximum(999999.000000000000000);
        spinDirectionalIntensity->setSingleStep(0.100000000000000);

        gridLayout_3->addWidget(spinDirectionalIntensity, 7, 1, 1, 1);

        label_16 = new QLabel(page_3);
        label_16->setObjectName("label_16");

        gridLayout_3->addWidget(label_16, 6, 0, 1, 1);

        btnDirectionalColor = new QgsColorButton(page_3);
        btnDirectionalColor->setObjectName("btnDirectionalColor");
        sizePolicy.setHeightForWidth(btnDirectionalColor->sizePolicy().hasHeightForWidth());
        btnDirectionalColor->setSizePolicy(sizePolicy);
        btnDirectionalColor->setMinimumSize(QSize(120, 0));

        gridLayout_3->addWidget(btnDirectionalColor, 6, 1, 1, 1);

        label_11 = new QLabel(page_3);
        label_11->setObjectName("label_11");

        gridLayout_3->addWidget(label_11, 0, 0, 1, 2);

        gridLayout_5 = new QGridLayout();
        gridLayout_5->setObjectName("gridLayout_5");
        gridLayout_5->setContentsMargins(-1, 0, -1, -1);
        label_13 = new QLabel(page_3);
        label_13->setObjectName("label_13");

        gridLayout_5->addWidget(label_13, 0, 2, 1, 1);

        labelY = new QLabel(page_3);
        labelY->setObjectName("labelY");

        gridLayout_5->addWidget(labelY, 0, 3, 1, 1);

        label_10 = new QLabel(page_3);
        label_10->setObjectName("label_10");

        gridLayout_5->addWidget(label_10, 0, 4, 1, 1);

        label_17 = new QLabel(page_3);
        label_17->setObjectName("label_17");

        gridLayout_5->addWidget(label_17, 0, 0, 1, 1);

        labelX = new QLabel(page_3);
        labelX->setObjectName("labelX");

        gridLayout_5->addWidget(labelX, 0, 1, 1, 1);

        labelZ = new QLabel(page_3);
        labelZ->setObjectName("labelZ");

        gridLayout_5->addWidget(labelZ, 0, 5, 1, 1);


        gridLayout_3->addLayout(gridLayout_5, 2, 1, 1, 1);

        mStackedWidget->addWidget(page_3);
        mPageSunLight = new QWidget();
        mPageSunLight->setObjectName("mPageSunLight");
        gridLayout_2 = new QGridLayout(mPageSunLight);
        gridLayout_2->setObjectName("gridLayout_2");
        mSunColorButton = new QgsColorButton(mPageSunLight);
        mSunColorButton->setObjectName("mSunColorButton");
        sizePolicy.setHeightForWidth(mSunColorButton->sizePolicy().hasHeightForWidth());
        mSunColorButton->setSizePolicy(sizePolicy);
        mSunColorButton->setMinimumSize(QSize(120, 0));

        gridLayout_2->addWidget(mSunColorButton, 1, 1, 1, 1);

        label_18 = new QLabel(mPageSunLight);
        label_18->setObjectName("label_18");

        gridLayout_2->addWidget(label_18, 2, 0, 1, 1);

        label_21 = new QLabel(mPageSunLight);
        label_21->setObjectName("label_21");

        gridLayout_2->addWidget(label_21, 4, 0, 1, 1);

        label_19 = new QLabel(mPageSunLight);
        label_19->setObjectName("label_19");

        gridLayout_2->addWidget(label_19, 1, 0, 1, 1);

        mSunTimeWidget = new QgsDateTimeEdit(mPageSunLight);
        mSunTimeWidget->setObjectName("mSunTimeWidget");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(mSunTimeWidget->sizePolicy().hasHeightForWidth());
        mSunTimeWidget->setSizePolicy(sizePolicy1);
        mSunTimeWidget->setTimeSpec(Qt::TimeSpec::LocalTime);

        gridLayout_2->addWidget(mSunTimeWidget, 0, 1, 1, 1);

        label_22 = new QLabel(mPageSunLight);
        label_22->setObjectName("label_22");

        gridLayout_2->addWidget(label_22, 5, 0, 1, 1);

        mSunTemperatureSpin = new QgsDoubleSpinBox(mPageSunLight);
        mSunTemperatureSpin->setObjectName("mSunTemperatureSpin");
        mSunTemperatureSpin->setMinimum(-273.149999999999977);
        mSunTemperatureSpin->setMaximum(1000.000000000000000);

        gridLayout_2->addWidget(mSunTemperatureSpin, 3, 1, 1, 1);

        mLabelSunTime = new QLabel(mPageSunLight);
        mLabelSunTime->setObjectName("mLabelSunTime");

        gridLayout_2->addWidget(mLabelSunTime, 0, 0, 1, 1);

        mSunIntensitySpin = new QgsDoubleSpinBox(mPageSunLight);
        mSunIntensitySpin->setObjectName("mSunIntensitySpin");
        mSunIntensitySpin->setDecimals(2);
        mSunIntensitySpin->setMaximum(999999.000000000000000);
        mSunIntensitySpin->setSingleStep(0.100000000000000);

        gridLayout_2->addWidget(mSunIntensitySpin, 2, 1, 1, 1);

        label_20 = new QLabel(mPageSunLight);
        label_20->setObjectName("label_20");

        gridLayout_2->addWidget(label_20, 3, 0, 1, 1);

        mSunPressureSpin = new QgsDoubleSpinBox(mPageSunLight);
        mSunPressureSpin->setObjectName("mSunPressureSpin");
        mSunPressureSpin->setMaximum(5000.000000000000000);

        gridLayout_2->addWidget(mSunPressureSpin, 4, 1, 1, 1);

        mSunElevationSpin = new QgsDoubleSpinBox(mPageSunLight);
        mSunElevationSpin->setObjectName("mSunElevationSpin");
        mSunElevationSpin->setDecimals(0);
        mSunElevationSpin->setMaximum(9999.000000000000000);

        gridLayout_2->addWidget(mSunElevationSpin, 5, 1, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer_2, 6, 0, 1, 1);

        mStackedWidget->addWidget(mPageSunLight);

        verticalLayout->addWidget(mStackedWidget);

        QWidget::setTabOrder(mLightsListView, btnAddLight);
        QWidget::setTabOrder(btnAddLight, btnRemoveLight);
        QWidget::setTabOrder(btnRemoveLight, spinPositionX);
        QWidget::setTabOrder(spinPositionX, spinPositionY);
        QWidget::setTabOrder(spinPositionY, spinPositionZ);
        QWidget::setTabOrder(spinPositionZ, btnColor);
        QWidget::setTabOrder(btnColor, spinIntensity);
        QWidget::setTabOrder(spinIntensity, spinA0);
        QWidget::setTabOrder(spinA0, spinA1);
        QWidget::setTabOrder(spinA1, spinA2);
        QWidget::setTabOrder(spinA2, dialAzimuth);
        QWidget::setTabOrder(dialAzimuth, spinBoxAzimuth);
        QWidget::setTabOrder(spinBoxAzimuth, spinBoxAltitude);
        QWidget::setTabOrder(spinBoxAltitude, sliderAltitude);
        QWidget::setTabOrder(sliderAltitude, btnDirectionalColor);
        QWidget::setTabOrder(btnDirectionalColor, spinDirectionalIntensity);
        QWidget::setTabOrder(spinDirectionalIntensity, mSunTimeWidget);
        QWidget::setTabOrder(mSunTimeWidget, mSunColorButton);
        QWidget::setTabOrder(mSunColorButton, mSunIntensitySpin);
        QWidget::setTabOrder(mSunIntensitySpin, mSunTemperatureSpin);
        QWidget::setTabOrder(mSunTemperatureSpin, mSunPressureSpin);
        QWidget::setTabOrder(mSunPressureSpin, mSunElevationSpin);

        retranslateUi(QgsLightsWidget);

        mStackedWidget->setCurrentIndex(3);


        QMetaObject::connectSlotsByName(QgsLightsWidget);
    } // setupUi

    void retranslateUi(QWidget *QgsLightsWidget)
    {
        btnAddLight->setText(QCoreApplication::translate("QgsLightsWidget", "...", nullptr));
        btnRemoveLight->setText(QCoreApplication::translate("QgsLightsWidget", "...", nullptr));
        labelPointLightCrs->setText(QCoreApplication::translate("QgsLightsWidget", "Coordinates in 3D map CRS", nullptr));
        label_3->setText(QCoreApplication::translate("QgsLightsWidget", "Z", nullptr));
        label_9->setText(QCoreApplication::translate("QgsLightsWidget", "<html><head/><body><p>A<span style=\" vertical-align:sub;\">2</span></p></body></html>", nullptr));
        label_7->setText(QCoreApplication::translate("QgsLightsWidget", "<html><head/><body><p>A<span style=\" vertical-align:sub;\">0</span></p></body></html>", nullptr));
        label_2->setText(QCoreApplication::translate("QgsLightsWidget", "Y", nullptr));
        label_4->setText(QCoreApplication::translate("QgsLightsWidget", "Intensity", nullptr));
        label_8->setText(QCoreApplication::translate("QgsLightsWidget", "<html><head/><body><p>A<span style=\" vertical-align:sub;\">1</span></p></body></html>", nullptr));
        label_5->setText(QCoreApplication::translate("QgsLightsWidget", "Color", nullptr));
        label->setText(QCoreApplication::translate("QgsLightsWidget", "X", nullptr));
        label_6->setText(QCoreApplication::translate("QgsLightsWidget", "<html><head/><body><p>Attenuation (A<span style=\" vertical-align:sub;\">0</span>+A<span style=\" vertical-align:sub;\">1</span>&times;D+A<span style=\" vertical-align:sub;\">2</span>&times;D<span style=\" vertical-align:super;\">2</span>)</p></body></html>", nullptr));
        label_14->setText(QCoreApplication::translate("QgsLightsWidget", "Intensity", nullptr));
        label_12->setText(QCoreApplication::translate("QgsLightsWidget", "Azimuth", nullptr));
        spinBoxAzimuth->setSuffix(QCoreApplication::translate("QgsLightsWidget", "\302\260", nullptr));
        altitudeSpinBox->setText(QCoreApplication::translate("QgsLightsWidget", "Altitude", nullptr));
        spinBoxAltitude->setSuffix(QCoreApplication::translate("QgsLightsWidget", "\302\260", nullptr));
        label_16->setText(QCoreApplication::translate("QgsLightsWidget", "Color", nullptr));
        label_11->setText(QCoreApplication::translate("QgsLightsWidget", "Light Direction", nullptr));
        label_13->setText(QCoreApplication::translate("QgsLightsWidget", "Y", nullptr));
        labelY->setText(QCoreApplication::translate("QgsLightsWidget", "--", nullptr));
        label_10->setText(QCoreApplication::translate("QgsLightsWidget", "Z", nullptr));
        label_17->setText(QCoreApplication::translate("QgsLightsWidget", "X", nullptr));
        labelX->setText(QCoreApplication::translate("QgsLightsWidget", "--", nullptr));
        labelZ->setText(QCoreApplication::translate("QgsLightsWidget", "--", nullptr));
        label_18->setText(QCoreApplication::translate("QgsLightsWidget", "Intensity", nullptr));
        label_21->setText(QCoreApplication::translate("QgsLightsWidget", "Atmospheric pressure (hPa)", nullptr));
        label_19->setText(QCoreApplication::translate("QgsLightsWidget", "Color", nullptr));
        label_22->setText(QCoreApplication::translate("QgsLightsWidget", "Reference elevation", nullptr));
        mSunTemperatureSpin->setSuffix(QCoreApplication::translate("QgsLightsWidget", "\302\260", nullptr));
        mLabelSunTime->setText(QCoreApplication::translate("QgsLightsWidget", "Reference time", nullptr));
        label_20->setText(QCoreApplication::translate("QgsLightsWidget", "Temperature (Celsius)", nullptr));
        mSunPressureSpin->setSuffix(QString());
        mSunElevationSpin->setSuffix(QCoreApplication::translate("QgsLightsWidget", "m", nullptr));
        (void)QgsLightsWidget;
    } // retranslateUi

};

namespace Ui {
    class QgsLightsWidget: public Ui_QgsLightsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QGSLIGHTSWIDGET_H
