/********************************************************************************
** Form generated from reading UI file 'metalroughtexturedmaterialwidgetbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_METALROUGHTEXTUREDMATERIALWIDGETBASE_H
#define UI_METALROUGHTEXTUREDMATERIALWIDGETBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include "qgsdoublespinbox.h"
#include "qgsfilecontentsourcelineedit.h"
#include "qgsmaterialpreviewwidget.h"
#include "qgsopacitywidget.h"
#include "qgspropertyoverridebutton.h"

QT_BEGIN_NAMESPACE

class Ui_MetalRoughTexturedMaterialWidgetBase
{
public:
    QGridLayout *gridLayout;
    QgsImageSourceLineEdit *mEmissionTextureWidget;
    QgsPropertyOverrideButton *mTextureRotationDataDefinedButton;
    QgsImageSourceLineEdit *mMetalnessTextureWidget;
    QLabel *lblTextureRotation;
    QgsImageSourceLineEdit *mAmbientOcclusionTextureWidget;
    QgsOpacityWidget *mOpacityWidget;
    QSpacerItem *verticalSpacer;
    QgsMaterialPreviewWidget *mPreviewWidget;
    QLabel *lblTextureScale_7;
    QLabel *lblTextureScale_5;
    QLabel *lblTextureScale_9;
    QgsImageSourceLineEdit *mNormalTextureWidget;
    QgsImageSourceLineEdit *mRoughnessTextureWidget;
    QLabel *lblTextureScale_10;
    QgsDoubleSpinBox *mParallaxScaleSpinBox;
    QLabel *lblTextureScale_8;
    QgsDoubleSpinBox *textureRotationSpinBox;
    QLabel *lblTextureScale_4;
    QLabel *lblTextureScale;
    QgsImageSourceLineEdit *mHeightTextureWidget;
    QgsDoubleSpinBox *mEmissionStrengthSpinBox;
    QgsPropertyOverrideButton *mTextureScaleDataDefinedButton;
    QLabel *lblTextureScale_6;
    QgsDoubleSpinBox *textureScaleSpinBox;
    QLabel *lblTextureScale_3;
    QLabel *lblTextureScale_2;
    QLabel *lblTextureScale_11;
    QgsImageSourceLineEdit *mBaseColorTextureWidget;
    QGridLayout *gridLayout_2;
    QLabel *label_6;
    QgsDoubleSpinBox *textureOffsetYSpin;
    QgsDoubleSpinBox *textureOffsetXSpin;
    QLabel *label_7;
    QLabel *lblTextureRotation_2;
    QgsPropertyOverrideButton *mTextureOffsetDataDefinedButton;

    void setupUi(QWidget *MetalRoughTexturedMaterialWidgetBase)
    {
        if (MetalRoughTexturedMaterialWidgetBase->objectName().isEmpty())
            MetalRoughTexturedMaterialWidgetBase->setObjectName("MetalRoughTexturedMaterialWidgetBase");
        MetalRoughTexturedMaterialWidgetBase->resize(720, 423);
        MetalRoughTexturedMaterialWidgetBase->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(MetalRoughTexturedMaterialWidgetBase);
        gridLayout->setObjectName("gridLayout");
        mEmissionTextureWidget = new QgsImageSourceLineEdit(MetalRoughTexturedMaterialWidgetBase);
        mEmissionTextureWidget->setObjectName("mEmissionTextureWidget");
        mEmissionTextureWidget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        gridLayout->addWidget(mEmissionTextureWidget, 8, 2, 1, 2);

        mTextureRotationDataDefinedButton = new QgsPropertyOverrideButton(MetalRoughTexturedMaterialWidgetBase);
        mTextureRotationDataDefinedButton->setObjectName("mTextureRotationDataDefinedButton");

        gridLayout->addWidget(mTextureRotationDataDefinedButton, 12, 3, 1, 1);

        mMetalnessTextureWidget = new QgsImageSourceLineEdit(MetalRoughTexturedMaterialWidgetBase);
        mMetalnessTextureWidget->setObjectName("mMetalnessTextureWidget");
        mMetalnessTextureWidget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        gridLayout->addWidget(mMetalnessTextureWidget, 2, 2, 1, 2);

        lblTextureRotation = new QLabel(MetalRoughTexturedMaterialWidgetBase);
        lblTextureRotation->setObjectName("lblTextureRotation");

        gridLayout->addWidget(lblTextureRotation, 12, 1, 1, 1);

        mAmbientOcclusionTextureWidget = new QgsImageSourceLineEdit(MetalRoughTexturedMaterialWidgetBase);
        mAmbientOcclusionTextureWidget->setObjectName("mAmbientOcclusionTextureWidget");
        mAmbientOcclusionTextureWidget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        gridLayout->addWidget(mAmbientOcclusionTextureWidget, 7, 2, 1, 2);

        mOpacityWidget = new QgsOpacityWidget(MetalRoughTexturedMaterialWidgetBase);
        mOpacityWidget->setObjectName("mOpacityWidget");
        mOpacityWidget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        gridLayout->addWidget(mOpacityWidget, 10, 2, 1, 2);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 14, 2, 1, 1);

        mPreviewWidget = new QgsMaterialPreviewWidget(MetalRoughTexturedMaterialWidgetBase);
        mPreviewWidget->setObjectName("mPreviewWidget");
        mPreviewWidget->setMinimumSize(QSize(200, 200));

        gridLayout->addWidget(mPreviewWidget, 0, 0, 15, 1);

        lblTextureScale_7 = new QLabel(MetalRoughTexturedMaterialWidgetBase);
        lblTextureScale_7->setObjectName("lblTextureScale_7");

        gridLayout->addWidget(lblTextureScale_7, 8, 1, 1, 1);

        lblTextureScale_5 = new QLabel(MetalRoughTexturedMaterialWidgetBase);
        lblTextureScale_5->setObjectName("lblTextureScale_5");

        gridLayout->addWidget(lblTextureScale_5, 4, 1, 1, 1);

        lblTextureScale_9 = new QLabel(MetalRoughTexturedMaterialWidgetBase);
        lblTextureScale_9->setObjectName("lblTextureScale_9");

        gridLayout->addWidget(lblTextureScale_9, 5, 1, 1, 1);

        mNormalTextureWidget = new QgsImageSourceLineEdit(MetalRoughTexturedMaterialWidgetBase);
        mNormalTextureWidget->setObjectName("mNormalTextureWidget");
        mNormalTextureWidget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        gridLayout->addWidget(mNormalTextureWidget, 4, 2, 1, 2);

        mRoughnessTextureWidget = new QgsImageSourceLineEdit(MetalRoughTexturedMaterialWidgetBase);
        mRoughnessTextureWidget->setObjectName("mRoughnessTextureWidget");
        mRoughnessTextureWidget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        gridLayout->addWidget(mRoughnessTextureWidget, 3, 2, 1, 2);

        lblTextureScale_10 = new QLabel(MetalRoughTexturedMaterialWidgetBase);
        lblTextureScale_10->setObjectName("lblTextureScale_10");

        gridLayout->addWidget(lblTextureScale_10, 6, 1, 1, 1);

        mParallaxScaleSpinBox = new QgsDoubleSpinBox(MetalRoughTexturedMaterialWidgetBase);
        mParallaxScaleSpinBox->setObjectName("mParallaxScaleSpinBox");
        mParallaxScaleSpinBox->setMinimum(0.010000000000000);
        mParallaxScaleSpinBox->setMaximum(100000.000000000000000);
        mParallaxScaleSpinBox->setSingleStep(1.000000000000000);
        mParallaxScaleSpinBox->setValue(10.000000000000000);

        gridLayout->addWidget(mParallaxScaleSpinBox, 6, 2, 1, 2);

        lblTextureScale_8 = new QLabel(MetalRoughTexturedMaterialWidgetBase);
        lblTextureScale_8->setObjectName("lblTextureScale_8");

        gridLayout->addWidget(lblTextureScale_8, 9, 1, 1, 1);

        textureRotationSpinBox = new QgsDoubleSpinBox(MetalRoughTexturedMaterialWidgetBase);
        textureRotationSpinBox->setObjectName("textureRotationSpinBox");
        textureRotationSpinBox->setMinimum(-360.000000000000000);
        textureRotationSpinBox->setMaximum(360.000000000000000);
        textureRotationSpinBox->setSingleStep(0.500000000000000);

        gridLayout->addWidget(textureRotationSpinBox, 12, 2, 1, 1);

        lblTextureScale_4 = new QLabel(MetalRoughTexturedMaterialWidgetBase);
        lblTextureScale_4->setObjectName("lblTextureScale_4");

        gridLayout->addWidget(lblTextureScale_4, 3, 1, 1, 1);

        lblTextureScale = new QLabel(MetalRoughTexturedMaterialWidgetBase);
        lblTextureScale->setObjectName("lblTextureScale");

        gridLayout->addWidget(lblTextureScale, 11, 1, 1, 1);

        mHeightTextureWidget = new QgsImageSourceLineEdit(MetalRoughTexturedMaterialWidgetBase);
        mHeightTextureWidget->setObjectName("mHeightTextureWidget");
        mHeightTextureWidget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        gridLayout->addWidget(mHeightTextureWidget, 5, 2, 1, 2);

        mEmissionStrengthSpinBox = new QgsDoubleSpinBox(MetalRoughTexturedMaterialWidgetBase);
        mEmissionStrengthSpinBox->setObjectName("mEmissionStrengthSpinBox");
        mEmissionStrengthSpinBox->setMinimum(0.010000000000000);
        mEmissionStrengthSpinBox->setMaximum(100000.000000000000000);
        mEmissionStrengthSpinBox->setSingleStep(1.000000000000000);
        mEmissionStrengthSpinBox->setValue(100.000000000000000);

        gridLayout->addWidget(mEmissionStrengthSpinBox, 9, 2, 1, 2);

        mTextureScaleDataDefinedButton = new QgsPropertyOverrideButton(MetalRoughTexturedMaterialWidgetBase);
        mTextureScaleDataDefinedButton->setObjectName("mTextureScaleDataDefinedButton");

        gridLayout->addWidget(mTextureScaleDataDefinedButton, 11, 3, 1, 1);

        lblTextureScale_6 = new QLabel(MetalRoughTexturedMaterialWidgetBase);
        lblTextureScale_6->setObjectName("lblTextureScale_6");

        gridLayout->addWidget(lblTextureScale_6, 7, 1, 1, 1);

        textureScaleSpinBox = new QgsDoubleSpinBox(MetalRoughTexturedMaterialWidgetBase);
        textureScaleSpinBox->setObjectName("textureScaleSpinBox");
        textureScaleSpinBox->setMinimum(0.010000000000000);
        textureScaleSpinBox->setMaximum(100000.000000000000000);
        textureScaleSpinBox->setSingleStep(1.000000000000000);
        textureScaleSpinBox->setValue(100.000000000000000);

        gridLayout->addWidget(textureScaleSpinBox, 11, 2, 1, 1);

        lblTextureScale_3 = new QLabel(MetalRoughTexturedMaterialWidgetBase);
        lblTextureScale_3->setObjectName("lblTextureScale_3");

        gridLayout->addWidget(lblTextureScale_3, 2, 1, 1, 1);

        lblTextureScale_2 = new QLabel(MetalRoughTexturedMaterialWidgetBase);
        lblTextureScale_2->setObjectName("lblTextureScale_2");

        gridLayout->addWidget(lblTextureScale_2, 1, 1, 1, 1);

        lblTextureScale_11 = new QLabel(MetalRoughTexturedMaterialWidgetBase);
        lblTextureScale_11->setObjectName("lblTextureScale_11");

        gridLayout->addWidget(lblTextureScale_11, 10, 1, 1, 1);

        mBaseColorTextureWidget = new QgsImageSourceLineEdit(MetalRoughTexturedMaterialWidgetBase);
        mBaseColorTextureWidget->setObjectName("mBaseColorTextureWidget");
        mBaseColorTextureWidget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        gridLayout->addWidget(mBaseColorTextureWidget, 1, 2, 1, 2);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        label_6 = new QLabel(MetalRoughTexturedMaterialWidgetBase);
        label_6->setObjectName("label_6");

        gridLayout_2->addWidget(label_6, 1, 0, 1, 1);

        textureOffsetYSpin = new QgsDoubleSpinBox(MetalRoughTexturedMaterialWidgetBase);
        textureOffsetYSpin->setObjectName("textureOffsetYSpin");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(textureOffsetYSpin->sizePolicy().hasHeightForWidth());
        textureOffsetYSpin->setSizePolicy(sizePolicy);
        textureOffsetYSpin->setDecimals(6);
        textureOffsetYSpin->setMinimum(-99999999.989999994635582);
        textureOffsetYSpin->setMaximum(99999999.989999994635582);
        textureOffsetYSpin->setSingleStep(0.200000000000000);

        gridLayout_2->addWidget(textureOffsetYSpin, 1, 1, 1, 1);

        textureOffsetXSpin = new QgsDoubleSpinBox(MetalRoughTexturedMaterialWidgetBase);
        textureOffsetXSpin->setObjectName("textureOffsetXSpin");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(textureOffsetXSpin->sizePolicy().hasHeightForWidth());
        textureOffsetXSpin->setSizePolicy(sizePolicy1);
        textureOffsetXSpin->setDecimals(6);
        textureOffsetXSpin->setMinimum(-99999999.989999994635582);
        textureOffsetXSpin->setMaximum(99999999.989999994635582);
        textureOffsetXSpin->setSingleStep(0.200000000000000);

        gridLayout_2->addWidget(textureOffsetXSpin, 0, 1, 1, 1);

        label_7 = new QLabel(MetalRoughTexturedMaterialWidgetBase);
        label_7->setObjectName("label_7");

        gridLayout_2->addWidget(label_7, 0, 0, 1, 1);


        gridLayout->addLayout(gridLayout_2, 13, 2, 1, 1);

        lblTextureRotation_2 = new QLabel(MetalRoughTexturedMaterialWidgetBase);
        lblTextureRotation_2->setObjectName("lblTextureRotation_2");

        gridLayout->addWidget(lblTextureRotation_2, 13, 1, 1, 1);

        mTextureOffsetDataDefinedButton = new QgsPropertyOverrideButton(MetalRoughTexturedMaterialWidgetBase);
        mTextureOffsetDataDefinedButton->setObjectName("mTextureOffsetDataDefinedButton");

        gridLayout->addWidget(mTextureOffsetDataDefinedButton, 13, 3, 1, 1);

        QWidget::setTabOrder(mBaseColorTextureWidget, mMetalnessTextureWidget);
        QWidget::setTabOrder(mMetalnessTextureWidget, mRoughnessTextureWidget);
        QWidget::setTabOrder(mRoughnessTextureWidget, mNormalTextureWidget);
        QWidget::setTabOrder(mNormalTextureWidget, mHeightTextureWidget);
        QWidget::setTabOrder(mHeightTextureWidget, mParallaxScaleSpinBox);
        QWidget::setTabOrder(mParallaxScaleSpinBox, mAmbientOcclusionTextureWidget);
        QWidget::setTabOrder(mAmbientOcclusionTextureWidget, mEmissionTextureWidget);
        QWidget::setTabOrder(mEmissionTextureWidget, mEmissionStrengthSpinBox);
        QWidget::setTabOrder(mEmissionStrengthSpinBox, mOpacityWidget);
        QWidget::setTabOrder(mOpacityWidget, textureScaleSpinBox);
        QWidget::setTabOrder(textureScaleSpinBox, mTextureScaleDataDefinedButton);
        QWidget::setTabOrder(mTextureScaleDataDefinedButton, textureRotationSpinBox);
        QWidget::setTabOrder(textureRotationSpinBox, mTextureRotationDataDefinedButton);

        retranslateUi(MetalRoughTexturedMaterialWidgetBase);

        QMetaObject::connectSlotsByName(MetalRoughTexturedMaterialWidgetBase);
    } // setupUi

    void retranslateUi(QWidget *MetalRoughTexturedMaterialWidgetBase)
    {
#if QT_CONFIG(tooltip)
        mEmissionTextureWidget->setToolTip(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "Additional diffuse albedo texture which simulates emissive surfaces (eg Neon surfaces)", nullptr));
#endif // QT_CONFIG(tooltip)
        mTextureRotationDataDefinedButton->setText(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "\342\200\246", nullptr));
#if QT_CONFIG(tooltip)
        mMetalnessTextureWidget->setToolTip(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "A texture that defines whether a surface appears to be dielectric or conductor", nullptr));
#endif // QT_CONFIG(tooltip)
        lblTextureRotation->setText(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "Texture rotation", nullptr));
#if QT_CONFIG(tooltip)
        mAmbientOcclusionTextureWidget->setToolTip(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "A texture which defines how much of the ambient light is accessible to a surface point.", nullptr));
#endif // QT_CONFIG(tooltip)
        lblTextureScale_7->setText(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "Emission texture", nullptr));
        lblTextureScale_5->setText(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "Normal texture", nullptr));
        lblTextureScale_9->setText(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "Height (displacement) texture", nullptr));
#if QT_CONFIG(tooltip)
        mNormalTextureWidget->setToolTip(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "The detail normal used to perturb the surface using bump mapping. Must be set to an OpenGL format normal map (not DirectX).", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        mRoughnessTextureWidget->setToolTip(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "A texture that defines the perceived smoothness or roughness of a surface. Smooth surfaces exhibit sharp reflections.", nullptr));
#endif // QT_CONFIG(tooltip)
        lblTextureScale_10->setText(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "Height texture strength", nullptr));
#if QT_CONFIG(tooltip)
        mParallaxScaleSpinBox->setToolTip(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "Scaling factor for the height displacement effect. If visual artifacts are observed in the displacement, lowering the strength will help.", nullptr));
#endif // QT_CONFIG(tooltip)
        mParallaxScaleSpinBox->setSuffix(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", " %", nullptr));
        lblTextureScale_8->setText(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "Emission strength", nullptr));
        textureRotationSpinBox->setSuffix(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", " \302\260", nullptr));
        lblTextureScale_4->setText(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "Roughness texture", nullptr));
        lblTextureScale->setText(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "Texture scale", nullptr));
#if QT_CONFIG(tooltip)
        mHeightTextureWidget->setToolTip(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "Displacement texture for applying height offsets to the material.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        mEmissionStrengthSpinBox->setToolTip(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "Scaling factor for the emission effect.", nullptr));
#endif // QT_CONFIG(tooltip)
        mEmissionStrengthSpinBox->setSuffix(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", " %", nullptr));
        mTextureScaleDataDefinedButton->setText(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "\342\200\246", nullptr));
        lblTextureScale_6->setText(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "Ambient occlusion texture", nullptr));
        textureScaleSpinBox->setSuffix(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", " %", nullptr));
        lblTextureScale_3->setText(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "Metalness texture", nullptr));
        lblTextureScale_2->setText(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "Base color texture", nullptr));
        lblTextureScale_11->setText(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "Opacity", nullptr));
#if QT_CONFIG(tooltip)
        mBaseColorTextureWidget->setToolTip(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "Diffuse albedo texture for non-metallic surfaces, and specular color texture for metallic surfaces", nullptr));
#endif // QT_CONFIG(tooltip)
        label_6->setText(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "y", nullptr));
        label_7->setText(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "x", nullptr));
        lblTextureRotation_2->setText(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "Texture offset", nullptr));
        mTextureOffsetDataDefinedButton->setText(QCoreApplication::translate("MetalRoughTexturedMaterialWidgetBase", "\342\200\246", nullptr));
        (void)MetalRoughTexturedMaterialWidgetBase;
    } // retranslateUi

};

namespace Ui {
    class MetalRoughTexturedMaterialWidgetBase: public Ui_MetalRoughTexturedMaterialWidgetBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_METALROUGHTEXTUREDMATERIALWIDGETBASE_H
