/********************************************************************************
** Form generated from reading UI file 'phongtexturedmaterialwidgetbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PHONGTEXTUREDMATERIALWIDGETBASE_H
#define UI_PHONGTEXTUREDMATERIALWIDGETBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include "qgscolorbutton.h"
#include "qgsdoublespinbox.h"
#include "qgsfilecontentsourcelineedit.h"
#include "qgsmaterialpreviewwidget.h"
#include "qgsopacitywidget.h"
#include "qgspropertyoverridebutton.h"

QT_BEGIN_NAMESPACE

class Ui_PhongTexturedMaterialWidgetBase
{
public:
    QGridLayout *gridLayout;
    QLabel *lblTextureRotation;
    QLabel *lblOpacity;
    QGridLayout *gridLayout_2;
    QLabel *label_7;
    QgsDoubleSpinBox *textureOffsetXSpin;
    QgsDoubleSpinBox *textureOffsetYSpin;
    QLabel *label_6;
    QLabel *lblTextureScale;
    QgsDoubleSpinBox *textureRotationSpinBox;
    QLabel *lblAmbient;
    QLabel *lblSpecular;
    QLabel *lblShininess;
    QgsDoubleSpinBox *textureScaleSpinBox;
    QgsMaterialPreviewWidget *mPreviewWidget;
    QSpacerItem *verticalSpacer;
    QLabel *lblTextureScale_2;
    QgsPropertyOverrideButton *mTextureScaleDataDefinedButton;
    QgsPropertyOverrideButton *mTextureRotationDataDefinedButton;
    QgsPropertyOverrideButton *mTextureOffsetDataDefinedButton;
    QLabel *lblTextureRotation_2;
    QgsColorButton *btnAmbient;
    QgsColorButton *btnSpecular;
    QgsDoubleSpinBox *spinShininess;
    QgsOpacityWidget *mOpacityWidget;
    QgsImageSourceLineEdit *textureFile;

    void setupUi(QWidget *PhongTexturedMaterialWidgetBase)
    {
        if (PhongTexturedMaterialWidgetBase->objectName().isEmpty())
            PhongTexturedMaterialWidgetBase->setObjectName("PhongTexturedMaterialWidgetBase");
        PhongTexturedMaterialWidgetBase->resize(720, 423);
        PhongTexturedMaterialWidgetBase->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(PhongTexturedMaterialWidgetBase);
        gridLayout->setObjectName("gridLayout");
        lblTextureRotation = new QLabel(PhongTexturedMaterialWidgetBase);
        lblTextureRotation->setObjectName("lblTextureRotation");

        gridLayout->addWidget(lblTextureRotation, 6, 1, 1, 1);

        lblOpacity = new QLabel(PhongTexturedMaterialWidgetBase);
        lblOpacity->setObjectName("lblOpacity");

        gridLayout->addWidget(lblOpacity, 3, 1, 1, 1);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        label_7 = new QLabel(PhongTexturedMaterialWidgetBase);
        label_7->setObjectName("label_7");

        gridLayout_2->addWidget(label_7, 0, 0, 1, 1);

        textureOffsetXSpin = new QgsDoubleSpinBox(PhongTexturedMaterialWidgetBase);
        textureOffsetXSpin->setObjectName("textureOffsetXSpin");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(textureOffsetXSpin->sizePolicy().hasHeightForWidth());
        textureOffsetXSpin->setSizePolicy(sizePolicy);
        textureOffsetXSpin->setDecimals(6);
        textureOffsetXSpin->setMinimum(-99999999.989999994635582);
        textureOffsetXSpin->setMaximum(99999999.989999994635582);
        textureOffsetXSpin->setSingleStep(0.200000000000000);

        gridLayout_2->addWidget(textureOffsetXSpin, 0, 1, 1, 1);

        textureOffsetYSpin = new QgsDoubleSpinBox(PhongTexturedMaterialWidgetBase);
        textureOffsetYSpin->setObjectName("textureOffsetYSpin");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(textureOffsetYSpin->sizePolicy().hasHeightForWidth());
        textureOffsetYSpin->setSizePolicy(sizePolicy1);
        textureOffsetYSpin->setDecimals(6);
        textureOffsetYSpin->setMinimum(-99999999.989999994635582);
        textureOffsetYSpin->setMaximum(99999999.989999994635582);
        textureOffsetYSpin->setSingleStep(0.200000000000000);

        gridLayout_2->addWidget(textureOffsetYSpin, 1, 1, 1, 1);

        label_6 = new QLabel(PhongTexturedMaterialWidgetBase);
        label_6->setObjectName("label_6");

        gridLayout_2->addWidget(label_6, 1, 0, 1, 1);


        gridLayout->addLayout(gridLayout_2, 8, 2, 1, 1);

        lblTextureScale = new QLabel(PhongTexturedMaterialWidgetBase);
        lblTextureScale->setObjectName("lblTextureScale");

        gridLayout->addWidget(lblTextureScale, 5, 1, 1, 1);

        textureRotationSpinBox = new QgsDoubleSpinBox(PhongTexturedMaterialWidgetBase);
        textureRotationSpinBox->setObjectName("textureRotationSpinBox");
        textureRotationSpinBox->setMinimum(-360.000000000000000);
        textureRotationSpinBox->setMaximum(360.000000000000000);
        textureRotationSpinBox->setSingleStep(0.500000000000000);

        gridLayout->addWidget(textureRotationSpinBox, 6, 2, 1, 1);

        lblAmbient = new QLabel(PhongTexturedMaterialWidgetBase);
        lblAmbient->setObjectName("lblAmbient");

        gridLayout->addWidget(lblAmbient, 0, 1, 1, 1);

        lblSpecular = new QLabel(PhongTexturedMaterialWidgetBase);
        lblSpecular->setObjectName("lblSpecular");

        gridLayout->addWidget(lblSpecular, 1, 1, 1, 1);

        lblShininess = new QLabel(PhongTexturedMaterialWidgetBase);
        lblShininess->setObjectName("lblShininess");

        gridLayout->addWidget(lblShininess, 2, 1, 1, 1);

        textureScaleSpinBox = new QgsDoubleSpinBox(PhongTexturedMaterialWidgetBase);
        textureScaleSpinBox->setObjectName("textureScaleSpinBox");
        textureScaleSpinBox->setMinimum(0.010000000000000);
        textureScaleSpinBox->setMaximum(100000.000000000000000);
        textureScaleSpinBox->setSingleStep(1.000000000000000);
        textureScaleSpinBox->setValue(100.000000000000000);

        gridLayout->addWidget(textureScaleSpinBox, 5, 2, 1, 1);

        mPreviewWidget = new QgsMaterialPreviewWidget(PhongTexturedMaterialWidgetBase);
        mPreviewWidget->setObjectName("mPreviewWidget");
        mPreviewWidget->setMinimumSize(QSize(200, 200));

        gridLayout->addWidget(mPreviewWidget, 0, 0, 10, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 9, 2, 1, 1);

        lblTextureScale_2 = new QLabel(PhongTexturedMaterialWidgetBase);
        lblTextureScale_2->setObjectName("lblTextureScale_2");

        gridLayout->addWidget(lblTextureScale_2, 4, 1, 1, 1);

        mTextureScaleDataDefinedButton = new QgsPropertyOverrideButton(PhongTexturedMaterialWidgetBase);
        mTextureScaleDataDefinedButton->setObjectName("mTextureScaleDataDefinedButton");

        gridLayout->addWidget(mTextureScaleDataDefinedButton, 5, 3, 1, 1);

        mTextureRotationDataDefinedButton = new QgsPropertyOverrideButton(PhongTexturedMaterialWidgetBase);
        mTextureRotationDataDefinedButton->setObjectName("mTextureRotationDataDefinedButton");

        gridLayout->addWidget(mTextureRotationDataDefinedButton, 6, 3, 1, 1);

        mTextureOffsetDataDefinedButton = new QgsPropertyOverrideButton(PhongTexturedMaterialWidgetBase);
        mTextureOffsetDataDefinedButton->setObjectName("mTextureOffsetDataDefinedButton");

        gridLayout->addWidget(mTextureOffsetDataDefinedButton, 8, 3, 1, 1);

        lblTextureRotation_2 = new QLabel(PhongTexturedMaterialWidgetBase);
        lblTextureRotation_2->setObjectName("lblTextureRotation_2");

        gridLayout->addWidget(lblTextureRotation_2, 8, 1, 1, 1);

        btnAmbient = new QgsColorButton(PhongTexturedMaterialWidgetBase);
        btnAmbient->setObjectName("btnAmbient");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(btnAmbient->sizePolicy().hasHeightForWidth());
        btnAmbient->setSizePolicy(sizePolicy2);
        btnAmbient->setMinimumSize(QSize(120, 0));

        gridLayout->addWidget(btnAmbient, 0, 2, 1, 2);

        btnSpecular = new QgsColorButton(PhongTexturedMaterialWidgetBase);
        btnSpecular->setObjectName("btnSpecular");
        sizePolicy2.setHeightForWidth(btnSpecular->sizePolicy().hasHeightForWidth());
        btnSpecular->setSizePolicy(sizePolicy2);
        btnSpecular->setMinimumSize(QSize(120, 0));

        gridLayout->addWidget(btnSpecular, 1, 2, 1, 2);

        spinShininess = new QgsDoubleSpinBox(PhongTexturedMaterialWidgetBase);
        spinShininess->setObjectName("spinShininess");
        spinShininess->setMaximum(1000.000000000000000);

        gridLayout->addWidget(spinShininess, 2, 2, 1, 2);

        mOpacityWidget = new QgsOpacityWidget(PhongTexturedMaterialWidgetBase);
        mOpacityWidget->setObjectName("mOpacityWidget");
        mOpacityWidget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        gridLayout->addWidget(mOpacityWidget, 3, 2, 1, 2);

        textureFile = new QgsImageSourceLineEdit(PhongTexturedMaterialWidgetBase);
        textureFile->setObjectName("textureFile");
        textureFile->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        gridLayout->addWidget(textureFile, 4, 2, 1, 2);

        QWidget::setTabOrder(btnAmbient, btnSpecular);
        QWidget::setTabOrder(btnSpecular, spinShininess);
        QWidget::setTabOrder(spinShininess, mOpacityWidget);
        QWidget::setTabOrder(mOpacityWidget, textureFile);
        QWidget::setTabOrder(textureFile, textureScaleSpinBox);
        QWidget::setTabOrder(textureScaleSpinBox, mTextureScaleDataDefinedButton);
        QWidget::setTabOrder(mTextureScaleDataDefinedButton, textureRotationSpinBox);
        QWidget::setTabOrder(textureRotationSpinBox, mTextureRotationDataDefinedButton);
        QWidget::setTabOrder(mTextureRotationDataDefinedButton, textureOffsetXSpin);
        QWidget::setTabOrder(textureOffsetXSpin, textureOffsetYSpin);
        QWidget::setTabOrder(textureOffsetYSpin, mTextureOffsetDataDefinedButton);

        retranslateUi(PhongTexturedMaterialWidgetBase);

        QMetaObject::connectSlotsByName(PhongTexturedMaterialWidgetBase);
    } // setupUi

    void retranslateUi(QWidget *PhongTexturedMaterialWidgetBase)
    {
        lblTextureRotation->setText(QCoreApplication::translate("PhongTexturedMaterialWidgetBase", "Texture rotation", nullptr));
        lblOpacity->setText(QCoreApplication::translate("PhongTexturedMaterialWidgetBase", "Opacity", nullptr));
        label_7->setText(QCoreApplication::translate("PhongTexturedMaterialWidgetBase", "x", nullptr));
        label_6->setText(QCoreApplication::translate("PhongTexturedMaterialWidgetBase", "y", nullptr));
        lblTextureScale->setText(QCoreApplication::translate("PhongTexturedMaterialWidgetBase", "Texture scale", nullptr));
        textureRotationSpinBox->setSuffix(QCoreApplication::translate("PhongTexturedMaterialWidgetBase", " \302\260", nullptr));
#if QT_CONFIG(tooltip)
        lblAmbient->setToolTip(QCoreApplication::translate("PhongTexturedMaterialWidgetBase", "Color of light that is scattered around the entire scene.", nullptr));
#endif // QT_CONFIG(tooltip)
        lblAmbient->setText(QCoreApplication::translate("PhongTexturedMaterialWidgetBase", "Ambient", nullptr));
#if QT_CONFIG(tooltip)
        lblSpecular->setToolTip(QCoreApplication::translate("PhongTexturedMaterialWidgetBase", "Color of light reflecting from smooth surfaces.", nullptr));
#endif // QT_CONFIG(tooltip)
        lblSpecular->setText(QCoreApplication::translate("PhongTexturedMaterialWidgetBase", "Specular", nullptr));
#if QT_CONFIG(tooltip)
        lblShininess->setToolTip(QCoreApplication::translate("PhongTexturedMaterialWidgetBase", "How shiny smooth surfaces are.", nullptr));
#endif // QT_CONFIG(tooltip)
        lblShininess->setText(QCoreApplication::translate("PhongTexturedMaterialWidgetBase", "Shininess", nullptr));
        textureScaleSpinBox->setSuffix(QCoreApplication::translate("PhongTexturedMaterialWidgetBase", " %", nullptr));
        lblTextureScale_2->setText(QCoreApplication::translate("PhongTexturedMaterialWidgetBase", "Diffuse texture", nullptr));
        mTextureScaleDataDefinedButton->setText(QCoreApplication::translate("PhongTexturedMaterialWidgetBase", "\342\200\246", nullptr));
        mTextureRotationDataDefinedButton->setText(QCoreApplication::translate("PhongTexturedMaterialWidgetBase", "\342\200\246", nullptr));
        mTextureOffsetDataDefinedButton->setText(QCoreApplication::translate("PhongTexturedMaterialWidgetBase", "\342\200\246", nullptr));
        lblTextureRotation_2->setText(QCoreApplication::translate("PhongTexturedMaterialWidgetBase", "Texture offset", nullptr));
        (void)PhongTexturedMaterialWidgetBase;
    } // retranslateUi

};

namespace Ui {
    class PhongTexturedMaterialWidgetBase: public Ui_PhongTexturedMaterialWidgetBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PHONGTEXTUREDMATERIALWIDGETBASE_H
