/********************************************************************************
** Form generated from reading UI file 'qgsmaskingwidgetbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QGSMASKINGWIDGETBASE_H
#define UI_QGSMASKINGWIDGETBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "qgsmasksourceselectionwidget.h"
#include "qgsmessagebar.h"
#include "qgssymbollayerselectionwidget.h"

QT_BEGIN_NAMESPACE

class Ui_QgsMaskingWidgetBase
{
public:
    QVBoxLayout *verticalLayout_2;
    QgsMessageBar *mMessageBar;
    QLabel *label;
    QgsSymbolLayerSelectionWidget *mMaskTargetsWidget;
    QLabel *label_2;
    QgsMaskSourceSelectionWidget *mMaskSourcesWidget;

    void setupUi(QWidget *QgsMaskingWidgetBase)
    {
        if (QgsMaskingWidgetBase->objectName().isEmpty())
            QgsMaskingWidgetBase->setObjectName("QgsMaskingWidgetBase");
        QgsMaskingWidgetBase->resize(812, 538);
        QgsMaskingWidgetBase->setWindowTitle(QString::fromUtf8("Form"));
        verticalLayout_2 = new QVBoxLayout(QgsMaskingWidgetBase);
        verticalLayout_2->setObjectName("verticalLayout_2");
        mMessageBar = new QgsMessageBar(QgsMaskingWidgetBase);
        mMessageBar->setObjectName("mMessageBar");
        mMessageBar->setFrameShape(QFrame::Shape::StyledPanel);
        mMessageBar->setFrameShadow(QFrame::Shadow::Raised);

        verticalLayout_2->addWidget(mMessageBar);

        label = new QLabel(QgsMaskingWidgetBase);
        label->setObjectName("label");

        verticalLayout_2->addWidget(label);

        mMaskTargetsWidget = new QgsSymbolLayerSelectionWidget(QgsMaskingWidgetBase);
        mMaskTargetsWidget->setObjectName("mMaskTargetsWidget");

        verticalLayout_2->addWidget(mMaskTargetsWidget);

        label_2 = new QLabel(QgsMaskingWidgetBase);
        label_2->setObjectName("label_2");

        verticalLayout_2->addWidget(label_2);

        mMaskSourcesWidget = new QgsMaskSourceSelectionWidget(QgsMaskingWidgetBase);
        mMaskSourcesWidget->setObjectName("mMaskSourcesWidget");

        verticalLayout_2->addWidget(mMaskSourcesWidget);


        retranslateUi(QgsMaskingWidgetBase);

        QMetaObject::connectSlotsByName(QgsMaskingWidgetBase);
    } // setupUi

    void retranslateUi(QWidget *QgsMaskingWidgetBase)
    {
        label->setText(QCoreApplication::translate("QgsMaskingWidgetBase", "<b>Masked symbol layers</b>", nullptr));
        label_2->setText(QCoreApplication::translate("QgsMaskingWidgetBase", "<b>Mask sources</b>", nullptr));
        (void)QgsMaskingWidgetBase;
    } // retranslateUi

};

namespace Ui {
    class QgsMaskingWidgetBase: public Ui_QgsMaskingWidgetBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QGSMASKINGWIDGETBASE_H
