/********************************************************************************
** Form generated from reading UI file 'qgsbrowserpropertiesdialogbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QGSBROWSERPROPERTIESDIALOGBASE_H
#define UI_QGSBROWSERPROPERTIESDIALOGBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_QgsBrowserPropertiesDialogBase
{
public:
    QVBoxLayout *verticalLayout;
    QVBoxLayout *mLayout;
    QDialogButtonBox *mButtonBox;

    void setupUi(QDialog *QgsBrowserPropertiesDialogBase)
    {
        if (QgsBrowserPropertiesDialogBase->objectName().isEmpty())
            QgsBrowserPropertiesDialogBase->setObjectName("QgsBrowserPropertiesDialogBase");
        QgsBrowserPropertiesDialogBase->resize(800, 424);
        QgsBrowserPropertiesDialogBase->setWindowTitle(QString::fromUtf8("Dialog"));
        QgsBrowserPropertiesDialogBase->setSizeGripEnabled(true);
        verticalLayout = new QVBoxLayout(QgsBrowserPropertiesDialogBase);
        verticalLayout->setObjectName("verticalLayout");
        mLayout = new QVBoxLayout();
        mLayout->setObjectName("mLayout");

        verticalLayout->addLayout(mLayout);

        mButtonBox = new QDialogButtonBox(QgsBrowserPropertiesDialogBase);
        mButtonBox->setObjectName("mButtonBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(mButtonBox->sizePolicy().hasHeightForWidth());
        mButtonBox->setSizePolicy(sizePolicy);
        mButtonBox->setOrientation(Qt::Horizontal);
        mButtonBox->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout->addWidget(mButtonBox);


        retranslateUi(QgsBrowserPropertiesDialogBase);
        QObject::connect(mButtonBox, &QDialogButtonBox::accepted, QgsBrowserPropertiesDialogBase, qOverload<>(&QDialog::accept));
        QObject::connect(mButtonBox, &QDialogButtonBox::rejected, QgsBrowserPropertiesDialogBase, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(QgsBrowserPropertiesDialogBase);
    } // setupUi

    void retranslateUi(QDialog *QgsBrowserPropertiesDialogBase)
    {
        (void)QgsBrowserPropertiesDialogBase;
    } // retranslateUi

};

namespace Ui {
    class QgsBrowserPropertiesDialogBase: public Ui_QgsBrowserPropertiesDialogBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QGSBROWSERPROPERTIESDIALOGBASE_H
