/***************************************************************************
  qgslabeling.h
  --------------------------
  Date                 : January 2020
  Copyright            : (C) 2020 by Nyall Dawson
  Email                : nyall dot dawson at gmail dot com
 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef QGSLABELING_H
#define QGSLABELING_H

#include "qgis_core.h"
#include "qgis_sip.h"

#include <QFlags>

/**
 * \ingroup core
 * \class QgsLabeling
 *
 * \brief Contains constants and enums relating to labeling.
 *
 * \since QGIS 3.12
 */
class CORE_EXPORT QgsLabeling
{
  public:
};


#endif // QGSLABELING_H
