/********************************************************************************
** Form generated from reading UI file 'metalroughmaterialwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_METALROUGHMATERIALWIDGET_H
#define UI_METALROUGHMATERIALWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include "qgscolorbutton.h"
#include "qgsdoublespinbox.h"
#include "qgsmaterialpreviewwidget.h"
#include "qgsopacitywidget.h"
#include "qgspercentagewidget.h"
#include "qgspropertyoverridebutton.h"

QT_BEGIN_NAMESPACE

class Ui_MetalRoughMaterialWidget
{
public:
    QGridLayout *gridLayout;
    QLabel *lblSpecular;
    QgsDoubleSpinBox *mEmissionStrengthSpinBox;
    QLabel *lblMetalness;
    QgsPercentageWidget *mAnisotropyWidget;
    QLabel *lblRoughness_4;
    QgsPropertyOverrideButton *mEmissionColorDataDefinedButton;
    QgsPercentageWidget *mReflectanceWidget;
    QgsPercentageWidget *mClearCoatFactorWidget;
    QgsColorButton *mButtonEmissionColor;
    QLabel *lblRoughness_5;
    QgsPropertyOverrideButton *mBaseColorDataDefinedButton;
    QgsMaterialPreviewWidget *mPreviewWidget;
    QLabel *lblSpecular_3;
    QLabel *lblRoughness_2;
    QLabel *lblSpecular_6;
    QLabel *lblSpecular_2;
    QLabel *lblSpecular_7;
    QgsPercentageWidget *mClearCoatRoughnessWidget;
    QLabel *lblRoughness_3;
    QgsPercentageWidget *mRoughnessWidget;
    QSlider *mAnisotropyRotationWidget;
    QgsColorButton *mButtonBaseColor;
    QgsOpacityWidget *mOpacityWidget;
    QgsPercentageWidget *mMetalnessWidget;
    QSpacerItem *verticalSpacer;
    QLabel *lblRoughness;

    void setupUi(QWidget *MetalRoughMaterialWidget)
    {
        if (MetalRoughMaterialWidget->objectName().isEmpty())
            MetalRoughMaterialWidget->setObjectName("MetalRoughMaterialWidget");
        MetalRoughMaterialWidget->resize(739, 391);
        MetalRoughMaterialWidget->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(MetalRoughMaterialWidget);
        gridLayout->setObjectName("gridLayout");
        lblSpecular = new QLabel(MetalRoughMaterialWidget);
        lblSpecular->setObjectName("lblSpecular");

        gridLayout->addWidget(lblSpecular, 0, 1, 1, 1);

        mEmissionStrengthSpinBox = new QgsDoubleSpinBox(MetalRoughMaterialWidget);
        mEmissionStrengthSpinBox->setObjectName("mEmissionStrengthSpinBox");
        mEmissionStrengthSpinBox->setMinimum(0.010000000000000);
        mEmissionStrengthSpinBox->setMaximum(100000.000000000000000);
        mEmissionStrengthSpinBox->setSingleStep(1.000000000000000);
        mEmissionStrengthSpinBox->setValue(100.000000000000000);

        gridLayout->addWidget(mEmissionStrengthSpinBox, 9, 2, 1, 2);

        lblMetalness = new QLabel(MetalRoughMaterialWidget);
        lblMetalness->setObjectName("lblMetalness");

        gridLayout->addWidget(lblMetalness, 1, 1, 1, 1);

        mAnisotropyWidget = new QgsPercentageWidget(MetalRoughMaterialWidget);
        mAnisotropyWidget->setObjectName("mAnisotropyWidget");
        mAnisotropyWidget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        gridLayout->addWidget(mAnisotropyWidget, 4, 2, 1, 2);

        lblRoughness_4 = new QLabel(MetalRoughMaterialWidget);
        lblRoughness_4->setObjectName("lblRoughness_4");

        gridLayout->addWidget(lblRoughness_4, 4, 1, 1, 1);

        mEmissionColorDataDefinedButton = new QgsPropertyOverrideButton(MetalRoughMaterialWidget);
        mEmissionColorDataDefinedButton->setObjectName("mEmissionColorDataDefinedButton");

        gridLayout->addWidget(mEmissionColorDataDefinedButton, 8, 3, 1, 1);

        mReflectanceWidget = new QgsPercentageWidget(MetalRoughMaterialWidget);
        mReflectanceWidget->setObjectName("mReflectanceWidget");
        mReflectanceWidget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        gridLayout->addWidget(mReflectanceWidget, 3, 2, 1, 2);

        mClearCoatFactorWidget = new QgsPercentageWidget(MetalRoughMaterialWidget);
        mClearCoatFactorWidget->setObjectName("mClearCoatFactorWidget");
        mClearCoatFactorWidget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        gridLayout->addWidget(mClearCoatFactorWidget, 6, 2, 1, 2);

        mButtonEmissionColor = new QgsColorButton(MetalRoughMaterialWidget);
        mButtonEmissionColor->setObjectName("mButtonEmissionColor");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(mButtonEmissionColor->sizePolicy().hasHeightForWidth());
        mButtonEmissionColor->setSizePolicy(sizePolicy);
        mButtonEmissionColor->setMinimumSize(QSize(120, 0));

        gridLayout->addWidget(mButtonEmissionColor, 8, 2, 1, 1);

        lblRoughness_5 = new QLabel(MetalRoughMaterialWidget);
        lblRoughness_5->setObjectName("lblRoughness_5");

        gridLayout->addWidget(lblRoughness_5, 5, 1, 1, 1);

        mBaseColorDataDefinedButton = new QgsPropertyOverrideButton(MetalRoughMaterialWidget);
        mBaseColorDataDefinedButton->setObjectName("mBaseColorDataDefinedButton");

        gridLayout->addWidget(mBaseColorDataDefinedButton, 0, 3, 1, 1);

        mPreviewWidget = new QgsMaterialPreviewWidget(MetalRoughMaterialWidget);
        mPreviewWidget->setObjectName("mPreviewWidget");
        mPreviewWidget->setMinimumSize(QSize(200, 200));

        gridLayout->addWidget(mPreviewWidget, 0, 0, 12, 1);

        lblSpecular_3 = new QLabel(MetalRoughMaterialWidget);
        lblSpecular_3->setObjectName("lblSpecular_3");

        gridLayout->addWidget(lblSpecular_3, 9, 1, 1, 1);

        lblRoughness_2 = new QLabel(MetalRoughMaterialWidget);
        lblRoughness_2->setObjectName("lblRoughness_2");

        gridLayout->addWidget(lblRoughness_2, 10, 1, 1, 1);

        lblSpecular_6 = new QLabel(MetalRoughMaterialWidget);
        lblSpecular_6->setObjectName("lblSpecular_6");

        gridLayout->addWidget(lblSpecular_6, 6, 1, 1, 1);

        lblSpecular_2 = new QLabel(MetalRoughMaterialWidget);
        lblSpecular_2->setObjectName("lblSpecular_2");

        gridLayout->addWidget(lblSpecular_2, 8, 1, 1, 1);

        lblSpecular_7 = new QLabel(MetalRoughMaterialWidget);
        lblSpecular_7->setObjectName("lblSpecular_7");

        gridLayout->addWidget(lblSpecular_7, 7, 1, 1, 1);

        mClearCoatRoughnessWidget = new QgsPercentageWidget(MetalRoughMaterialWidget);
        mClearCoatRoughnessWidget->setObjectName("mClearCoatRoughnessWidget");
        mClearCoatRoughnessWidget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        gridLayout->addWidget(mClearCoatRoughnessWidget, 7, 2, 1, 2);

        lblRoughness_3 = new QLabel(MetalRoughMaterialWidget);
        lblRoughness_3->setObjectName("lblRoughness_3");

        gridLayout->addWidget(lblRoughness_3, 3, 1, 1, 1);

        mRoughnessWidget = new QgsPercentageWidget(MetalRoughMaterialWidget);
        mRoughnessWidget->setObjectName("mRoughnessWidget");
        mRoughnessWidget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        gridLayout->addWidget(mRoughnessWidget, 2, 2, 1, 2);

        mAnisotropyRotationWidget = new QSlider(MetalRoughMaterialWidget);
        mAnisotropyRotationWidget->setObjectName("mAnisotropyRotationWidget");
        mAnisotropyRotationWidget->setMaximum(360);
        mAnisotropyRotationWidget->setOrientation(Qt::Orientation::Horizontal);

        gridLayout->addWidget(mAnisotropyRotationWidget, 5, 2, 1, 2);

        mButtonBaseColor = new QgsColorButton(MetalRoughMaterialWidget);
        mButtonBaseColor->setObjectName("mButtonBaseColor");
        sizePolicy.setHeightForWidth(mButtonBaseColor->sizePolicy().hasHeightForWidth());
        mButtonBaseColor->setSizePolicy(sizePolicy);
        mButtonBaseColor->setMinimumSize(QSize(120, 0));

        gridLayout->addWidget(mButtonBaseColor, 0, 2, 1, 1);

        mOpacityWidget = new QgsOpacityWidget(MetalRoughMaterialWidget);
        mOpacityWidget->setObjectName("mOpacityWidget");
        mOpacityWidget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        gridLayout->addWidget(mOpacityWidget, 10, 2, 1, 2);

        mMetalnessWidget = new QgsPercentageWidget(MetalRoughMaterialWidget);
        mMetalnessWidget->setObjectName("mMetalnessWidget");
        mMetalnessWidget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        gridLayout->addWidget(mMetalnessWidget, 1, 2, 1, 2);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 11, 1, 1, 1);

        lblRoughness = new QLabel(MetalRoughMaterialWidget);
        lblRoughness->setObjectName("lblRoughness");

        gridLayout->addWidget(lblRoughness, 2, 1, 1, 1);

        QWidget::setTabOrder(mButtonBaseColor, mBaseColorDataDefinedButton);
        QWidget::setTabOrder(mBaseColorDataDefinedButton, mMetalnessWidget);
        QWidget::setTabOrder(mMetalnessWidget, mRoughnessWidget);
        QWidget::setTabOrder(mRoughnessWidget, mReflectanceWidget);
        QWidget::setTabOrder(mReflectanceWidget, mAnisotropyWidget);
        QWidget::setTabOrder(mAnisotropyWidget, mAnisotropyRotationWidget);
        QWidget::setTabOrder(mAnisotropyRotationWidget, mClearCoatFactorWidget);
        QWidget::setTabOrder(mClearCoatFactorWidget, mClearCoatRoughnessWidget);
        QWidget::setTabOrder(mClearCoatRoughnessWidget, mButtonEmissionColor);
        QWidget::setTabOrder(mButtonEmissionColor, mEmissionColorDataDefinedButton);
        QWidget::setTabOrder(mEmissionColorDataDefinedButton, mEmissionStrengthSpinBox);
        QWidget::setTabOrder(mEmissionStrengthSpinBox, mOpacityWidget);

        retranslateUi(MetalRoughMaterialWidget);

        QMetaObject::connectSlotsByName(MetalRoughMaterialWidget);
    } // setupUi

    void retranslateUi(QWidget *MetalRoughMaterialWidget)
    {
        lblSpecular->setText(QCoreApplication::translate("MetalRoughMaterialWidget", "Base color", nullptr));
#if QT_CONFIG(tooltip)
        mEmissionStrengthSpinBox->setToolTip(QCoreApplication::translate("MetalRoughMaterialWidget", "Scaling factor for the emission effect.", nullptr));
#endif // QT_CONFIG(tooltip)
        mEmissionStrengthSpinBox->setSuffix(QCoreApplication::translate("MetalRoughMaterialWidget", " %", nullptr));
        lblMetalness->setText(QCoreApplication::translate("MetalRoughMaterialWidget", "Metalness", nullptr));
        lblRoughness_4->setText(QCoreApplication::translate("MetalRoughMaterialWidget", "Anisotropy", nullptr));
        mEmissionColorDataDefinedButton->setText(QCoreApplication::translate("MetalRoughMaterialWidget", "...", nullptr));
        lblRoughness_5->setText(QCoreApplication::translate("MetalRoughMaterialWidget", "Anisotropy direction", nullptr));
        mBaseColorDataDefinedButton->setText(QCoreApplication::translate("MetalRoughMaterialWidget", "...", nullptr));
        lblSpecular_3->setText(QCoreApplication::translate("MetalRoughMaterialWidget", "Emission strength", nullptr));
        lblRoughness_2->setText(QCoreApplication::translate("MetalRoughMaterialWidget", "Opacity", nullptr));
        lblSpecular_6->setText(QCoreApplication::translate("MetalRoughMaterialWidget", "Clear coat strength", nullptr));
        lblSpecular_2->setText(QCoreApplication::translate("MetalRoughMaterialWidget", "Emission", nullptr));
        lblSpecular_7->setText(QCoreApplication::translate("MetalRoughMaterialWidget", "Clear coat roughness", nullptr));
        lblRoughness_3->setText(QCoreApplication::translate("MetalRoughMaterialWidget", "Reflectance", nullptr));
        lblRoughness->setText(QCoreApplication::translate("MetalRoughMaterialWidget", "Roughness", nullptr));
        (void)MetalRoughMaterialWidget;
    } // retranslateUi

};

namespace Ui {
    class MetalRoughMaterialWidget: public Ui_MetalRoughMaterialWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_METALROUGHMATERIALWIDGET_H
