"""
***************************************************************************
    QtSvgWidgets.py
    ---------------------
    Date                 : June 2026
    Copyright            : (C) 2026 Jean Felder
    Email                : jean dot felder at oslandia dot com
***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************
"""

__author__ = 'Jean Felder'
__date__ = 'June 2026'
__copyright__ = '(C) 2026, Jean Felder'

from PyQt6.QtSvgWidgets import *
