/********************************************************************************
** Form generated from reading UI file 'simplelinematerialwidgetbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SIMPLELINEMATERIALWIDGETBASE_H
#define UI_SIMPLELINEMATERIALWIDGETBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include "qgscolorbutton.h"
#include "qgsmaterialpreviewwidget.h"
#include "qgspropertyoverridebutton.h"

QT_BEGIN_NAMESPACE

class Ui_SimpleLineMaterialWidgetBase
{
public:
    QGridLayout *gridLayout;
    QgsColorButton *btnAmbient;
    QLabel *lblAmbient;
    QgsPropertyOverrideButton *mAmbientDataDefinedButton;
    QSpacerItem *verticalSpacer;
    QgsMaterialPreviewWidget *mPreviewWidget;

    void setupUi(QWidget *SimpleLineMaterialWidgetBase)
    {
        if (SimpleLineMaterialWidgetBase->objectName().isEmpty())
            SimpleLineMaterialWidgetBase->setObjectName("SimpleLineMaterialWidgetBase");
        SimpleLineMaterialWidgetBase->resize(703, 317);
        SimpleLineMaterialWidgetBase->setWindowTitle(QString::fromUtf8("Form"));
        gridLayout = new QGridLayout(SimpleLineMaterialWidgetBase);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        btnAmbient = new QgsColorButton(SimpleLineMaterialWidgetBase);
        btnAmbient->setObjectName("btnAmbient");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(btnAmbient->sizePolicy().hasHeightForWidth());
        btnAmbient->setSizePolicy(sizePolicy);
        btnAmbient->setMinimumSize(QSize(120, 0));

        gridLayout->addWidget(btnAmbient, 0, 2, 1, 1);

        lblAmbient = new QLabel(SimpleLineMaterialWidgetBase);
        lblAmbient->setObjectName("lblAmbient");

        gridLayout->addWidget(lblAmbient, 0, 1, 1, 1);

        mAmbientDataDefinedButton = new QgsPropertyOverrideButton(SimpleLineMaterialWidgetBase);
        mAmbientDataDefinedButton->setObjectName("mAmbientDataDefinedButton");

        gridLayout->addWidget(mAmbientDataDefinedButton, 0, 3, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 1, 2, 1, 1);

        mPreviewWidget = new QgsMaterialPreviewWidget(SimpleLineMaterialWidgetBase);
        mPreviewWidget->setObjectName("mPreviewWidget");
        mPreviewWidget->setMinimumSize(QSize(200, 200));

        gridLayout->addWidget(mPreviewWidget, 0, 0, 2, 1);

        gridLayout->setColumnStretch(0, 2);
        gridLayout->setColumnStretch(1, 1);
        gridLayout->setColumnStretch(2, 2);

        retranslateUi(SimpleLineMaterialWidgetBase);

        QMetaObject::connectSlotsByName(SimpleLineMaterialWidgetBase);
    } // setupUi

    void retranslateUi(QWidget *SimpleLineMaterialWidgetBase)
    {
        lblAmbient->setText(QCoreApplication::translate("SimpleLineMaterialWidgetBase", "Color", nullptr));
        mAmbientDataDefinedButton->setText(QCoreApplication::translate("SimpleLineMaterialWidgetBase", "...", nullptr));
        (void)SimpleLineMaterialWidgetBase;
    } // retranslateUi

};

namespace Ui {
    class SimpleLineMaterialWidgetBase: public Ui_SimpleLineMaterialWidgetBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SIMPLELINEMATERIALWIDGETBASE_H
