/********************************************************************************
** Form generated from reading UI file 'qgslayoutchartwidgetbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QGSLAYOUTCHARTWIDGETBASE_H
#define UI_QGSLAYOUTCHARTWIDGETBASE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <qgscollapsiblegroupbox.h>
#include "qgsfieldexpressionwidget.h"
#include "qgslayoutitemcombobox.h"
#include "qgsmaplayercombobox.h"
#include "qgsscrollarea.h"

QT_BEGIN_NAMESPACE

class Ui_QgsLayoutChartWidgetBase
{
public:
    QVBoxLayout *verticalLayout_2;
    QLabel *label;
    QgsScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *mainLayout;
    QgsCollapsibleGroupBoxBasic *mainPropertiesGroupBox;
    QGridLayout *mainPropertiesLayout;
    QLabel *chartTypeLabel;
    QComboBox *mChartTypeComboBox;
    QPushButton *mChartPropertiesButton;
    QCheckBox *mFlipAxesCheckBox;
    QCheckBox *mApplyRendererStyleCheckBox;
    QgsCollapsibleGroupBoxBasic *dataSourceGroupBox;
    QGridLayout *dataSourceLayout;
    QLabel *layerLabel;
    QgsMapLayerComboBox *mLayerComboBox;
    QCheckBox *mSortCheckBox;
    QgsFieldExpressionWidget *mSortExpressionWidget;
    QToolButton *mSortDirectionButton;
    QCheckBox *mGenerateCategoriesFromRendererCheckBox;
    QHBoxLayout *seriesControlsLayout;
    QPushButton *mAddSeriesPushButton;
    QPushButton *mRemoveSeriesPushButton;
    QListWidget *mSeriesListWidget;
    QPushButton *mSeriesPropertiesButton;
    QCheckBox *mFilterOnlyVisibleFeaturesCheckBox;
    QHBoxLayout *linkedMapLayout;
    QLabel *mLinkedMapLabel;
    QgsLayoutItemComboBox *mLinkedMapComboBox;
    QCheckBox *mIntersectAtlasCheckBox;

    void setupUi(QWidget *QgsLayoutChartWidgetBase)
    {
        if (QgsLayoutChartWidgetBase->objectName().isEmpty())
            QgsLayoutChartWidgetBase->setObjectName("QgsLayoutChartWidgetBase");
        QgsLayoutChartWidgetBase->resize(542, 682);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(QgsLayoutChartWidgetBase->sizePolicy().hasHeightForWidth());
        QgsLayoutChartWidgetBase->setSizePolicy(sizePolicy);
        verticalLayout_2 = new QVBoxLayout(QgsLayoutChartWidgetBase);
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(QgsLayoutChartWidgetBase);
        label->setObjectName("label");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy1);
        label->setStyleSheet(QString::fromUtf8("padding: 2px; font-weight: bold; background-color: palette(mid);"));

        verticalLayout_2->addWidget(label);

        scrollArea = new QgsScrollArea(QgsLayoutChartWidgetBase);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 542, 660));
        mainLayout = new QVBoxLayout(scrollAreaWidgetContents);
        mainLayout->setSpacing(6);
        mainLayout->setContentsMargins(11, 11, 11, 11);
        mainLayout->setObjectName("mainLayout");
        mainPropertiesGroupBox = new QgsCollapsibleGroupBoxBasic(scrollAreaWidgetContents);
        mainPropertiesGroupBox->setObjectName("mainPropertiesGroupBox");
        mainPropertiesGroupBox->setFocusPolicy(Qt::FocusPolicy::StrongFocus);
        mainPropertiesGroupBox->setProperty("syncGroup", QVariant(QString::fromUtf8("composeritem")));
        mainPropertiesGroupBox->setProperty("collapsed", QVariant(false));
        mainPropertiesLayout = new QGridLayout(mainPropertiesGroupBox);
        mainPropertiesLayout->setSpacing(6);
        mainPropertiesLayout->setContentsMargins(11, 11, 11, 11);
        mainPropertiesLayout->setObjectName("mainPropertiesLayout");
        chartTypeLabel = new QLabel(mainPropertiesGroupBox);
        chartTypeLabel->setObjectName("chartTypeLabel");
        chartTypeLabel->setWordWrap(true);

        mainPropertiesLayout->addWidget(chartTypeLabel, 0, 0, 1, 1);

        mChartTypeComboBox = new QComboBox(mainPropertiesGroupBox);
        mChartTypeComboBox->setObjectName("mChartTypeComboBox");

        mainPropertiesLayout->addWidget(mChartTypeComboBox, 0, 1, 1, 1);

        mChartPropertiesButton = new QPushButton(mainPropertiesGroupBox);
        mChartPropertiesButton->setObjectName("mChartPropertiesButton");

        mainPropertiesLayout->addWidget(mChartPropertiesButton, 1, 0, 1, 2);

        mFlipAxesCheckBox = new QCheckBox(mainPropertiesGroupBox);
        mFlipAxesCheckBox->setObjectName("mFlipAxesCheckBox");

        mainPropertiesLayout->addWidget(mFlipAxesCheckBox, 2, 0, 1, 2);

        mApplyRendererStyleCheckBox = new QCheckBox(mainPropertiesGroupBox);
        mApplyRendererStyleCheckBox->setObjectName("mApplyRendererStyleCheckBox");

        mainPropertiesLayout->addWidget(mApplyRendererStyleCheckBox, 3, 0, 1, 2);

        mainPropertiesLayout->setColumnStretch(1, 1);

        mainLayout->addWidget(mainPropertiesGroupBox);

        dataSourceGroupBox = new QgsCollapsibleGroupBoxBasic(scrollAreaWidgetContents);
        dataSourceGroupBox->setObjectName("dataSourceGroupBox");
        dataSourceGroupBox->setFocusPolicy(Qt::FocusPolicy::StrongFocus);
        dataSourceGroupBox->setProperty("syncGroup", QVariant(QString::fromUtf8("composeritem")));
        dataSourceGroupBox->setProperty("collapsed", QVariant(false));
        dataSourceLayout = new QGridLayout(dataSourceGroupBox);
        dataSourceLayout->setSpacing(6);
        dataSourceLayout->setContentsMargins(11, 11, 11, 11);
        dataSourceLayout->setObjectName("dataSourceLayout");
        layerLabel = new QLabel(dataSourceGroupBox);
        layerLabel->setObjectName("layerLabel");
        layerLabel->setWordWrap(true);

        dataSourceLayout->addWidget(layerLabel, 0, 0, 1, 1);

        mLayerComboBox = new QgsMapLayerComboBox(dataSourceGroupBox);
        mLayerComboBox->setObjectName("mLayerComboBox");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(mLayerComboBox->sizePolicy().hasHeightForWidth());
        mLayerComboBox->setSizePolicy(sizePolicy2);

        dataSourceLayout->addWidget(mLayerComboBox, 0, 1, 1, 2);

        mSortCheckBox = new QCheckBox(dataSourceGroupBox);
        mSortCheckBox->setObjectName("mSortCheckBox");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(mSortCheckBox->sizePolicy().hasHeightForWidth());
        mSortCheckBox->setSizePolicy(sizePolicy3);

        dataSourceLayout->addWidget(mSortCheckBox, 1, 0, 1, 1);

        mSortExpressionWidget = new QgsFieldExpressionWidget(dataSourceGroupBox);
        mSortExpressionWidget->setObjectName("mSortExpressionWidget");

        dataSourceLayout->addWidget(mSortExpressionWidget, 1, 1, 1, 1);

        mSortDirectionButton = new QToolButton(dataSourceGroupBox);
        mSortDirectionButton->setObjectName("mSortDirectionButton");
        mSortDirectionButton->setArrowType(Qt::ArrowType::UpArrow);

        dataSourceLayout->addWidget(mSortDirectionButton, 1, 2, 1, 1);

        mGenerateCategoriesFromRendererCheckBox = new QCheckBox(dataSourceGroupBox);
        mGenerateCategoriesFromRendererCheckBox->setObjectName("mGenerateCategoriesFromRendererCheckBox");

        dataSourceLayout->addWidget(mGenerateCategoriesFromRendererCheckBox, 2, 0, 1, 3);

        seriesControlsLayout = new QHBoxLayout();
        seriesControlsLayout->setSpacing(6);
        seriesControlsLayout->setObjectName("seriesControlsLayout");
        mAddSeriesPushButton = new QPushButton(dataSourceGroupBox);
        mAddSeriesPushButton->setObjectName("mAddSeriesPushButton");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/images/themes/default/symbologyAdd.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mAddSeriesPushButton->setIcon(icon);

        seriesControlsLayout->addWidget(mAddSeriesPushButton);

        mRemoveSeriesPushButton = new QPushButton(dataSourceGroupBox);
        mRemoveSeriesPushButton->setObjectName("mRemoveSeriesPushButton");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/images/themes/default/symbologyRemove.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mRemoveSeriesPushButton->setIcon(icon1);

        seriesControlsLayout->addWidget(mRemoveSeriesPushButton);


        dataSourceLayout->addLayout(seriesControlsLayout, 3, 0, 1, 3);

        mSeriesListWidget = new QListWidget(dataSourceGroupBox);
        mSeriesListWidget->setObjectName("mSeriesListWidget");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(mSeriesListWidget->sizePolicy().hasHeightForWidth());
        mSeriesListWidget->setSizePolicy(sizePolicy4);
        mSeriesListWidget->setMaximumSize(QSize(16777215, 90));

        dataSourceLayout->addWidget(mSeriesListWidget, 4, 0, 1, 3);

        mSeriesPropertiesButton = new QPushButton(dataSourceGroupBox);
        mSeriesPropertiesButton->setObjectName("mSeriesPropertiesButton");

        dataSourceLayout->addWidget(mSeriesPropertiesButton, 5, 0, 1, 3);

        mFilterOnlyVisibleFeaturesCheckBox = new QCheckBox(dataSourceGroupBox);
        mFilterOnlyVisibleFeaturesCheckBox->setObjectName("mFilterOnlyVisibleFeaturesCheckBox");

        dataSourceLayout->addWidget(mFilterOnlyVisibleFeaturesCheckBox, 6, 0, 1, 3);

        linkedMapLayout = new QHBoxLayout();
        linkedMapLayout->setSpacing(6);
        linkedMapLayout->setObjectName("linkedMapLayout");
        mLinkedMapLabel = new QLabel(dataSourceGroupBox);
        mLinkedMapLabel->setObjectName("mLinkedMapLabel");

        linkedMapLayout->addWidget(mLinkedMapLabel);

        mLinkedMapComboBox = new QgsLayoutItemComboBox(dataSourceGroupBox);
        mLinkedMapComboBox->setObjectName("mLinkedMapComboBox");

        linkedMapLayout->addWidget(mLinkedMapComboBox);


        dataSourceLayout->addLayout(linkedMapLayout, 7, 0, 1, 3);

        mIntersectAtlasCheckBox = new QCheckBox(dataSourceGroupBox);
        mIntersectAtlasCheckBox->setObjectName("mIntersectAtlasCheckBox");

        dataSourceLayout->addWidget(mIntersectAtlasCheckBox, 8, 0, 1, 3);

        dataSourceLayout->setColumnStretch(1, 1);

        mainLayout->addWidget(dataSourceGroupBox);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout_2->addWidget(scrollArea);

#if QT_CONFIG(shortcut)
        mLinkedMapLabel->setBuddy(mLinkedMapComboBox);
#endif // QT_CONFIG(shortcut)

        retranslateUi(QgsLayoutChartWidgetBase);

        QMetaObject::connectSlotsByName(QgsLayoutChartWidgetBase);
    } // setupUi

    void retranslateUi(QWidget *QgsLayoutChartWidgetBase)
    {
        QgsLayoutChartWidgetBase->setWindowTitle(QCoreApplication::translate("QgsLayoutChartWidgetBase", "Chart Options", nullptr));
        label->setText(QCoreApplication::translate("QgsLayoutChartWidgetBase", "Chart", nullptr));
        mainPropertiesGroupBox->setTitle(QCoreApplication::translate("QgsLayoutChartWidgetBase", "Main Properties", nullptr));
        chartTypeLabel->setText(QCoreApplication::translate("QgsLayoutChartWidgetBase", "Chart type", nullptr));
        mChartPropertiesButton->setText(QCoreApplication::translate("QgsLayoutChartWidgetBase", "Modify Chart Properties\342\200\246", nullptr));
        mFlipAxesCheckBox->setText(QCoreApplication::translate("QgsLayoutChartWidgetBase", "Flip axes", nullptr));
        mApplyRendererStyleCheckBox->setText(QCoreApplication::translate("QgsLayoutChartWidgetBase", "Apply layer symbology colors", nullptr));
        dataSourceGroupBox->setTitle(QCoreApplication::translate("QgsLayoutChartWidgetBase", "Data Source", nullptr));
        layerLabel->setText(QCoreApplication::translate("QgsLayoutChartWidgetBase", "Layer", nullptr));
        mSortCheckBox->setText(QCoreApplication::translate("QgsLayoutChartWidgetBase", "Sort by", nullptr));
#if QT_CONFIG(tooltip)
        mSortDirectionButton->setToolTip(QCoreApplication::translate("QgsLayoutChartWidgetBase", "Sort direction", nullptr));
#endif // QT_CONFIG(tooltip)
        mSortDirectionButton->setText(QCoreApplication::translate("QgsLayoutChartWidgetBase", "\342\200\246", nullptr));
        mGenerateCategoriesFromRendererCheckBox->setText(QCoreApplication::translate("QgsLayoutChartWidgetBase", "Generate categories from the layer symbology", nullptr));
#if QT_CONFIG(tooltip)
        mAddSeriesPushButton->setToolTip(QCoreApplication::translate("QgsLayoutChartWidgetBase", "Add a series", nullptr));
#endif // QT_CONFIG(tooltip)
        mAddSeriesPushButton->setText(QString());
#if QT_CONFIG(tooltip)
        mRemoveSeriesPushButton->setToolTip(QCoreApplication::translate("QgsLayoutChartWidgetBase", "Remove selected series", nullptr));
#endif // QT_CONFIG(tooltip)
        mRemoveSeriesPushButton->setText(QString());
        mSeriesPropertiesButton->setText(QCoreApplication::translate("QgsLayoutChartWidgetBase", "Modify Series\342\200\246", nullptr));
        mFilterOnlyVisibleFeaturesCheckBox->setText(QCoreApplication::translate("QgsLayoutChartWidgetBase", "Use only features visible within a map", nullptr));
        mLinkedMapLabel->setText(QCoreApplication::translate("QgsLayoutChartWidgetBase", "Linked map", nullptr));
        mIntersectAtlasCheckBox->setText(QCoreApplication::translate("QgsLayoutChartWidgetBase", "Use only features intersecting atlas feature", nullptr));
    } // retranslateUi

};

namespace Ui {
    class QgsLayoutChartWidgetBase: public Ui_QgsLayoutChartWidgetBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QGSLAYOUTCHARTWIDGETBASE_H
