/********************************************************************************
** Form generated from reading UI file 'skyboxrenderingsettingswidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SKYBOXRENDERINGSETTINGSWIDGET_H
#define UI_SKYBOXRENDERINGSETTINGSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QWidget>
#include "qgsfilecontentsourcelineedit.h"
#include "qgspercentagewidget.h"

QT_BEGIN_NAMESPACE

class Ui_SkyboxRenderingSettingsWidget
{
public:
    QGridLayout *gridLayout_2;
    QWidget *mCubeMapWidget;
    QGridLayout *faceTexturesLayout;
    QLabel *posZImageSourceLabel;
    QLabel *posXImageSourceLabel;
    QgsImageSourceLineEdit *negYImageSource;
    QgsImageSourceLineEdit *posXImageSource;
    QgsImageSourceLineEdit *negZImageSource;
    QComboBox *mMappingComboBox;
    QLabel *label;
    QLabel *posYImageSourceLabel;
    QgsImageSourceLineEdit *posZImageSource;
    QgsImageSourceLineEdit *negXImageSource;
    QgsImageSourceLineEdit *posYImageSource;
    QLabel *negYImageSourceLabel;
    QLabel *negXImageSourceLabel;
    QLabel *negZImageSourceLabel;
    QGroupBox *mGroupEnvironmentLight;
    QGridLayout *gridLayout;
    QLabel *label_2;
    QgsPercentageWidget *mEnvironmentLightStrengthWidget;

    void setupUi(QWidget *SkyboxRenderingSettingsWidget)
    {
        if (SkyboxRenderingSettingsWidget->objectName().isEmpty())
            SkyboxRenderingSettingsWidget->setObjectName("SkyboxRenderingSettingsWidget");
        SkyboxRenderingSettingsWidget->resize(354, 257);
        gridLayout_2 = new QGridLayout(SkyboxRenderingSettingsWidget);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        mCubeMapWidget = new QWidget(SkyboxRenderingSettingsWidget);
        mCubeMapWidget->setObjectName("mCubeMapWidget");
        faceTexturesLayout = new QGridLayout(mCubeMapWidget);
        faceTexturesLayout->setObjectName("faceTexturesLayout");
        faceTexturesLayout->setContentsMargins(0, 0, 0, 0);
        posZImageSourceLabel = new QLabel(mCubeMapWidget);
        posZImageSourceLabel->setObjectName("posZImageSourceLabel");

        faceTexturesLayout->addWidget(posZImageSourceLabel, 6, 0, 1, 1);

        posXImageSourceLabel = new QLabel(mCubeMapWidget);
        posXImageSourceLabel->setObjectName("posXImageSourceLabel");

        faceTexturesLayout->addWidget(posXImageSourceLabel, 4, 0, 1, 1);

        negYImageSource = new QgsImageSourceLineEdit(mCubeMapWidget);
        negYImageSource->setObjectName("negYImageSource");

        faceTexturesLayout->addWidget(negYImageSource, 2, 1, 1, 1);

        posXImageSource = new QgsImageSourceLineEdit(mCubeMapWidget);
        posXImageSource->setObjectName("posXImageSource");

        faceTexturesLayout->addWidget(posXImageSource, 4, 1, 1, 1);

        negZImageSource = new QgsImageSourceLineEdit(mCubeMapWidget);
        negZImageSource->setObjectName("negZImageSource");

        faceTexturesLayout->addWidget(negZImageSource, 3, 1, 1, 1);

        mMappingComboBox = new QComboBox(mCubeMapWidget);
        mMappingComboBox->setObjectName("mMappingComboBox");

        faceTexturesLayout->addWidget(mMappingComboBox, 0, 1, 1, 1);

        label = new QLabel(mCubeMapWidget);
        label->setObjectName("label");

        faceTexturesLayout->addWidget(label, 0, 0, 1, 1);

        posYImageSourceLabel = new QLabel(mCubeMapWidget);
        posYImageSourceLabel->setObjectName("posYImageSourceLabel");

        faceTexturesLayout->addWidget(posYImageSourceLabel, 5, 0, 1, 1);

        posZImageSource = new QgsImageSourceLineEdit(mCubeMapWidget);
        posZImageSource->setObjectName("posZImageSource");

        faceTexturesLayout->addWidget(posZImageSource, 6, 1, 1, 1);

        negXImageSource = new QgsImageSourceLineEdit(mCubeMapWidget);
        negXImageSource->setObjectName("negXImageSource");

        faceTexturesLayout->addWidget(negXImageSource, 1, 1, 1, 1);

        posYImageSource = new QgsImageSourceLineEdit(mCubeMapWidget);
        posYImageSource->setObjectName("posYImageSource");

        faceTexturesLayout->addWidget(posYImageSource, 5, 1, 1, 1);

        negYImageSourceLabel = new QLabel(mCubeMapWidget);
        negYImageSourceLabel->setObjectName("negYImageSourceLabel");

        faceTexturesLayout->addWidget(negYImageSourceLabel, 2, 0, 1, 1);

        negXImageSourceLabel = new QLabel(mCubeMapWidget);
        negXImageSourceLabel->setObjectName("negXImageSourceLabel");

        faceTexturesLayout->addWidget(negXImageSourceLabel, 1, 0, 1, 1);

        negZImageSourceLabel = new QLabel(mCubeMapWidget);
        negZImageSourceLabel->setObjectName("negZImageSourceLabel");

        faceTexturesLayout->addWidget(negZImageSourceLabel, 3, 0, 1, 1);

        faceTexturesLayout->setColumnStretch(1, 1);

        gridLayout_2->addWidget(mCubeMapWidget, 0, 1, 1, 2);

        mGroupEnvironmentLight = new QGroupBox(SkyboxRenderingSettingsWidget);
        mGroupEnvironmentLight->setObjectName("mGroupEnvironmentLight");
        mGroupEnvironmentLight->setCheckable(true);
        gridLayout = new QGridLayout(mGroupEnvironmentLight);
        gridLayout->setObjectName("gridLayout");
        label_2 = new QLabel(mGroupEnvironmentLight);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 0, 0, 1, 1);

        mEnvironmentLightStrengthWidget = new QgsPercentageWidget(mGroupEnvironmentLight);
        mEnvironmentLightStrengthWidget->setObjectName("mEnvironmentLightStrengthWidget");
        mEnvironmentLightStrengthWidget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        gridLayout->addWidget(mEnvironmentLightStrengthWidget, 0, 1, 1, 1);

        gridLayout->setColumnStretch(1, 1);

        gridLayout_2->addWidget(mGroupEnvironmentLight, 1, 1, 1, 2);

        QWidget::setTabOrder(mMappingComboBox, mGroupEnvironmentLight);
        QWidget::setTabOrder(mGroupEnvironmentLight, mEnvironmentLightStrengthWidget);

        retranslateUi(SkyboxRenderingSettingsWidget);

        QMetaObject::connectSlotsByName(SkyboxRenderingSettingsWidget);
    } // setupUi

    void retranslateUi(QWidget *SkyboxRenderingSettingsWidget)
    {
        posZImageSourceLabel->setText(QCoreApplication::translate("SkyboxRenderingSettingsWidget", "Top (+Z)", nullptr));
        posXImageSourceLabel->setText(QCoreApplication::translate("SkyboxRenderingSettingsWidget", "Right (+X)", nullptr));
        label->setText(QCoreApplication::translate("SkyboxRenderingSettingsWidget", "Convention", nullptr));
        posYImageSourceLabel->setText(QCoreApplication::translate("SkyboxRenderingSettingsWidget", "Front (+Y)", nullptr));
        negYImageSourceLabel->setText(QCoreApplication::translate("SkyboxRenderingSettingsWidget", "Back (-Y)", nullptr));
        negXImageSourceLabel->setText(QCoreApplication::translate("SkyboxRenderingSettingsWidget", "Left (-X)", nullptr));
        negZImageSourceLabel->setText(QCoreApplication::translate("SkyboxRenderingSettingsWidget", "Down (-Z)", nullptr));
#if QT_CONFIG(tooltip)
        mGroupEnvironmentLight->setToolTip(QCoreApplication::translate("SkyboxRenderingSettingsWidget", "When checked, the scene's lighting will reflect the skybox appearance", nullptr));
#endif // QT_CONFIG(tooltip)
        mGroupEnvironmentLight->setTitle(QCoreApplication::translate("SkyboxRenderingSettingsWidget", "Enable environmental lighting effects", nullptr));
        label_2->setText(QCoreApplication::translate("SkyboxRenderingSettingsWidget", "Strength", nullptr));
        (void)SkyboxRenderingSettingsWidget;
    } // retranslateUi

};

namespace Ui {
    class SkyboxRenderingSettingsWidget: public Ui_SkyboxRenderingSettingsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SKYBOXRENDERINGSETTINGSWIDGET_H
