/********************************************************************************
** Form generated from reading UI file 'qgsgroupwmsdatadialogbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QGSGROUPWMSDATADIALOGBASE_H
#define UI_QGSGROUPWMSDATADIALOGBASE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "qgsmaplayerserverpropertieswidget.h"
#include "qgsscrollarea.h"

QT_BEGIN_NAMESPACE

class Ui_QgsGroupWMSDataDialogBase
{
public:
    QVBoxLayout *verticalLayout;
    QgsScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_2;
    QgsMapLayerServerPropertiesWidget *mMapLayerServerPropertiesWidget;
    QHBoxLayout *group_request_layout;
    QLabel *label;
    QComboBox *mGroupRequestModeCombo;
    QCheckBox *mComputeTimeDimension;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *QgsGroupWMSDataDialogBase)
    {
        if (QgsGroupWMSDataDialogBase->objectName().isEmpty())
            QgsGroupWMSDataDialogBase->setObjectName("QgsGroupWMSDataDialogBase");
        QgsGroupWMSDataDialogBase->setWindowModality(Qt::WindowModality::WindowModal);
        QgsGroupWMSDataDialogBase->resize(566, 212);
        QIcon icon;
        icon.addFile(QString::fromUtf8("."), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        QgsGroupWMSDataDialogBase->setWindowIcon(icon);
        verticalLayout = new QVBoxLayout(QgsGroupWMSDataDialogBase);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        scrollArea = new QgsScrollArea(QgsGroupWMSDataDialogBase);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setFrameShape(QFrame::Shape::NoFrame);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 548, 162));
        verticalLayout_2 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        mMapLayerServerPropertiesWidget = new QgsMapLayerServerPropertiesWidget(scrollAreaWidgetContents);
        mMapLayerServerPropertiesWidget->setObjectName("mMapLayerServerPropertiesWidget");

        verticalLayout_2->addWidget(mMapLayerServerPropertiesWidget);

        group_request_layout = new QHBoxLayout();
        group_request_layout->setSpacing(6);
        group_request_layout->setObjectName("group_request_layout");
        label = new QLabel(scrollAreaWidgetContents);
        label->setObjectName("label");

        group_request_layout->addWidget(label);

        mGroupRequestModeCombo = new QComboBox(scrollAreaWidgetContents);
        mGroupRequestModeCombo->setObjectName("mGroupRequestModeCombo");

        group_request_layout->addWidget(mGroupRequestModeCombo);


        verticalLayout_2->addLayout(group_request_layout);

        mComputeTimeDimension = new QCheckBox(scrollAreaWidgetContents);
        mComputeTimeDimension->setObjectName("mComputeTimeDimension");

        verticalLayout_2->addWidget(mComputeTimeDimension);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout->addWidget(scrollArea);

        buttonBox = new QDialogButtonBox(QgsGroupWMSDataDialogBase);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::StandardButton::Cancel|QDialogButtonBox::StandardButton::Save);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(QgsGroupWMSDataDialogBase);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, QgsGroupWMSDataDialogBase, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, QgsGroupWMSDataDialogBase, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(QgsGroupWMSDataDialogBase);
    } // setupUi

    void retranslateUi(QDialog *QgsGroupWMSDataDialogBase)
    {
        QgsGroupWMSDataDialogBase->setWindowTitle(QCoreApplication::translate("QgsGroupWMSDataDialogBase", "Set Group WMS Properties", nullptr));
        label->setText(QCoreApplication::translate("QgsGroupWMSDataDialogBase", "Group request mode", nullptr));
#if QT_CONFIG(tooltip)
        mGroupRequestModeCombo->setToolTip(QCoreApplication::translate("QgsGroupWMSDataDialogBase", "<html><head/><body><p>An <span style=\" font-weight:700;\">opaque</span> layer group behaves like a single layer. It can be requested, but its children cannot. Still the children are rendered in GetMap, GetFeatureInfo, etc. when requesting the opaque layer group.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        mComputeTimeDimension->setToolTip(QCoreApplication::translate("QgsGroupWMSDataDialogBase", "When a GetCapabilities request is sent, QGIS server will return a TIME dimension computed as an union of all time dimension of its children recursively", nullptr));
#endif // QT_CONFIG(tooltip)
        mComputeTimeDimension->setText(QCoreApplication::translate("QgsGroupWMSDataDialogBase", "Compute TIME dimension from children", nullptr));
    } // retranslateUi

};

namespace Ui {
    class QgsGroupWMSDataDialogBase: public Ui_QgsGroupWMSDataDialogBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QGSGROUPWMSDATADIALOGBASE_H
