/********************************************************************************
** Form generated from reading UI file 'qgs3dcameracontrolswidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QGS3DCAMERACONTROLSWIDGET_H
#define UI_QGS3DCAMERACONTROLSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Qgs3DCameraControlsWidget
{
public:
    QGridLayout *gridLayout;
    QDoubleSpinBox *mCameraZ;
    QLabel *labelDistanceFromCenterPoint;
    QLabel *labelY;
    QDoubleSpinBox *mCameraHeading;
    QDoubleSpinBox *mCameraY;
    QDoubleSpinBox *mCameraX;
    QHBoxLayout *horizontalLayout;
    QDialogButtonBox *mHelpButtonBox;
    QSpacerItem *horizontalSpacer;
    QCheckBox *mLiveApplyCheck;
    QDialogButtonBox *mButtonBox;
    QSpacerItem *verticalSpacer;
    QDoubleSpinBox *mCameraPitch;
    QLabel *labelZ;
    QLabel *labelCRS;
    QDoubleSpinBox *mCameraDistanceFromCenterPoint;
    QLabel *labelX;
    QLabel *labelHeading;
    QLabel *labelPitch;

    void setupUi(QWidget *Qgs3DCameraControlsWidget)
    {
        if (Qgs3DCameraControlsWidget->objectName().isEmpty())
            Qgs3DCameraControlsWidget->setObjectName("Qgs3DCameraControlsWidget");
        Qgs3DCameraControlsWidget->resize(426, 316);
        Qgs3DCameraControlsWidget->setWindowTitle(QString::fromUtf8("Camera Controls"));
        gridLayout = new QGridLayout(Qgs3DCameraControlsWidget);
        gridLayout->setObjectName("gridLayout");
        mCameraZ = new QDoubleSpinBox(Qgs3DCameraControlsWidget);
        mCameraZ->setObjectName("mCameraZ");

        gridLayout->addWidget(mCameraZ, 3, 1, 1, 1);

        labelDistanceFromCenterPoint = new QLabel(Qgs3DCameraControlsWidget);
        labelDistanceFromCenterPoint->setObjectName("labelDistanceFromCenterPoint");

        gridLayout->addWidget(labelDistanceFromCenterPoint, 5, 0, 1, 1);

        labelY = new QLabel(Qgs3DCameraControlsWidget);
        labelY->setObjectName("labelY");

        gridLayout->addWidget(labelY, 2, 0, 1, 1);

        mCameraHeading = new QDoubleSpinBox(Qgs3DCameraControlsWidget);
        mCameraHeading->setObjectName("mCameraHeading");
        mCameraHeading->setDecimals(0);

        gridLayout->addWidget(mCameraHeading, 7, 1, 1, 1);

        mCameraY = new QDoubleSpinBox(Qgs3DCameraControlsWidget);
        mCameraY->setObjectName("mCameraY");

        gridLayout->addWidget(mCameraY, 2, 1, 1, 1);

        mCameraX = new QDoubleSpinBox(Qgs3DCameraControlsWidget);
        mCameraX->setObjectName("mCameraX");

        gridLayout->addWidget(mCameraX, 1, 1, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        mHelpButtonBox = new QDialogButtonBox(Qgs3DCameraControlsWidget);
        mHelpButtonBox->setObjectName("mHelpButtonBox");
        mHelpButtonBox->setStandardButtons(QDialogButtonBox::StandardButton::Help);

        horizontalLayout->addWidget(mHelpButtonBox);

        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        mLiveApplyCheck = new QCheckBox(Qgs3DCameraControlsWidget);
        mLiveApplyCheck->setObjectName("mLiveApplyCheck");
        mLiveApplyCheck->setChecked(true);
        mLiveApplyCheck->setAutoRepeat(false);

        horizontalLayout->addWidget(mLiveApplyCheck);

        mButtonBox = new QDialogButtonBox(Qgs3DCameraControlsWidget);
        mButtonBox->setObjectName("mButtonBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(mButtonBox->sizePolicy().hasHeightForWidth());
        mButtonBox->setSizePolicy(sizePolicy);
        mButtonBox->setStandardButtons(QDialogButtonBox::StandardButton::Apply);

        horizontalLayout->addWidget(mButtonBox);


        gridLayout->addLayout(horizontalLayout, 9, 0, 1, 2);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 8, 0, 1, 1);

        mCameraPitch = new QDoubleSpinBox(Qgs3DCameraControlsWidget);
        mCameraPitch->setObjectName("mCameraPitch");
        mCameraPitch->setDecimals(0);

        gridLayout->addWidget(mCameraPitch, 6, 1, 1, 1);

        labelZ = new QLabel(Qgs3DCameraControlsWidget);
        labelZ->setObjectName("labelZ");

        gridLayout->addWidget(labelZ, 3, 0, 1, 1);

        labelCRS = new QLabel(Qgs3DCameraControlsWidget);
        labelCRS->setObjectName("labelCRS");

        gridLayout->addWidget(labelCRS, 0, 0, 1, 2);

        mCameraDistanceFromCenterPoint = new QDoubleSpinBox(Qgs3DCameraControlsWidget);
        mCameraDistanceFromCenterPoint->setObjectName("mCameraDistanceFromCenterPoint");

        gridLayout->addWidget(mCameraDistanceFromCenterPoint, 5, 1, 1, 1);

        labelX = new QLabel(Qgs3DCameraControlsWidget);
        labelX->setObjectName("labelX");

        gridLayout->addWidget(labelX, 1, 0, 1, 1);

        labelHeading = new QLabel(Qgs3DCameraControlsWidget);
        labelHeading->setObjectName("labelHeading");

        gridLayout->addWidget(labelHeading, 7, 0, 1, 1);

        labelPitch = new QLabel(Qgs3DCameraControlsWidget);
        labelPitch->setObjectName("labelPitch");

        gridLayout->addWidget(labelPitch, 6, 0, 1, 1);

        QWidget::setTabOrder(mCameraX, mCameraY);
        QWidget::setTabOrder(mCameraY, mCameraZ);
        QWidget::setTabOrder(mCameraZ, mCameraDistanceFromCenterPoint);
        QWidget::setTabOrder(mCameraDistanceFromCenterPoint, mCameraPitch);
        QWidget::setTabOrder(mCameraPitch, mCameraHeading);
        QWidget::setTabOrder(mCameraHeading, mLiveApplyCheck);

        retranslateUi(Qgs3DCameraControlsWidget);

        QMetaObject::connectSlotsByName(Qgs3DCameraControlsWidget);
    } // setupUi

    void retranslateUi(QWidget *Qgs3DCameraControlsWidget)
    {
        labelDistanceFromCenterPoint->setText(QCoreApplication::translate("Qgs3DCameraControlsWidget", "Distance from center", nullptr));
        labelY->setText(QCoreApplication::translate("Qgs3DCameraControlsWidget", "Looking at Y", nullptr));
        mCameraHeading->setSuffix(QCoreApplication::translate("Qgs3DCameraControlsWidget", "\302\260", nullptr));
#if QT_CONFIG(tooltip)
        mLiveApplyCheck->setToolTip(QCoreApplication::translate("Qgs3DCameraControlsWidget", "If checked, the map canvas will automatically update whenever an option has been changed without the requirement to click Apply", nullptr));
#endif // QT_CONFIG(tooltip)
        mLiveApplyCheck->setText(QCoreApplication::translate("Qgs3DCameraControlsWidget", "Live update", nullptr));
        mCameraPitch->setSuffix(QCoreApplication::translate("Qgs3DCameraControlsWidget", "\302\260", nullptr));
        labelZ->setText(QCoreApplication::translate("Qgs3DCameraControlsWidget", "Looking at Z", nullptr));
        labelCRS->setText(QCoreApplication::translate("Qgs3DCameraControlsWidget", "Coordinates in 3D Map CRS (EPSG: #CODE)", nullptr));
        labelX->setText(QCoreApplication::translate("Qgs3DCameraControlsWidget", "Looking at X", nullptr));
        labelHeading->setText(QCoreApplication::translate("Qgs3DCameraControlsWidget", "Heading", nullptr));
        labelPitch->setText(QCoreApplication::translate("Qgs3DCameraControlsWidget", "Pitch", nullptr));
        (void)Qgs3DCameraControlsWidget;
    } // retranslateUi

};

namespace Ui {
    class Qgs3DCameraControlsWidget: public Ui_Qgs3DCameraControlsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QGS3DCAMERACONTROLSWIDGET_H
