/********************************************************************************
** Form generated from reading UI file 'widget_templatedline.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_WIDGET_TEMPLATEDLINE_H
#define UI_WIDGET_TEMPLATEDLINE_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <qgscollapsiblegroupbox.h>
#include "qgscollapsiblegroupbox.h"
#include "qgsdoublespinbox.h"
#include "qgspropertyoverridebutton.h"
#include "qgsunitselectionwidget.h"

QT_BEGIN_NAMESPACE

class Ui_WidgetTemplatedLine
{
public:
    QAction *mEditBlankSegmentsAction;
    QAction *mAddExtraItemAction;
    QAction *mModifyExtraItemAction;
    QGridLayout *gridLayout_2;
    QGridLayout *gridLayout;
    QgsDoubleSpinBox *mHashRotationSpinBox;
    QSpacerItem *horizontalSpacer_3;
    QgsPropertyOverrideButton *mHashLengthDDBtn;
    QgsDoubleSpinBox *mSpinAverageAngleLength;
    QgsDoubleSpinBox *spinOffset;
    QLabel *mHashRotationLabel;
    QgsUnitSelectionWidget *mOffsetUnitWidget;
    QLabel *label;
    QLabel *label_6;
    QLabel *mHashLengthLabel;
    QgsPropertyOverrideButton *mHashRotationDDBtn;
    QgsUnitSelectionWidget *mHashLengthUnitWidget;
    QgsUnitSelectionWidget *mOffsetAlongLineUnitWidget;
    QCheckBox *chkRotateMarker;
    QgsUnitSelectionWidget *mAverageAngleUnit;
    QLabel *label_3;
    QgsPropertyOverrideButton *mOffsetAlongLineDDBtn;
    QgsDoubleSpinBox *mSpinHashLength;
    QLabel *mRingsLabel;
    QgsDoubleSpinBox *mSpinOffsetAlongLine;
    QComboBox *mRingFilterComboBox;
    QgsPropertyOverrideButton *mAverageAngleDDBtn;
    QgsPropertyOverrideButton *mLineOffsetDDBtn;
    QCheckBox *mCheckPlaceOnEveryPart;
    QGridLayout *gridLayout_3;
    QHBoxLayout *horizontalLayout_4;
    QCheckBox *mCheckInterval;
    QgsDoubleSpinBox *spinInterval;
    QgsUnitSelectionWidget *mIntervalUnitWidget;
    QSpacerItem *horizontalSpacer_2;
    QgsPropertyOverrideButton *mIntervalDDBtn;
    QCheckBox *mCheckVertex;
    QCheckBox *mCheckVertexLast;
    QCheckBox *mCheckVertexFirst;
    QCheckBox *mCheckCentralPoint;
    QCheckBox *mCheckSegmentCentralPoint;
    QCheckBox *mCheckCurvePoint;
    QSpacerItem *verticalSpacer;
    QgsCollapsibleGroupBox *mPerFeatureCustomizationGrpBox;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_9;
    QLabel *label_2;
    QSpacerItem *horizontalSpacer_4;
    QToolButton *mEditBlankSegmentsBtn;
    QgsUnitSelectionWidget *mBlankSegmentsUnitWidget;
    QgsPropertyOverrideButton *mBlankSegmentsDDButton;
    QHBoxLayout *horizontalLayout;
    QLabel *lblExtraItems;
    QSpacerItem *horizontalSpacer_5;
    QToolButton *mAddExtraItemBtn;
    QToolButton *mModifyExtraItemBtn;
    QgsPropertyOverrideButton *mExtraItemsDDBtn;
    QgsPropertyOverrideButton *mPlacementDDBtn;
    QLabel *mPlacementLabel;
    QSpacerItem *horizontalSpacer;
    QgsCollapsibleGroupBoxBasic *mTrimGroupBox;
    QGridLayout *gridLayout_5;
    QgsPropertyOverrideButton *mTrimDistanceEndDDBtn;
    QHBoxLayout *horizontalLayout_7;
    QgsDoubleSpinBox *mTrimStartDistanceSpin;
    QgsUnitSelectionWidget *mTrimDistanceStartUnitWidget;
    QHBoxLayout *horizontalLayout_8;
    QgsDoubleSpinBox *mTrimDistanceEndSpin;
    QgsUnitSelectionWidget *mTrimDistanceEndUnitWidget;
    QLabel *label_10;
    QLabel *label_11;
    QgsPropertyOverrideButton *mTrimDistanceStartDDBtn;

    void setupUi(QWidget *WidgetTemplatedLine)
    {
        if (WidgetTemplatedLine->objectName().isEmpty())
            WidgetTemplatedLine->setObjectName("WidgetTemplatedLine");
        WidgetTemplatedLine->resize(385, 830);
        WidgetTemplatedLine->setWindowTitle(QString::fromUtf8("Form"));
        mEditBlankSegmentsAction = new QAction(WidgetTemplatedLine);
        mEditBlankSegmentsAction->setObjectName("mEditBlankSegmentsAction");
        mEditBlankSegmentsAction->setCheckable(true);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/images/themes/default/mActionShowAllLayers.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mEditBlankSegmentsAction->setIcon(icon);
        mEditBlankSegmentsAction->setMenuRole(QAction::MenuRole::NoRole);
        mAddExtraItemAction = new QAction(WidgetTemplatedLine);
        mAddExtraItemAction->setObjectName("mAddExtraItemAction");
        mAddExtraItemAction->setCheckable(true);
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/images/themes/default/mActionAdd.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mAddExtraItemAction->setIcon(icon1);
        mAddExtraItemAction->setMenuRole(QAction::MenuRole::NoRole);
        mModifyExtraItemAction = new QAction(WidgetTemplatedLine);
        mModifyExtraItemAction->setObjectName("mModifyExtraItemAction");
        mModifyExtraItemAction->setCheckable(true);
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/images/themes/default/mActionSelect.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mModifyExtraItemAction->setIcon(icon2);
        mModifyExtraItemAction->setMenuRole(QAction::MenuRole::NoRole);
        gridLayout_2 = new QGridLayout(WidgetTemplatedLine);
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(1, -1, 1, 1);
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(-1, 0, -1, -1);
        mHashRotationSpinBox = new QgsDoubleSpinBox(WidgetTemplatedLine);
        mHashRotationSpinBox->setObjectName("mHashRotationSpinBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(mHashRotationSpinBox->sizePolicy().hasHeightForWidth());
        mHashRotationSpinBox->setSizePolicy(sizePolicy);
        mHashRotationSpinBox->setWrapping(true);
        mHashRotationSpinBox->setDecimals(2);
        mHashRotationSpinBox->setMinimum(-360.000000000000000);
        mHashRotationSpinBox->setMaximum(360.000000000000000);
        mHashRotationSpinBox->setSingleStep(0.500000000000000);

        gridLayout->addWidget(mHashRotationSpinBox, 3, 2, 1, 1);

        horizontalSpacer_3 = new QSpacerItem(10, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_3, 5, 0, 1, 1);

        mHashLengthDDBtn = new QgsPropertyOverrideButton(WidgetTemplatedLine);
        mHashLengthDDBtn->setObjectName("mHashLengthDDBtn");

        gridLayout->addWidget(mHashLengthDDBtn, 2, 4, 1, 1);

        mSpinAverageAngleLength = new QgsDoubleSpinBox(WidgetTemplatedLine);
        mSpinAverageAngleLength->setObjectName("mSpinAverageAngleLength");
        sizePolicy.setHeightForWidth(mSpinAverageAngleLength->sizePolicy().hasHeightForWidth());
        mSpinAverageAngleLength->setSizePolicy(sizePolicy);
        mSpinAverageAngleLength->setDecimals(6);
        mSpinAverageAngleLength->setMaximum(10000000.000000000000000);
        mSpinAverageAngleLength->setSingleStep(0.200000000000000);
        mSpinAverageAngleLength->setValue(1.000000000000000);

        gridLayout->addWidget(mSpinAverageAngleLength, 5, 2, 1, 1);

        spinOffset = new QgsDoubleSpinBox(WidgetTemplatedLine);
        spinOffset->setObjectName("spinOffset");
        sizePolicy.setHeightForWidth(spinOffset->sizePolicy().hasHeightForWidth());
        spinOffset->setSizePolicy(sizePolicy);
        spinOffset->setDecimals(6);
        spinOffset->setMinimum(-999999999.000000000000000);
        spinOffset->setMaximum(999999999.000000000000000);
        spinOffset->setSingleStep(0.200000000000000);

        gridLayout->addWidget(spinOffset, 6, 2, 1, 1);

        mHashRotationLabel = new QLabel(WidgetTemplatedLine);
        mHashRotationLabel->setObjectName("mHashRotationLabel");

        gridLayout->addWidget(mHashRotationLabel, 3, 0, 1, 2);

        mOffsetUnitWidget = new QgsUnitSelectionWidget(WidgetTemplatedLine);
        mOffsetUnitWidget->setObjectName("mOffsetUnitWidget");
        mOffsetUnitWidget->setMinimumSize(QSize(20, 0));
        mOffsetUnitWidget->setFocusPolicy(Qt::FocusPolicy::TabFocus);

        gridLayout->addWidget(mOffsetUnitWidget, 6, 3, 1, 1);

        label = new QLabel(WidgetTemplatedLine);
        label->setObjectName("label");

        gridLayout->addWidget(label, 1, 0, 1, 2);

        label_6 = new QLabel(WidgetTemplatedLine);
        label_6->setObjectName("label_6");

        gridLayout->addWidget(label_6, 5, 1, 1, 1);

        mHashLengthLabel = new QLabel(WidgetTemplatedLine);
        mHashLengthLabel->setObjectName("mHashLengthLabel");

        gridLayout->addWidget(mHashLengthLabel, 2, 0, 1, 2);

        mHashRotationDDBtn = new QgsPropertyOverrideButton(WidgetTemplatedLine);
        mHashRotationDDBtn->setObjectName("mHashRotationDDBtn");

        gridLayout->addWidget(mHashRotationDDBtn, 3, 4, 1, 1);

        mHashLengthUnitWidget = new QgsUnitSelectionWidget(WidgetTemplatedLine);
        mHashLengthUnitWidget->setObjectName("mHashLengthUnitWidget");
        mHashLengthUnitWidget->setMinimumSize(QSize(20, 0));
        mHashLengthUnitWidget->setFocusPolicy(Qt::FocusPolicy::TabFocus);

        gridLayout->addWidget(mHashLengthUnitWidget, 2, 3, 1, 1);

        mOffsetAlongLineUnitWidget = new QgsUnitSelectionWidget(WidgetTemplatedLine);
        mOffsetAlongLineUnitWidget->setObjectName("mOffsetAlongLineUnitWidget");
        mOffsetAlongLineUnitWidget->setMinimumSize(QSize(0, 0));
        mOffsetAlongLineUnitWidget->setFocusPolicy(Qt::FocusPolicy::TabFocus);

        gridLayout->addWidget(mOffsetAlongLineUnitWidget, 1, 3, 1, 1);

        chkRotateMarker = new QCheckBox(WidgetTemplatedLine);
        chkRotateMarker->setObjectName("chkRotateMarker");

        gridLayout->addWidget(chkRotateMarker, 4, 0, 1, 5);

        mAverageAngleUnit = new QgsUnitSelectionWidget(WidgetTemplatedLine);
        mAverageAngleUnit->setObjectName("mAverageAngleUnit");
        mAverageAngleUnit->setMinimumSize(QSize(20, 0));
        mAverageAngleUnit->setFocusPolicy(Qt::FocusPolicy::TabFocus);

        gridLayout->addWidget(mAverageAngleUnit, 5, 3, 1, 1);

        label_3 = new QLabel(WidgetTemplatedLine);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 6, 0, 1, 2);

        mOffsetAlongLineDDBtn = new QgsPropertyOverrideButton(WidgetTemplatedLine);
        mOffsetAlongLineDDBtn->setObjectName("mOffsetAlongLineDDBtn");

        gridLayout->addWidget(mOffsetAlongLineDDBtn, 1, 4, 1, 1);

        mSpinHashLength = new QgsDoubleSpinBox(WidgetTemplatedLine);
        mSpinHashLength->setObjectName("mSpinHashLength");
        sizePolicy.setHeightForWidth(mSpinHashLength->sizePolicy().hasHeightForWidth());
        mSpinHashLength->setSizePolicy(sizePolicy);
        mSpinHashLength->setDecimals(6);
        mSpinHashLength->setMaximum(10000000.000000000000000);
        mSpinHashLength->setSingleStep(0.200000000000000);
        mSpinHashLength->setValue(1.000000000000000);

        gridLayout->addWidget(mSpinHashLength, 2, 2, 1, 1);

        mRingsLabel = new QLabel(WidgetTemplatedLine);
        mRingsLabel->setObjectName("mRingsLabel");

        gridLayout->addWidget(mRingsLabel, 7, 0, 1, 2);

        mSpinOffsetAlongLine = new QgsDoubleSpinBox(WidgetTemplatedLine);
        mSpinOffsetAlongLine->setObjectName("mSpinOffsetAlongLine");
        sizePolicy.setHeightForWidth(mSpinOffsetAlongLine->sizePolicy().hasHeightForWidth());
        mSpinOffsetAlongLine->setSizePolicy(sizePolicy);
        mSpinOffsetAlongLine->setDecimals(6);
        mSpinOffsetAlongLine->setMinimum(-10000000.000000000000000);
        mSpinOffsetAlongLine->setMaximum(10000000.000000000000000);
        mSpinOffsetAlongLine->setSingleStep(0.200000000000000);
        mSpinOffsetAlongLine->setValue(1.000000000000000);

        gridLayout->addWidget(mSpinOffsetAlongLine, 1, 2, 1, 1);

        mRingFilterComboBox = new QComboBox(WidgetTemplatedLine);
        mRingFilterComboBox->setObjectName("mRingFilterComboBox");

        gridLayout->addWidget(mRingFilterComboBox, 7, 2, 1, 3);

        mAverageAngleDDBtn = new QgsPropertyOverrideButton(WidgetTemplatedLine);
        mAverageAngleDDBtn->setObjectName("mAverageAngleDDBtn");

        gridLayout->addWidget(mAverageAngleDDBtn, 5, 4, 1, 1);

        mLineOffsetDDBtn = new QgsPropertyOverrideButton(WidgetTemplatedLine);
        mLineOffsetDDBtn->setObjectName("mLineOffsetDDBtn");

        gridLayout->addWidget(mLineOffsetDDBtn, 6, 4, 1, 1);

        mCheckPlaceOnEveryPart = new QCheckBox(WidgetTemplatedLine);
        mCheckPlaceOnEveryPart->setObjectName("mCheckPlaceOnEveryPart");

        gridLayout->addWidget(mCheckPlaceOnEveryPart, 0, 0, 1, 5);


        gridLayout_2->addLayout(gridLayout, 2, 0, 1, 3);

        gridLayout_3 = new QGridLayout();
        gridLayout_3->setObjectName("gridLayout_3");
        gridLayout_3->setContentsMargins(0, 0, -1, 0);
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        mCheckInterval = new QCheckBox(WidgetTemplatedLine);
        mCheckInterval->setObjectName("mCheckInterval");
        mCheckInterval->setChecked(true);

        horizontalLayout_4->addWidget(mCheckInterval);

        spinInterval = new QgsDoubleSpinBox(WidgetTemplatedLine);
        spinInterval->setObjectName("spinInterval");
        sizePolicy.setHeightForWidth(spinInterval->sizePolicy().hasHeightForWidth());
        spinInterval->setSizePolicy(sizePolicy);
        spinInterval->setDecimals(6);
        spinInterval->setMaximum(10000000.000000000000000);
        spinInterval->setSingleStep(0.200000000000000);
        spinInterval->setValue(1.000000000000000);
        spinInterval->setProperty("showClearButton", QVariant(false));

        horizontalLayout_4->addWidget(spinInterval);

        mIntervalUnitWidget = new QgsUnitSelectionWidget(WidgetTemplatedLine);
        mIntervalUnitWidget->setObjectName("mIntervalUnitWidget");
        mIntervalUnitWidget->setMinimumSize(QSize(0, 0));
        mIntervalUnitWidget->setFocusPolicy(Qt::FocusPolicy::TabFocus);

        horizontalLayout_4->addWidget(mIntervalUnitWidget);


        gridLayout_3->addLayout(horizontalLayout_4, 0, 1, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(10, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_2, 3, 0, 1, 1);

        mIntervalDDBtn = new QgsPropertyOverrideButton(WidgetTemplatedLine);
        mIntervalDDBtn->setObjectName("mIntervalDDBtn");

        gridLayout_3->addWidget(mIntervalDDBtn, 0, 2, 1, 1);

        mCheckVertex = new QCheckBox(WidgetTemplatedLine);
        mCheckVertex->setObjectName("mCheckVertex");

        gridLayout_3->addWidget(mCheckVertex, 3, 1, 1, 1);

        mCheckVertexLast = new QCheckBox(WidgetTemplatedLine);
        mCheckVertexLast->setObjectName("mCheckVertexLast");

        gridLayout_3->addWidget(mCheckVertexLast, 2, 1, 1, 1);

        mCheckVertexFirst = new QCheckBox(WidgetTemplatedLine);
        mCheckVertexFirst->setObjectName("mCheckVertexFirst");

        gridLayout_3->addWidget(mCheckVertexFirst, 1, 1, 1, 1);

        mCheckCentralPoint = new QCheckBox(WidgetTemplatedLine);
        mCheckCentralPoint->setObjectName("mCheckCentralPoint");

        gridLayout_3->addWidget(mCheckCentralPoint, 4, 1, 1, 1);

        mCheckSegmentCentralPoint = new QCheckBox(WidgetTemplatedLine);
        mCheckSegmentCentralPoint->setObjectName("mCheckSegmentCentralPoint");

        gridLayout_3->addWidget(mCheckSegmentCentralPoint, 5, 1, 1, 1);

        mCheckCurvePoint = new QCheckBox(WidgetTemplatedLine);
        mCheckCurvePoint->setObjectName("mCheckCurvePoint");

        gridLayout_3->addWidget(mCheckCurvePoint, 6, 1, 1, 1);


        gridLayout_2->addLayout(gridLayout_3, 1, 0, 1, 3);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer, 6, 0, 1, 1);

        mPerFeatureCustomizationGrpBox = new QgsCollapsibleGroupBox(WidgetTemplatedLine);
        mPerFeatureCustomizationGrpBox->setObjectName("mPerFeatureCustomizationGrpBox");
        mPerFeatureCustomizationGrpBox->setProperty("collapsed", QVariant(true));
        verticalLayout = new QVBoxLayout(mPerFeatureCustomizationGrpBox);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        label_2 = new QLabel(mPerFeatureCustomizationGrpBox);
        label_2->setObjectName("label_2");

        horizontalLayout_9->addWidget(label_2);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer_4);

        mEditBlankSegmentsBtn = new QToolButton(mPerFeatureCustomizationGrpBox);
        mEditBlankSegmentsBtn->setObjectName("mEditBlankSegmentsBtn");
        mEditBlankSegmentsBtn->setIcon(icon);
        mEditBlankSegmentsBtn->setCheckable(true);

        horizontalLayout_9->addWidget(mEditBlankSegmentsBtn);

        mBlankSegmentsUnitWidget = new QgsUnitSelectionWidget(mPerFeatureCustomizationGrpBox);
        mBlankSegmentsUnitWidget->setObjectName("mBlankSegmentsUnitWidget");
        mBlankSegmentsUnitWidget->setMinimumSize(QSize(20, 0));
        mBlankSegmentsUnitWidget->setFocusPolicy(Qt::FocusPolicy::TabFocus);

        horizontalLayout_9->addWidget(mBlankSegmentsUnitWidget);

        mBlankSegmentsDDButton = new QgsPropertyOverrideButton(mPerFeatureCustomizationGrpBox);
        mBlankSegmentsDDButton->setObjectName("mBlankSegmentsDDButton");

        horizontalLayout_9->addWidget(mBlankSegmentsDDButton);


        verticalLayout->addLayout(horizontalLayout_9);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        lblExtraItems = new QLabel(mPerFeatureCustomizationGrpBox);
        lblExtraItems->setObjectName("lblExtraItems");

        horizontalLayout->addWidget(lblExtraItems);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_5);

        mAddExtraItemBtn = new QToolButton(mPerFeatureCustomizationGrpBox);
        mAddExtraItemBtn->setObjectName("mAddExtraItemBtn");
        mAddExtraItemBtn->setIcon(icon1);
        mAddExtraItemBtn->setCheckable(true);

        horizontalLayout->addWidget(mAddExtraItemBtn);

        mModifyExtraItemBtn = new QToolButton(mPerFeatureCustomizationGrpBox);
        mModifyExtraItemBtn->setObjectName("mModifyExtraItemBtn");
        mModifyExtraItemBtn->setIcon(icon2);
        mModifyExtraItemBtn->setCheckable(true);

        horizontalLayout->addWidget(mModifyExtraItemBtn);

        mExtraItemsDDBtn = new QgsPropertyOverrideButton(mPerFeatureCustomizationGrpBox);
        mExtraItemsDDBtn->setObjectName("mExtraItemsDDBtn");

        horizontalLayout->addWidget(mExtraItemsDDBtn);


        verticalLayout->addLayout(horizontalLayout);


        gridLayout_2->addWidget(mPerFeatureCustomizationGrpBox, 4, 0, 1, 3);

        mPlacementDDBtn = new QgsPropertyOverrideButton(WidgetTemplatedLine);
        mPlacementDDBtn->setObjectName("mPlacementDDBtn");

        gridLayout_2->addWidget(mPlacementDDBtn, 0, 1, 1, 1);

        mPlacementLabel = new QLabel(WidgetTemplatedLine);
        mPlacementLabel->setObjectName("mPlacementLabel");

        gridLayout_2->addWidget(mPlacementLabel, 0, 0, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer, 0, 2, 1, 1);

        mTrimGroupBox = new QgsCollapsibleGroupBoxBasic(WidgetTemplatedLine);
        mTrimGroupBox->setObjectName("mTrimGroupBox");
        gridLayout_5 = new QGridLayout(mTrimGroupBox);
        gridLayout_5->setObjectName("gridLayout_5");
        mTrimDistanceEndDDBtn = new QgsPropertyOverrideButton(mTrimGroupBox);
        mTrimDistanceEndDDBtn->setObjectName("mTrimDistanceEndDDBtn");

        gridLayout_5->addWidget(mTrimDistanceEndDDBtn, 1, 3, 1, 1);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        mTrimStartDistanceSpin = new QgsDoubleSpinBox(mTrimGroupBox);
        mTrimStartDistanceSpin->setObjectName("mTrimStartDistanceSpin");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(mTrimStartDistanceSpin->sizePolicy().hasHeightForWidth());
        mTrimStartDistanceSpin->setSizePolicy(sizePolicy1);
        mTrimStartDistanceSpin->setDecimals(6);
        mTrimStartDistanceSpin->setMaximum(100000.000000000000000);
        mTrimStartDistanceSpin->setSingleStep(0.200000000000000);
        mTrimStartDistanceSpin->setValue(0.000000000000000);
        mTrimStartDistanceSpin->setProperty("showClearButton", QVariant(true));

        horizontalLayout_7->addWidget(mTrimStartDistanceSpin);

        mTrimDistanceStartUnitWidget = new QgsUnitSelectionWidget(mTrimGroupBox);
        mTrimDistanceStartUnitWidget->setObjectName("mTrimDistanceStartUnitWidget");
        mTrimDistanceStartUnitWidget->setMinimumSize(QSize(0, 0));
        mTrimDistanceStartUnitWidget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        horizontalLayout_7->addWidget(mTrimDistanceStartUnitWidget);


        gridLayout_5->addLayout(horizontalLayout_7, 0, 1, 1, 2);

        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        mTrimDistanceEndSpin = new QgsDoubleSpinBox(mTrimGroupBox);
        mTrimDistanceEndSpin->setObjectName("mTrimDistanceEndSpin");
        sizePolicy1.setHeightForWidth(mTrimDistanceEndSpin->sizePolicy().hasHeightForWidth());
        mTrimDistanceEndSpin->setSizePolicy(sizePolicy1);
        mTrimDistanceEndSpin->setDecimals(6);
        mTrimDistanceEndSpin->setMaximum(100000.000000000000000);
        mTrimDistanceEndSpin->setSingleStep(0.200000000000000);
        mTrimDistanceEndSpin->setValue(0.000000000000000);
        mTrimDistanceEndSpin->setProperty("showClearButton", QVariant(true));

        horizontalLayout_8->addWidget(mTrimDistanceEndSpin);

        mTrimDistanceEndUnitWidget = new QgsUnitSelectionWidget(mTrimGroupBox);
        mTrimDistanceEndUnitWidget->setObjectName("mTrimDistanceEndUnitWidget");
        mTrimDistanceEndUnitWidget->setMinimumSize(QSize(0, 0));
        mTrimDistanceEndUnitWidget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);

        horizontalLayout_8->addWidget(mTrimDistanceEndUnitWidget);


        gridLayout_5->addLayout(horizontalLayout_8, 1, 1, 1, 2);

        label_10 = new QLabel(mTrimGroupBox);
        label_10->setObjectName("label_10");

        gridLayout_5->addWidget(label_10, 0, 0, 1, 1);

        label_11 = new QLabel(mTrimGroupBox);
        label_11->setObjectName("label_11");

        gridLayout_5->addWidget(label_11, 1, 0, 1, 1);

        mTrimDistanceStartDDBtn = new QgsPropertyOverrideButton(mTrimGroupBox);
        mTrimDistanceStartDDBtn->setObjectName("mTrimDistanceStartDDBtn");

        gridLayout_5->addWidget(mTrimDistanceStartDDBtn, 0, 3, 1, 1);

        gridLayout_5->setColumnStretch(1, 1);

        gridLayout_2->addWidget(mTrimGroupBox, 3, 0, 1, 3);

        QWidget::setTabOrder(mPlacementDDBtn, mCheckInterval);
        QWidget::setTabOrder(mCheckInterval, spinInterval);
        QWidget::setTabOrder(spinInterval, mIntervalUnitWidget);
        QWidget::setTabOrder(mIntervalUnitWidget, mIntervalDDBtn);
        QWidget::setTabOrder(mIntervalDDBtn, mCheckVertex);
        QWidget::setTabOrder(mCheckVertex, mCheckVertexLast);
        QWidget::setTabOrder(mCheckVertexLast, mCheckVertexFirst);
        QWidget::setTabOrder(mCheckVertexFirst, mCheckCentralPoint);
        QWidget::setTabOrder(mCheckCentralPoint, mCheckSegmentCentralPoint);
        QWidget::setTabOrder(mCheckSegmentCentralPoint, mCheckCurvePoint);
        QWidget::setTabOrder(mCheckCurvePoint, mCheckPlaceOnEveryPart);
        QWidget::setTabOrder(mCheckPlaceOnEveryPart, mSpinOffsetAlongLine);
        QWidget::setTabOrder(mSpinOffsetAlongLine, mOffsetAlongLineUnitWidget);
        QWidget::setTabOrder(mOffsetAlongLineUnitWidget, mOffsetAlongLineDDBtn);
        QWidget::setTabOrder(mOffsetAlongLineDDBtn, mSpinHashLength);
        QWidget::setTabOrder(mSpinHashLength, mHashLengthUnitWidget);
        QWidget::setTabOrder(mHashLengthUnitWidget, mHashLengthDDBtn);
        QWidget::setTabOrder(mHashLengthDDBtn, mHashRotationSpinBox);
        QWidget::setTabOrder(mHashRotationSpinBox, mHashRotationDDBtn);
        QWidget::setTabOrder(mHashRotationDDBtn, chkRotateMarker);
        QWidget::setTabOrder(chkRotateMarker, mSpinAverageAngleLength);
        QWidget::setTabOrder(mSpinAverageAngleLength, mAverageAngleUnit);
        QWidget::setTabOrder(mAverageAngleUnit, mAverageAngleDDBtn);
        QWidget::setTabOrder(mAverageAngleDDBtn, spinOffset);
        QWidget::setTabOrder(spinOffset, mOffsetUnitWidget);
        QWidget::setTabOrder(mOffsetUnitWidget, mLineOffsetDDBtn);
        QWidget::setTabOrder(mLineOffsetDDBtn, mRingFilterComboBox);

        retranslateUi(WidgetTemplatedLine);

        QMetaObject::connectSlotsByName(WidgetTemplatedLine);
    } // setupUi

    void retranslateUi(QWidget *WidgetTemplatedLine)
    {
        mEditBlankSegmentsAction->setText(QCoreApplication::translate("WidgetTemplatedLine", "Edit blank segments", nullptr));
        mAddExtraItemAction->setText(QCoreApplication::translate("WidgetTemplatedLine", "Add an extra item", nullptr));
        mModifyExtraItemAction->setText(QCoreApplication::translate("WidgetTemplatedLine", "Select and modify (move, rotate, delete) extra items", nullptr));
        mHashRotationSpinBox->setSuffix(QCoreApplication::translate("WidgetTemplatedLine", " \302\260", nullptr));
        mHashLengthDDBtn->setText(QCoreApplication::translate("WidgetTemplatedLine", "\342\200\246", nullptr));
        mHashRotationLabel->setText(QCoreApplication::translate("WidgetTemplatedLine", "Hash rotation", nullptr));
        label->setText(QCoreApplication::translate("WidgetTemplatedLine", "Offset along line", nullptr));
        label_6->setText(QCoreApplication::translate("WidgetTemplatedLine", "Average angle over", nullptr));
        mHashLengthLabel->setText(QCoreApplication::translate("WidgetTemplatedLine", "Hash length", nullptr));
        mHashRotationDDBtn->setText(QCoreApplication::translate("WidgetTemplatedLine", "\342\200\246", nullptr));
        chkRotateMarker->setText(QCoreApplication::translate("WidgetTemplatedLine", "Rotate hash to follow line direction", nullptr));
        label_3->setText(QCoreApplication::translate("WidgetTemplatedLine", "Line offset", nullptr));
        mOffsetAlongLineDDBtn->setText(QCoreApplication::translate("WidgetTemplatedLine", "\342\200\246", nullptr));
        mRingsLabel->setText(QCoreApplication::translate("WidgetTemplatedLine", "Rings", nullptr));
        mAverageAngleDDBtn->setText(QCoreApplication::translate("WidgetTemplatedLine", "\342\200\246", nullptr));
        mLineOffsetDDBtn->setText(QCoreApplication::translate("WidgetTemplatedLine", "\342\200\246", nullptr));
#if QT_CONFIG(tooltip)
        mCheckPlaceOnEveryPart->setToolTip(QCoreApplication::translate("WidgetTemplatedLine", "If checked, the \"first vertex\" or \"last vertex\" placements will also render at the first and last vertex for every part of multipart geometries", nullptr));
#endif // QT_CONFIG(tooltip)
        mCheckPlaceOnEveryPart->setText(QCoreApplication::translate("WidgetTemplatedLine", "Place on every part extremity", nullptr));
        mCheckInterval->setText(QCoreApplication::translate("WidgetTemplatedLine", "With interval", nullptr));
        mIntervalDDBtn->setText(QCoreApplication::translate("WidgetTemplatedLine", "\342\200\246", nullptr));
#if QT_CONFIG(tooltip)
        mCheckVertex->setToolTip(QCoreApplication::translate("WidgetTemplatedLine", "Shows symbols on inner vertices, i.e. all vertices except the first or last", nullptr));
#endif // QT_CONFIG(tooltip)
        mCheckVertex->setText(QCoreApplication::translate("WidgetTemplatedLine", "On inner vertices", nullptr));
        mCheckVertexLast->setText(QCoreApplication::translate("WidgetTemplatedLine", "On last vertex", nullptr));
        mCheckVertexFirst->setText(QCoreApplication::translate("WidgetTemplatedLine", "On first vertex", nullptr));
        mCheckCentralPoint->setText(QCoreApplication::translate("WidgetTemplatedLine", "On central point", nullptr));
        mCheckSegmentCentralPoint->setText(QCoreApplication::translate("WidgetTemplatedLine", "On central point of segments", nullptr));
        mCheckCurvePoint->setText(QCoreApplication::translate("WidgetTemplatedLine", "On every curve point", nullptr));
        mPerFeatureCustomizationGrpBox->setTitle(QCoreApplication::translate("WidgetTemplatedLine", "Per feature customization", nullptr));
        label_2->setText(QCoreApplication::translate("WidgetTemplatedLine", "Blank Segments", nullptr));
        mEditBlankSegmentsBtn->setText(QString());
        mBlankSegmentsDDButton->setText(QCoreApplication::translate("WidgetTemplatedLine", "\342\200\246", nullptr));
        lblExtraItems->setText(QCoreApplication::translate("WidgetTemplatedLine", "Extra items", nullptr));
        mAddExtraItemBtn->setText(QCoreApplication::translate("WidgetTemplatedLine", "...", nullptr));
        mModifyExtraItemBtn->setText(QCoreApplication::translate("WidgetTemplatedLine", "...", nullptr));
        mPlacementDDBtn->setText(QCoreApplication::translate("WidgetTemplatedLine", "\342\200\246", nullptr));
        mPlacementLabel->setText(QCoreApplication::translate("WidgetTemplatedLine", "Hash placement", nullptr));
        mTrimGroupBox->setTitle(QCoreApplication::translate("WidgetTemplatedLine", "Trim Lines", nullptr));
        mTrimDistanceEndDDBtn->setText(QCoreApplication::translate("WidgetTemplatedLine", "\342\200\246", nullptr));
#if QT_CONFIG(tooltip)
        mTrimStartDistanceSpin->setToolTip(QCoreApplication::translate("WidgetTemplatedLine", "Allows for the line rendering to trim off the start from the actual line string when drawing the line", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        mTrimDistanceEndSpin->setToolTip(QCoreApplication::translate("WidgetTemplatedLine", "Allows for the line rendering to trim off the end from the actual line string when drawing the line", nullptr));
#endif // QT_CONFIG(tooltip)
        label_10->setText(QCoreApplication::translate("WidgetTemplatedLine", "Start", nullptr));
        label_11->setText(QCoreApplication::translate("WidgetTemplatedLine", "End", nullptr));
        mTrimDistanceStartDDBtn->setText(QCoreApplication::translate("WidgetTemplatedLine", "\342\200\246", nullptr));
        (void)WidgetTemplatedLine;
    } // retranslateUi

};

namespace Ui {
    class WidgetTemplatedLine: public Ui_WidgetTemplatedLine {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_WIDGET_TEMPLATEDLINE_H
