/********************************************************************************
** Form generated from reading UI file 'qgslayoutmapclippingwidgetbase.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QGSLAYOUTMAPCLIPPINGWIDGETBASE_H
#define UI_QGSLAYOUTMAPCLIPPINGWIDGETBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "qgscollapsiblegroupbox.h"
#include "qgslayoutitemcombobox.h"

QT_BEGIN_NAMESPACE

class Ui_QgsLayoutMapClippingWidgetBase
{
public:
    QVBoxLayout *verticalLayout;
    QgsCollapsibleGroupBox *mClipToAtlasCheckBox;
    QGridLayout *gridLayout;
    QTreeView *mLayersTreeView;
    QComboBox *mAtlasClippingTypeComboBox;
    QLabel *label;
    QLabel *mClipToAtlasLabel;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *mSelectAllButton;
    QPushButton *mDeselectAllButton;
    QPushButton *mInvertSelectionButton;
    QRadioButton *mRadioClipSelectedLayers;
    QRadioButton *mRadioClipAllLayers;
    QCheckBox *mForceLabelsInsideCheckBox;
    QCheckBox *mClipFrameCheckBox;
    QgsCollapsibleGroupBox *mClipToItemCheckBox;
    QGridLayout *gridLayout_2;
    QLabel *label_2;
    QLabel *mClipToItemLabel;
    QComboBox *mItemClippingTypeComboBox;
    QCheckBox *mForceLabelsInsideItemCheckBox;
    QgsLayoutItemComboBox *mClipItemComboBox;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *QgsLayoutMapClippingWidgetBase)
    {
        if (QgsLayoutMapClippingWidgetBase->objectName().isEmpty())
            QgsLayoutMapClippingWidgetBase->setObjectName("QgsLayoutMapClippingWidgetBase");
        QgsLayoutMapClippingWidgetBase->resize(318, 619);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(QgsLayoutMapClippingWidgetBase->sizePolicy().hasHeightForWidth());
        QgsLayoutMapClippingWidgetBase->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(QgsLayoutMapClippingWidgetBase);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        mClipToAtlasCheckBox = new QgsCollapsibleGroupBox(QgsLayoutMapClippingWidgetBase);
        mClipToAtlasCheckBox->setObjectName("mClipToAtlasCheckBox");
        mClipToAtlasCheckBox->setCheckable(true);
        gridLayout = new QGridLayout(mClipToAtlasCheckBox);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName("gridLayout");
        mLayersTreeView = new QTreeView(mClipToAtlasCheckBox);
        mLayersTreeView->setObjectName("mLayersTreeView");
        mLayersTreeView->setHeaderHidden(true);
        mLayersTreeView->header()->setDefaultSectionSize(26);

        gridLayout->addWidget(mLayersTreeView, 7, 0, 1, 1);

        mAtlasClippingTypeComboBox = new QComboBox(mClipToAtlasCheckBox);
        mAtlasClippingTypeComboBox->setObjectName("mAtlasClippingTypeComboBox");

        gridLayout->addWidget(mAtlasClippingTypeComboBox, 2, 0, 1, 1);

        label = new QLabel(mClipToAtlasCheckBox);
        label->setObjectName("label");
        label->setWordWrap(true);

        gridLayout->addWidget(label, 1, 0, 1, 1);

        mClipToAtlasLabel = new QLabel(mClipToAtlasCheckBox);
        mClipToAtlasLabel->setObjectName("mClipToAtlasLabel");
        mClipToAtlasLabel->setWordWrap(true);

        gridLayout->addWidget(mClipToAtlasLabel, 0, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        mSelectAllButton = new QPushButton(mClipToAtlasCheckBox);
        mSelectAllButton->setObjectName("mSelectAllButton");

        horizontalLayout->addWidget(mSelectAllButton);

        mDeselectAllButton = new QPushButton(mClipToAtlasCheckBox);
        mDeselectAllButton->setObjectName("mDeselectAllButton");

        horizontalLayout->addWidget(mDeselectAllButton);

        mInvertSelectionButton = new QPushButton(mClipToAtlasCheckBox);
        mInvertSelectionButton->setObjectName("mInvertSelectionButton");

        horizontalLayout->addWidget(mInvertSelectionButton);


        gridLayout->addLayout(horizontalLayout, 8, 0, 1, 1);

        mRadioClipSelectedLayers = new QRadioButton(mClipToAtlasCheckBox);
        mRadioClipSelectedLayers->setObjectName("mRadioClipSelectedLayers");

        gridLayout->addWidget(mRadioClipSelectedLayers, 6, 0, 1, 1);

        mRadioClipAllLayers = new QRadioButton(mClipToAtlasCheckBox);
        mRadioClipAllLayers->setObjectName("mRadioClipAllLayers");

        gridLayout->addWidget(mRadioClipAllLayers, 5, 0, 1, 1);

        mForceLabelsInsideCheckBox = new QCheckBox(mClipToAtlasCheckBox);
        mForceLabelsInsideCheckBox->setObjectName("mForceLabelsInsideCheckBox");

        gridLayout->addWidget(mForceLabelsInsideCheckBox, 3, 0, 1, 1);

        mClipFrameCheckBox = new QCheckBox(mClipToAtlasCheckBox);
        mClipFrameCheckBox->setObjectName("mClipFrameCheckBox");

        gridLayout->addWidget(mClipFrameCheckBox, 4, 0, 1, 1);


        verticalLayout->addWidget(mClipToAtlasCheckBox);

        mClipToItemCheckBox = new QgsCollapsibleGroupBox(QgsLayoutMapClippingWidgetBase);
        mClipToItemCheckBox->setObjectName("mClipToItemCheckBox");
        mClipToItemCheckBox->setCheckable(true);
        gridLayout_2 = new QGridLayout(mClipToItemCheckBox);
        gridLayout_2->setSpacing(6);
        gridLayout_2->setContentsMargins(11, 11, 11, 11);
        gridLayout_2->setObjectName("gridLayout_2");
        label_2 = new QLabel(mClipToItemCheckBox);
        label_2->setObjectName("label_2");
        label_2->setWordWrap(true);

        gridLayout_2->addWidget(label_2, 2, 0, 1, 1);

        mClipToItemLabel = new QLabel(mClipToItemCheckBox);
        mClipToItemLabel->setObjectName("mClipToItemLabel");
        mClipToItemLabel->setWordWrap(true);

        gridLayout_2->addWidget(mClipToItemLabel, 0, 0, 1, 1);

        mItemClippingTypeComboBox = new QComboBox(mClipToItemCheckBox);
        mItemClippingTypeComboBox->setObjectName("mItemClippingTypeComboBox");

        gridLayout_2->addWidget(mItemClippingTypeComboBox, 3, 0, 1, 1);

        mForceLabelsInsideItemCheckBox = new QCheckBox(mClipToItemCheckBox);
        mForceLabelsInsideItemCheckBox->setObjectName("mForceLabelsInsideItemCheckBox");

        gridLayout_2->addWidget(mForceLabelsInsideItemCheckBox, 4, 0, 1, 1);

        mClipItemComboBox = new QgsLayoutItemComboBox(mClipToItemCheckBox);
        mClipItemComboBox->setObjectName("mClipItemComboBox");

        gridLayout_2->addWidget(mClipItemComboBox, 1, 0, 1, 1);


        verticalLayout->addWidget(mClipToItemCheckBox);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        QWidget::setTabOrder(mClipToAtlasCheckBox, mAtlasClippingTypeComboBox);
        QWidget::setTabOrder(mAtlasClippingTypeComboBox, mForceLabelsInsideCheckBox);
        QWidget::setTabOrder(mForceLabelsInsideCheckBox, mRadioClipAllLayers);
        QWidget::setTabOrder(mRadioClipAllLayers, mRadioClipSelectedLayers);
        QWidget::setTabOrder(mRadioClipSelectedLayers, mLayersTreeView);
        QWidget::setTabOrder(mLayersTreeView, mSelectAllButton);
        QWidget::setTabOrder(mSelectAllButton, mDeselectAllButton);
        QWidget::setTabOrder(mDeselectAllButton, mInvertSelectionButton);
        QWidget::setTabOrder(mInvertSelectionButton, mClipToItemCheckBox);
        QWidget::setTabOrder(mClipToItemCheckBox, mClipItemComboBox);
        QWidget::setTabOrder(mClipItemComboBox, mItemClippingTypeComboBox);
        QWidget::setTabOrder(mItemClippingTypeComboBox, mForceLabelsInsideItemCheckBox);

        retranslateUi(QgsLayoutMapClippingWidgetBase);

        QMetaObject::connectSlotsByName(QgsLayoutMapClippingWidgetBase);
    } // setupUi

    void retranslateUi(QWidget *QgsLayoutMapClippingWidgetBase)
    {
        QgsLayoutMapClippingWidgetBase->setWindowTitle(QCoreApplication::translate("QgsLayoutMapClippingWidgetBase", "Map Options", nullptr));
        mClipToAtlasCheckBox->setTitle(QCoreApplication::translate("QgsLayoutMapClippingWidgetBase", "Clip to atlas feature", nullptr));
        label->setText(QCoreApplication::translate("QgsLayoutMapClippingWidgetBase", "The clipping mode determines how features from vector layers will be clipped.", nullptr));
        mClipToAtlasLabel->setText(QCoreApplication::translate("QgsLayoutMapClippingWidgetBase", "<b>When enabled, map layers will be automatically clipped to the boundary of the current atlas feature.</b>", nullptr));
        mSelectAllButton->setText(QCoreApplication::translate("QgsLayoutMapClippingWidgetBase", "Select All", nullptr));
        mDeselectAllButton->setText(QCoreApplication::translate("QgsLayoutMapClippingWidgetBase", "Deselect All", nullptr));
        mInvertSelectionButton->setText(QCoreApplication::translate("QgsLayoutMapClippingWidgetBase", "Invert Selection", nullptr));
        mRadioClipSelectedLayers->setText(QCoreApplication::translate("QgsLayoutMapClippingWidgetBase", "Clip selected layers", nullptr));
        mRadioClipAllLayers->setText(QCoreApplication::translate("QgsLayoutMapClippingWidgetBase", "Clip all layers", nullptr));
        mForceLabelsInsideCheckBox->setText(QCoreApplication::translate("QgsLayoutMapClippingWidgetBase", "Force labels inside atlas feature", nullptr));
        mClipFrameCheckBox->setText(QCoreApplication::translate("QgsLayoutMapClippingWidgetBase", "Clip frame to atlas feature", nullptr));
        mClipToItemCheckBox->setTitle(QCoreApplication::translate("QgsLayoutMapClippingWidgetBase", "Clip to item", nullptr));
        label_2->setText(QCoreApplication::translate("QgsLayoutMapClippingWidgetBase", "The clipping mode determines how features from vector layers will be clipped.", nullptr));
        mClipToItemLabel->setText(QCoreApplication::translate("QgsLayoutMapClippingWidgetBase", "<b>When enabled, the map will be automatically clipped to the selected shape.</b>", nullptr));
        mForceLabelsInsideItemCheckBox->setText(QCoreApplication::translate("QgsLayoutMapClippingWidgetBase", "Force labels inside clipping shape", nullptr));
    } // retranslateUi

};

namespace Ui {
    class QgsLayoutMapClippingWidgetBase: public Ui_QgsLayoutMapClippingWidgetBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QGSLAYOUTMAPCLIPPINGWIDGETBASE_H
